@@ -4,11 +4,14 @@
  *  See COPYRIGHT notice in top-level directory.
  *
  *********************************************************************/
-/* $Id: aggregation.c 2325 2016-02-28 07:49:13Z wkliao $ */
+/* $Id: aggregation.c 2728 2016-12-18 17:49:20Z wkliao $ */
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <assert.h>
+#include <string.h> /* strcpy(), strncpy() */
+#include <unistd.h> /* getopt() */
+#include <errno.h>
+
 #include <mpi.h>
 #include <pnetcdf.h>
 
@@ -107,10 +110,16 @@
  *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
+#ifndef MPI_OFFSET
+#define MPI_OFFSET MPI_LONG_LONG_INT
+#endif
+
 #define NVARS 5
 
 #define ERR(e) {if((e)!=NC_NOERR)printf("Error at line=%d: %s\n", __LINE__, ncmpi_strerror(e));}
 
+static int debug;
+
 /*----< print_info() >------------------------------------------------------*/
 static
 void print_info(MPI_Info *info_used)
@@ -138,7 +147,7 @@ int benchmark_write(char       *filename,
                     MPI_Info   *w_info_used,
                     double     *timing)  /* [6] */
 {
-    int i, j, k, verbose, rank, nprocs, err, num_reqs;
+    int i, j, k, rank, nprocs, err, num_reqs;
     int ncid, cmode, varid[NVARS], dimid[6], *reqs, *sts, psizes[2];
     void *buf[NVARS];
     double start_t, end_t;
@@ -146,7 +155,6 @@ int benchmark_write(char       *filename,
     MPI_Offset gsizes[2], start[2], count[2];
     MPI_Info info=MPI_INFO_NULL;
 
-    verbose = 0;
     MPI_Comm_rank(comm, &rank);
     MPI_Comm_size(comm, &nprocs);
 
@@ -158,27 +166,26 @@ int benchmark_write(char       *filename,
     /* disable the fixed-size variable alignments */
     MPI_Info_set(info, "nc_var_align_size", "1");
 
-    /* initialize I/O buffer with random numbers */
-    srand(rank);
+    /* initialize I/O buffer */
     for (i=0; i<NVARS; i++) {
         if (i % 4 == 0) {
             int *int_b = (int*) malloc(len * len * sizeof(int));
-            for (j=0; j<len*len; j++) int_b[j] = rank; /* rand(); */
+            for (j=0; j<len*len; j++) int_b[j] = rank;
             buf[i] = (void*)int_b;
         }
         else if (i % 4 == 1) {
             float *flt_b = (float*) malloc(len * len * sizeof(float));
-            for (j=0; j<len*len; j++) flt_b[j] = rank; /* rand(); */
+            for (j=0; j<len*len; j++) flt_b[j] = rank;
             buf[i] = (void*)flt_b;
         }
         else if (i % 4 == 2) {
             short *shr_b = (short*) malloc(len * len * sizeof(short));
-            for (j=0; j<len*len; j++) shr_b[j] = rank; /* rand(); */
+            for (j=0; j<len*len; j++) shr_b[j] = rank;
             buf[i] = (void*)shr_b;
         }
         else {
             double *dbl_b = (double*) malloc(len * len * sizeof(double));
-            for (j=0; j<len*len; j++) dbl_b[j] = rank; /* rand(); */
+            for (j=0; j<len*len; j++) dbl_b[j] = rank;
             buf[i] = (void*)dbl_b;
         }
     }
@@ -256,7 +263,7 @@ int benchmark_write(char       *filename,
             err = ncmpi_iput_vara_int(ncid, varid[i], start, count, int_b,
                                       &reqs[k++]);
             ERR(err)
-            if (verbose) printf("block-block %d: start=%lld %lld count=%lld %lld\n",i,start[0],start[1],count[0],count[1]);
+            if (debug) printf("block-block %d: start=%lld %lld count=%lld %lld\n",i,start[0],start[1],count[0],count[1]);
         }
         else if (i % 4 == 1) {
             float *flt_b = (float*) buf[i];
@@ -264,12 +271,12 @@ int benchmark_write(char       *filename,
             count[0] = len;
             count[1] = 1;
             for (j=0; j<len; j++) {
-                start[1] = rank + j * nprocs;
+                start[1] = rank + (MPI_Offset)j * nprocs;
                 err = ncmpi_iput_vara_float(ncid, varid[i], start, count,
                                             flt_b, &reqs[k++]);
                 ERR(err)
                 flt_b += len;
-                if (verbose) printf("*-cyclic i=%d j=%d: start=%lld %lld count=%lld %lld\n",i,j,start[0],start[1],count[0],count[1]);
+                if (debug) printf("*-cyclic i=%d j=%d: start=%lld %lld count=%lld %lld\n",i,j,start[0],start[1],count[0],count[1]);
             }
         }
         else if (i % 4 == 2) {
@@ -281,7 +288,7 @@ int benchmark_write(char       *filename,
             err = ncmpi_iput_vara_short(ncid, varid[i], start, count,
                                         shr_b, &reqs[k++]);
             ERR(err)
-            if (verbose) printf("block-* i=0 start=%lld %lld count=%lld %lld\n",start[0],start[1],count[0],count[1]);
+            if (debug) printf("block-* i=0 start=%lld %lld count=%lld %lld\n",start[0],start[1],count[0],count[1]);
 
             shr_b += len * (len/2);
             start[1] = len/2;
@@ -289,7 +296,7 @@ int benchmark_write(char       *filename,
             err = ncmpi_iput_vara_short(ncid, varid[i], start, count,
                                         shr_b, &reqs[k++]);
             ERR(err)
-            if (verbose) printf("block-* i=1 start=%lld %lld count=%lld %lld\n",start[0],start[1],count[0],count[1]);
+            if (debug) printf("block-* i=1 start=%lld %lld count=%lld %lld\n",start[0],start[1],count[0],count[1]);
         }
         else {
             double *dbl_b = (double*) buf[i];
@@ -300,7 +307,7 @@ int benchmark_write(char       *filename,
             err = ncmpi_iput_vara_double(ncid, varid[i], start, count, dbl_b,
                                          &reqs[k++]);
             ERR(err)
-            if (verbose) printf("*-block %d: start=%lld %lld count=%lld %lld\n",i,start[0],start[1],count[0],count[1]);
+            if (debug) printf("*-block %d: start=%lld %lld count=%lld %lld\n",i,start[0],start[1],count[0],count[1]);
         }
     }
     num_reqs = k;
@@ -352,7 +359,7 @@ int benchmark_read(char       *filename,
                    MPI_Info   *r_info_used,
                    double     *timing)  /* [5] */
 {
-    int i, j, k, verbose, rank, nprocs, s_rank, err, num_reqs;
+    int i, j, k, rank, nprocs, s_rank, err, num_reqs;
     int ncid, omode, varid[NVARS], *reqs, *sts, psizes[2];
     void *buf[NVARS];
     double start_t, end_t;
@@ -360,7 +367,6 @@ int benchmark_read(char       *filename,
     MPI_Offset start[2], count[2];
     MPI_Info info=MPI_INFO_NULL;
 
-    verbose = 0;
     MPI_Comm_rank(comm, &rank);
     MPI_Comm_size(comm, &nprocs);
     s_rank = (rank + nprocs / 2 ) % nprocs;
@@ -391,7 +397,7 @@ int benchmark_read(char       *filename,
     /* Note that PnetCDF read the file in chunks of size 256KB, thus the read
      * amount may be more than the file header size
      */
-    if (verbose) {
+    if (debug) {
         MPI_Offset h_size, h_extent;
         ncmpi_inq_header_size(ncid, &h_size);
         ncmpi_inq_header_extent(ncid, &h_extent);
@@ -430,7 +436,7 @@ int benchmark_read(char       *filename,
             count[0] = len;
             count[1] = 1;
             for (j=0; j<len; j++) {
-                start[1] = s_rank + j * nprocs;
+                start[1] = s_rank + (MPI_Offset)j * nprocs;
                 err = ncmpi_iget_vara_float(ncid, varid[i], start, count,
                                             flt_b, &reqs[k++]);
                 ERR(err)
@@ -506,9 +512,24 @@ int benchmark_read(char       *filename,
     return 1;
 }
 
+static void
+usage(char *argv0)
+{   
+    char *help =
+    "Usage: %s [-h | -q | -d] len file_name\n"
+    "       [-h] Print help\n"
+    "       [-q] Quiet mode\n"
+    "       [-d] Debug mode\n"
+    "       len: local variable of size len x len (default 10)\n"
+    "       filename: output netCDF file name (default ./testfile.nc)\n";
+    fprintf(stderr, help, argv0);
+}
+
 /*----< main() >--------------------------------------------------------------*/
 int main(int argc, char** argv) {
-    int rank, nprocs;
+    extern int optind;
+    char filename[256];
+    int i, rank, nprocs, verbose=1;
     double timing[11], max_t[11];
     MPI_Offset len, w_size=0, r_size=0, sum_w_size, sum_r_size;
     MPI_Comm comm=MPI_COMM_WORLD;
@@ -518,15 +539,31 @@ int main(int argc, char** argv) {
     MPI_Comm_rank(comm, &rank);
     MPI_Comm_size(comm, &nprocs);
 
-    if (argc != 3) {
-        if (!rank) printf("Usage: %s len filename\n",argv[0]);
-        MPI_Finalize();
-        return 0;
-    }
-    len = atoll(argv[1]);
+    /* get command-line arguments */
+    debug = 0;
+    while ((i = getopt(argc, argv, "hqd")) != EOF)
+        switch(i) {
+            case 'q': verbose = 0;
+                      break;
+            case 'd': debug = 1;
+                      break;
+            case 'h':
+            default:  if (rank==0) usage(argv[0]);
+                      MPI_Finalize();
+                      return 0;
+        }
+    argc -= optind;
+    argv += optind;
 
-    benchmark_write(argv[2], len, &w_size, &w_info_used, timing);
-    benchmark_read (argv[2], len, &r_size, &r_info_used, timing+6);
+    len = 10;
+    if (argc > 0) len = strtoll(argv[0],NULL,10);
+    len = (len <= 0) ? 10 : len;
+
+    if (argc > 1) snprintf(filename, 256, "%s", argv[1]);
+    else          strcpy(filename, "testfile.nc");
+
+    benchmark_write(filename, len, &w_size, &w_info_used, timing);
+    benchmark_read (filename, len, &r_size, &r_info_used, timing+6);
 
     MPI_Reduce(&timing, &max_t,     11, MPI_DOUBLE, MPI_MAX, 0, comm);
 #ifdef MPI_OFFSET
@@ -536,7 +573,7 @@ int main(int argc, char** argv) {
     MPI_Reduce(&w_size, &sum_w_size, 1, MPI_LONG_LONG, MPI_SUM, 0, comm);
     MPI_Reduce(&r_size, &sum_r_size, 1, MPI_LONG_LONG, MPI_SUM, 0, comm);
 #endif
-    if (rank == 0) {
+    if (verbose && rank == 0) {
         double bw = sum_w_size;
         bw /= 1048576.0;
         print_info(&w_info_used);
