@@ -4,7 +4,7 @@
  *  See COPYRIGHT notice in top-level directory.
  *
  *********************************************************************/
-/* $Id: bput_varn_uint.c 2325 2016-02-28 07:49:13Z wkliao $ */
+/* $Id: bput_varn_uint.c 2717 2016-12-18 01:20:47Z wkliao $ */
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  * This example tests nonblocking buffered write varn APIs, including
@@ -15,8 +15,8 @@
  * The compile and run commands are given below, together with an ncmpidump of
  * the output file.
  *
- *    % mpicc -O2 -o i_varn_uint i_varn_uint.c -lpnetcdf
- *    % mpiexec -n 4 ./i_varn_uint /pvfs2/wkliao/testfile.nc
+ *    % mpicc -O2 -o bput_varn_uint bput_varn_uint.c -lpnetcdf
+ *    % mpiexec -n 4 ./bput_varn_uint /pvfs2/wkliao/testfile.nc
  *    % ncmpidump /pvfs2/wkliao/testfile.nc
  *    netcdf testfile {
  *    // file format: CDF-5 (big variables)
@@ -58,19 +58,20 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h> /* strcpy(), strncpy() */
 #include <unistd.h> /* getopt() */
-#include <string.h> /* strcpy() */
 #include <mpi.h>
 #include <pnetcdf.h>
 
+#ifndef MPI_OFFSET
+#define MPI_OFFSET MPI_LONG_LONG_INT
+#endif
+
 #define NY 4
 #define NX 10
 #define NDIMS 2
 
-#define ERR \
-    if (err != NC_NOERR) { \
-        printf("Error at line=%d: %s\n", __LINE__, ncmpi_strerror(err)); \
-    }
+#define ERR {if(err!=NC_NOERR){printf("Error at line=%d: %s\n", __LINE__, ncmpi_strerror(err));nerrs++;}}
 
 #define ERRS(n,a) { \
     int _i; \
@@ -78,12 +79,14 @@
         if ((a)[_i] != NC_NOERR) { \
             printf("Error at line=%d: err[%d] %s\n", __LINE__, _i, \
                    ncmpi_strerror((a)[_i])); \
+                   nerrs++; \
         } \
     } \
 }
 
 static MPI_Offset *** calloc_3D(size_t z, size_t y, size_t x)
 {
+    if (z*y*x == 0) return NULL;
     int _j, _k;
     MPI_Offset ***buf  = (MPI_Offset***) malloc(z     * sizeof(MPI_Offset**));
     MPI_Offset  **bufy = (MPI_Offset**)  malloc(z*y   * sizeof(MPI_Offset*));
@@ -114,9 +117,9 @@ usage(char *argv0)
     fprintf(stderr, help, argv0);
 }
 
-static void check_contents(int ncid, int *varid)
+static int check_contents(int ncid, int *varid)
 {
-    int i, j, err;
+    int i, j, err, nerrs=0;
     unsigned int expected[4][NY*NX] = {{1, 1, 1, 3, 3, 0, 0, 2, 3, 3,
                                         0, 2, 2, 2, 1, 3, 3, 2, 2, 2,
                                         3, 3, 2, 1, 1, 1, 0, 0, 3, 3,
@@ -143,18 +146,21 @@ static void check_contents(int ncid, int *varid)
 
         /* check if the contents of buf are expected */
         for (j=0; j<NY*NX; j++)
-            if (r_buffer[j] != expected[i][j])
+            if (r_buffer[j] != expected[i][j]) {
                 printf("Expected file contents [%d][%d]=%u, but got %u\n",
                        i,j,expected[i][j],r_buffer[j]);
+                nerrs++;
+            }
     }
     free(r_buffer);
+    return nerrs;
 }
 
 int main(int argc, char** argv)
 {
     extern int optind;
-    char *filename="testfile.nc", exec[128];
-    int i, j, k, n, rank, nprocs, verbose=1, err;
+    char filename[256], *exec;
+    int i, j, k, n, rank, nprocs, verbose=1, err, nerrs=0;
     int ncid, cmode, varid[4], dimid[2], nreqs, reqs[4], sts[4];
     unsigned int *buffer[4];
     int num_segs[4], req_lens[4];
@@ -164,7 +170,7 @@ int main(int argc, char** argv)
     MPI_Init(&argc, &argv);
     MPI_Comm_rank(MPI_COMM_WORLD, &rank);
     MPI_Comm_size(MPI_COMM_WORLD, &nprocs);
-    strcpy(exec, argv[0]);
+    exec = argv[0];
 
     /* get command-line arguments */
     while ((i = getopt(argc, argv, "hq")) != EOF)
@@ -178,12 +184,13 @@ int main(int argc, char** argv)
         }
     argc -= optind;
     argv += optind;
-    if (argc == 1) filename = argv[0]; /* optional argument */
+    if (argc == 1) snprintf(filename, 256, "%s", argv[0]);
+    else           strcpy(filename, "testfile.nc");
 
     if (nprocs != 4 && rank == 0 && verbose)
         printf("Warning: %s is intended to run on 4 processes\n",exec);
 
-    /* set an MPI-IO hint to disable file offset alignment for fix-sized
+    /* set an MPI-IO hint to disable file offset alignment for fixed-size
      * variables */
     MPI_Info_create(&info);
     MPI_Info_set(info, "nc_var_align_size", "1");
@@ -350,6 +357,6 @@ int main(int argc, char** argv)
     }
 
     MPI_Finalize();
-    return 0;
+    return nerrs;
 }
 
