@@ -4,7 +4,7 @@
  *  See COPYRIGHT notice in top-level directory.
  *
  *********************************************************************/
-/* $Id: nonblocking_write.c 2325 2016-02-28 07:49:13Z wkliao $ */
+/* $Id: nonblocking_write.c 2717 2016-12-18 01:20:47Z wkliao $ */
 
 /*    This example is similar to collective_write.c but using nonblocking APIs.
  *    It creates a netcdf file in CD-5 format and writes a number of
@@ -34,19 +34,19 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <string.h> /* strcpy() */
+#include <string.h> /* strcpy(), strncpy() */
 #include <unistd.h> /* getopt() */
 #include <mpi.h>
 #include <pnetcdf.h>
 
+#ifndef MPI_OFFSET
+#define MPI_OFFSET MPI_LONG_LONG_INT
+#endif
+
 #define NDIMS    3
 #define NUM_VARS 10
 
-#define HANDLE_ERROR {                                \
-    if (err != NC_NOERR)                              \
-        printf("Error at line %d (%s)\n", __LINE__,   \
-               ncmpi_strerror(err));                  \
-}
+#define ERR {if(err!=NC_NOERR){printf("Error at line=%d: %s\n", __LINE__, ncmpi_strerror(err));nerrs++;}}
 
 static void
 usage(char *argv0)
@@ -88,11 +88,11 @@ void print_info(MPI_Info *info_used)
 int main(int argc, char **argv)
 {
     extern int optind;
-    int i, j, verbose=1, err;
+    int i, j, verbose=1, err, nerrs=0;
     int nprocs, len, *buf[NUM_VARS], bufsize, rank;
     int gsizes[NDIMS], psizes[NDIMS];
     double write_timing, max_write_timing, write_bw;
-    char *filename="testfile.nc", str[512];
+    char filename[256], str[512];
     int ncid, dimids[NDIMS], varids[NUM_VARS], req[NUM_VARS], st[NUM_VARS];
     MPI_Offset starts[NDIMS], counts[NDIMS], write_size, sum_write_size;
     MPI_Offset bbufsize;
@@ -114,9 +114,12 @@ int main(int argc, char **argv)
         }
     argc -= optind;
     argv += optind;
-    if (argc >= 1) filename = argv[0];  /* optional argument */
+    if (argc >= 1) snprintf(filename, 256, "%s", argv[0]);
+    else           strcpy(filename, "testfile.nc");
+
     len = 10; 
-    if (argc >= 2) len = atoi(argv[1]); /* optional argument */
+    if (argc >= 2) len = (int)strtol(argv[1],NULL,10); /* optional argument */
+    len = (len <= 0) ? 10 : len;
 
     for (i=0; i<NDIMS; i++) psizes[i] = 0;
 
@@ -133,17 +136,16 @@ int main(int argc, char **argv)
         bufsize   *= len;
     }
 
-    /* allocate buffer and initialize with random numbers */
-    srand(rank);
+    /* allocate buffer and initialize with some non-zero numbers */
     for (i=0; i<NUM_VARS; i++) {
         buf[i] = (int *) malloc(bufsize * sizeof(int));
-        for (j=0; j<bufsize; j++) buf[i][j] = rand();
+        for (j=0; j<bufsize; j++) buf[i][j] = rank * i + 123 + j;
     }
 
     MPI_Barrier(MPI_COMM_WORLD);
     write_timing = MPI_Wtime();
 
-    /* set an MPI-IO hint to disable file offset alignment for fix-sized
+    /* set an MPI-IO hint to disable file offset alignment for fixed-size
      * variables */
     MPI_Info_create(&info);
     MPI_Info_set(info, "nc_var_align_size", "1");
@@ -163,33 +165,33 @@ int main(int argc, char **argv)
     for (i=0; i<NDIMS; i++) {
         sprintf(str, "%c", 'x'+i);
         err = ncmpi_def_dim(ncid, str, gsizes[i], &dimids[i]);
-        HANDLE_ERROR
+        ERR
     }
 
     /* define variables */
     for (i=0; i<NUM_VARS; i++) {
         sprintf(str, "var%d", i);
         err = ncmpi_def_var(ncid, str, NC_INT, NDIMS, dimids, &varids[i]);
-        HANDLE_ERROR
+        ERR
     }
 
     /* exit the define mode */
     err = ncmpi_enddef(ncid);
-    HANDLE_ERROR
+    ERR
 
     /* get all the hints used */
     err = ncmpi_inq_file_info(ncid, &info_used);
-    HANDLE_ERROR
+    ERR
 
     /* write one variable at a time using iput */
     for (i=0; i<NUM_VARS; i++) {
         err = ncmpi_iput_vara_int(ncid, varids[i], starts, counts, buf[i], &req[i]);
-        HANDLE_ERROR
+        ERR
     }
 
     /* wait for the nonblocking I/O to complete */
     err = ncmpi_wait_all(ncid, NUM_VARS, req, st);
-    HANDLE_ERROR
+    ERR
     for (i=0; i<NUM_VARS; i++) {
         if (st[i] != NC_NOERR)
             printf("Error: nonblocking write fails on request %d (%s)\n",
@@ -201,17 +203,17 @@ int main(int argc, char **argv)
     /* bbufsize must be max of data type converted before and after */
     bbufsize = bufsize * NUM_VARS * sizeof(int);
     err = ncmpi_buffer_attach(ncid, bbufsize);
-    HANDLE_ERROR
+    ERR
 
     for (i=0; i<NUM_VARS; i++) {
         err = ncmpi_bput_vara_int(ncid, varids[i], starts, counts, buf[i], &req[i]);
-        HANDLE_ERROR
+        ERR
         /* can safely change contents or free up the buf[i] here */
     }
 
     /* wait for the nonblocking I/O to complete */
     err = ncmpi_wait_all(ncid, NUM_VARS, req, st);
-    HANDLE_ERROR
+    ERR
     for (i=0; i<NUM_VARS; i++) {
         if (st[i] != NC_NOERR)
             printf("Error: nonblocking write fails on request %d (%s)\n",
@@ -220,30 +222,22 @@ int main(int argc, char **argv)
 
     /* detach the temporary buffer */
     err = ncmpi_buffer_detach(ncid);
-    HANDLE_ERROR
+    ERR
 
     MPI_Offset put_size;
     ncmpi_inq_put_size(ncid, &put_size);
-#ifdef MPI_OFFSET
     MPI_Allreduce(MPI_IN_PLACE, &put_size, 1, MPI_OFFSET, MPI_SUM, MPI_COMM_WORLD);
-#else
-    MPI_Allreduce(MPI_IN_PLACE, &put_size, 1, MPI_LONG_LONG, MPI_SUM, MPI_COMM_WORLD);
-#endif
 
     /* close the file */
     err = ncmpi_close(ncid);
-    HANDLE_ERROR
+    ERR
 
     write_timing = MPI_Wtime() - write_timing;
 
     write_size = bufsize * NUM_VARS * sizeof(int);
     for (i=0; i<NUM_VARS; i++) free(buf[i]);
 
-#ifdef MPI_OFFSET
     MPI_Reduce(&write_size,   &sum_write_size,   1, MPI_OFFSET, MPI_SUM, 0, MPI_COMM_WORLD);
-#else
-    MPI_Reduce(&write_size,   &sum_write_size,   1, MPI_LONG_LONG, MPI_SUM, 0, MPI_COMM_WORLD);
-#endif
     MPI_Reduce(&write_timing, &max_write_timing, 1, MPI_DOUBLE, MPI_MAX, 0, MPI_COMM_WORLD);
 
     if (rank == 0 && verbose) {
@@ -277,6 +271,6 @@ int main(int argc, char **argv)
     }
 
     MPI_Finalize();
-    return 0;
+    return nerrs;
 }
 
