@@ -4,7 +4,7 @@
  *  See COPYRIGHT notice in top-level directory.
  *
  *********************************************************************/
-/* $Id: transpose.c 2325 2016-02-28 07:49:13Z wkliao $ */
+/* $Id: transpose.c 2717 2016-12-18 01:20:47Z wkliao $ */
 
 /*
  *    This example shows how to use varm API to write six 3D integer array
@@ -31,18 +31,18 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <string.h> /* strlen() */
+#include <string.h> /* strcpy(), strncpy() */
 #include <unistd.h> /* getopt() */
 #include <mpi.h>
 #include <pnetcdf.h>
 
+#ifndef MPI_OFFSET
+#define MPI_OFFSET MPI_LONG_LONG_INT
+#endif
+
 #define NDIMS 3
 
-#define HANDLE_ERROR {                                \
-    if (err != NC_NOERR)                              \
-        printf("Error at line %d (%s)\n", __LINE__,   \
-               ncmpi_strerror(err));                  \
-}
+#define ERR {if(err!=NC_NOERR){printf("Error at line=%d: %s\n", __LINE__, ncmpi_strerror(err));nerrs++;}}
 
 static void
 usage(char *argv0)
@@ -61,8 +61,8 @@ usage(char *argv0)
 int main(int argc, char **argv)
 {
     extern int optind;
-    char *filename="testfile.nc", str[512];
-    int i, j, k, rank, nprocs, len, ncid, bufsize, verbose=1, err;
+    char filename[256], str[512];
+    int i, j, k, rank, nprocs, len, ncid, bufsize, verbose=1, err, nerrs=0;
     int *buf, psizes[NDIMS], dimids[NDIMS], dimidsT[NDIMS];
     int XYZ_id, XZY_id, YZX_id, YXZ_id, ZYX_id, ZXY_id;
     MPI_Offset gsizes[NDIMS], starts[NDIMS], counts[NDIMS], imap[NDIMS];
@@ -85,9 +85,12 @@ int main(int argc, char **argv)
         }
     argc -= optind;
     argv += optind;
-    if (argc >= 1) filename = argv[0];  /* optional argument */
+    if (argc >= 1) snprintf(filename, 256, "%s", argv[0]);
+    else           strcpy(filename, "testfile.nc");
+
     len = 2;
-    if (argc >= 2) len = atoi(argv[1]); /* optional argument */
+    if (argc >= 2) len = (int)strtol(argv[1],NULL,10); /* optional argument */
+    len = (len <= 0) ? 2 : len;
 
     for (i=0; i<NDIMS; i++)
         psizes[i] = 0;
@@ -115,9 +118,9 @@ int main(int argc, char **argv)
 
     bufsize = 1;
     for (i=0; i<NDIMS; i++) {
-        gsizes[i]  = (len + i) * psizes[i]; /* global array size */
-        starts[i] *= (len + i);             /* start indices */
-        counts[i]  = (len + i);             /* array elements */
+        gsizes[i]  = (MPI_Offset)(len + i) * psizes[i]; /* global array size */
+        starts[i] *= (MPI_Offset)(len + i);             /* start indices */
+        counts[i]  = (MPI_Offset)(len + i);             /* array elements */
         bufsize   *= (len + i);
     }
 
@@ -131,7 +134,7 @@ int main(int argc, char **argv)
                                        + (starts[1]+j)*gsizes[2]
                                        + (starts[2]+i);
 
-    /* set an MPI-IO hint to disable file offset alignment for fix-sized
+    /* set an MPI-IO hint to disable file offset alignment for fixed-size
      * variables */
     MPI_Info_create(&info);
     MPI_Info_set(info, "nc_var_align_size", "1");
@@ -151,45 +154,45 @@ int main(int argc, char **argv)
     for (i=0; i<NDIMS; i++) {
         sprintf(str, "%c", 'Z'-i);
         err = ncmpi_def_dim(ncid, str, gsizes[i], &dimids[i]);
-        HANDLE_ERROR
+        ERR
     }
 
     /* define variable with no transposed file layout: ZYX */
     err = ncmpi_def_var(ncid, "ZYX_var", NC_INT, NDIMS, dimids, &ZYX_id);
-    HANDLE_ERROR
+    ERR
 
     /* define variable with transposed file layout: ZYX -> ZXY */
     dimidsT[0] = dimids[0]; dimidsT[1] = dimids[2]; dimidsT[2] = dimids[1];
     err = ncmpi_def_var(ncid, "ZXY_var", NC_INT, NDIMS, dimidsT, &ZXY_id);
-    HANDLE_ERROR
+    ERR
 
     /* define variable with transposed file layout: ZYX -> YZX */
     dimidsT[0] = dimids[1]; dimidsT[1] = dimids[0]; dimidsT[2] = dimids[2];
     err = ncmpi_def_var(ncid, "YZX_var", NC_INT, NDIMS, dimidsT, &YZX_id);
-    HANDLE_ERROR
+    ERR
 
     /* define variable with transposed file layout: ZYX -> YXZ */
     dimidsT[0] = dimids[1]; dimidsT[1] = dimids[2]; dimidsT[2] = dimids[0];
     err = ncmpi_def_var(ncid, "YXZ_var", NC_INT, NDIMS, dimidsT, &YXZ_id);
-    HANDLE_ERROR
+    ERR
 
     /* define variable with transposed file layout: ZYX -> XZY */
     dimidsT[0] = dimids[2]; dimidsT[1] = dimids[0]; dimidsT[2] = dimids[1];
     err = ncmpi_def_var(ncid, "XZY_var", NC_INT, NDIMS, dimidsT, &XZY_id);
-    HANDLE_ERROR
+    ERR
 
     /* define variable with transposed file layout: ZYX -> XYZ */
     dimidsT[0] = dimids[2]; dimidsT[1] = dimids[1]; dimidsT[2] = dimids[0];
     err = ncmpi_def_var(ncid, "XYZ_var", NC_INT, NDIMS, dimidsT, &XYZ_id);
-    HANDLE_ERROR
+    ERR
 
     /* exit the define mode */
     err = ncmpi_enddef(ncid);
-    HANDLE_ERROR
+    ERR
 
     /* write the whole variable in file: ZYX */
     err = ncmpi_put_vara_int_all(ncid, ZYX_id, starts, counts, buf);
-    HANDLE_ERROR
+    ERR
 
     /* ZYX -> ZXY: */
     imap[1] = 1; imap[2] = counts[2]; imap[0] = counts[1]*counts[2];
@@ -197,7 +200,7 @@ int main(int argc, char **argv)
     countsT[0] = counts[0]; countsT[1] = counts[2]; countsT[2] = counts[1];
     /* write the transposed variable */
     err = ncmpi_put_varm_int_all(ncid, ZXY_id, startsT, countsT, NULL, imap, buf);
-    HANDLE_ERROR
+    ERR
 
     /* ZYX -> YZX: */
     imap[2] = 1; imap[0] = counts[2]; imap[1] = counts[1]*counts[2];
@@ -205,7 +208,7 @@ int main(int argc, char **argv)
     countsT[0] = counts[1]; countsT[1] = counts[0]; countsT[2] = counts[2];
     /* write the transposed variable */
     err = ncmpi_put_varm_int_all(ncid, YZX_id, startsT, countsT, NULL, imap, buf);
-    HANDLE_ERROR
+    ERR
 
     /* ZYX -> YXZ: */
     imap[1] = 1; imap[0] = counts[2]; imap[2] = counts[1]*counts[2];
@@ -213,7 +216,7 @@ int main(int argc, char **argv)
     countsT[0] = counts[1]; countsT[1] = counts[2]; countsT[2] = counts[0];
     /* write the transposed variable */
     err = ncmpi_put_varm_int_all(ncid, YXZ_id, startsT, countsT, NULL, imap, buf);
-    HANDLE_ERROR
+    ERR
 
     /* ZYX -> XZY: */
     imap[0] = 1; imap[2] = counts[2]; imap[1] = counts[1]*counts[2];
@@ -221,7 +224,7 @@ int main(int argc, char **argv)
     countsT[0] = counts[2]; countsT[1] = counts[0]; countsT[2] = counts[1];
     /* write the transposed variable */
     err = ncmpi_put_varm_int_all(ncid, XZY_id, startsT, countsT, NULL, imap, buf);
-    HANDLE_ERROR
+    ERR
 
     /* ZYX -> XYZ: */
     imap[0] = 1; imap[1] = counts[2]; imap[2] = counts[1]*counts[2];
@@ -229,15 +232,15 @@ int main(int argc, char **argv)
     countsT[0] = counts[2]; countsT[1] = counts[1]; countsT[2] = counts[0];
     /* write the transposed variable */
     err = ncmpi_put_varm_int_all(ncid, XYZ_id, startsT, countsT, NULL, imap, buf);
-    HANDLE_ERROR
+    ERR
 
     /* close the file */
     err = ncmpi_close(ncid);
-    HANDLE_ERROR
+    ERR
 
     free(buf);
 
     MPI_Finalize();
-    return 0;
+    return nerrs;
 }
 
