@@ -2,7 +2,7 @@
 !  Copyright (C) 2012, Northwestern University and Argonne National Laboratory
 !  See COPYRIGHT notice in top-level directory.
 !
-! $Id: column_wise.f 2325 2016-02-28 07:49:13Z wkliao $
+! $Id: column_wise.f 2476 2016-09-06 01:05:33Z wkliao $
 
 ! This example makes a number of nonblocking API calls, each writes a single
 ! row of a 2D integer array. Each process writes NY rows and any two
@@ -69,7 +69,7 @@
           integer NY, NX
           PARAMETER(NX=10, NY=4)
 
-          character*128 filename, cmd
+          character*256 filename, cmd
           integer i, j, rank, nprocs, err, ierr, num_reqs, get_args
           integer ncid, cmode, varid, dimid(2)
           integer buf(NX, NY)
@@ -99,7 +99,7 @@
      +                   MPI_COMM_WORLD, err)
 
           ! set an MPI-IO hint to disable file offset alignment for
-          ! fix-sized variables
+          ! fixed-size variables
           call MPI_Info_create(info, err)
           call MPI_Info_set(info, "nc_var_align_size", "1", err)
 
@@ -190,7 +190,7 @@
  998      format(A,I13,A)
           err = nfmpi_inq_malloc_size(malloc_size)
           if (err .EQ. NF_NOERR) then
-              call MPI_Reduce(malloc_size, sum_size, 1, MPI_OFFSET, 
+              call MPI_Reduce(malloc_size, sum_size, 1, MPI_INTEGER8, 
      +                        MPI_SUM, 0, MPI_COMM_WORLD, err)
               if (rank .EQ. 0 .AND. sum_size .GT. 0)
      +            print 998,
