@@ -2,12 +2,12 @@
  *  Copyright (C) 2003, Northwestern University and Argonne National Laboratory
  *  See COPYRIGHT notice in top-level directory.
  */
-/* $Id: error.c 2320 2016-02-24 13:18:32Z wkliao $ */
+/* $Id: error.c 2639 2016-11-18 15:06:10Z wkliao $ */
 
 /*LINTLIBRARY*/
 
 #if HAVE_CONFIG_H
-# include "ncconfig.h"
+# include <ncconfig.h>
 #endif
 
 #include <stdio.h>
@@ -231,7 +231,7 @@ ncmpi_strerror(int err)
             /* Some APIs require argument count cannot be a NULL pointer
              */
         case NC_EINVAL_CMODE:
-            return "Invalid file create mode, cannot have both NC_64BIT_OFFSET & NC_64BIT_DATA";
+            return "Invalid file create mode";
         case NC_ETYPESIZE:
             return "MPI derived data type size error (bigger than the variable size)";
         case NC_ETYPE_MISMATCH:
@@ -244,6 +244,10 @@ ncmpi_strerror(int err)
             return "Attempting operation only for record variables";
         case NC_ENOTFILL:
             return "Attempting to fill a variable when its fill mode is off";
+        case NC_EINVAL_OMODE:
+            return "Invalid file open mode";
+        case NC_EPENDING:
+            return "Pending nonblocking request is found at file close";
         case NC_EMULTIDEFINE:
             return "File header is inconsistent among processes";
             /* this error means the metadata (dimension names, variable names,
@@ -251,23 +255,25 @@ ncmpi_strerror(int err)
              * in the file header) is inconsistent among all MPI processes.
              */
         case NC_EMULTIDEFINE_OMODE:
-            return "Bad file create/open mode or modes are inconsistent across processes.";
+            return "File open mode is inconsistent among processes.";
         case NC_EMULTIDEFINE_DIM_NUM:
-            return "Number of dimensions is defined inconsistently among processes.";
+            return "Number of dimensions is inconsistent among processes.";
         case NC_EMULTIDEFINE_DIM_SIZE:
-            return "Dimension size is defined inconsistently among processes.";
+            return "Dimension size is inconsistent among processes.";
+        case NC_EMULTIDEFINE_DIM_NAME:
+            return "Dimension name is inconsistent among processes.";
         case NC_EMULTIDEFINE_VAR_NUM:
-            return "Number of variables is defined inconsistently among processes.";
+            return "Number of variables is inconsistent among processes.";
         case NC_EMULTIDEFINE_VAR_NAME:
-            return "Variable names are defined inconsistently among processes.";
+            return "Variable name is inconsistent among processes.";
         case NC_EMULTIDEFINE_VAR_NDIMS:
-            return "Dimensionality of this variable is defined inconsistently among processes.";
+            return "Dimensionality of this variable is inconsistent among processes.";
         case NC_EMULTIDEFINE_VAR_DIMIDS:
-            return "Dimension IDs used to define this variable is inconsistent among processes.";
+            return "Dimension IDs used to define this variable are inconsistent among processes.";
         case NC_EMULTIDEFINE_VAR_TYPE:
-            return "Data type of this variable is defined inconsistently among processes.";
+            return "Data type of this variable is inconsistent among processes.";
         case NC_EMULTIDEFINE_VAR_LEN:
-            return "Number of elements of this variable is defined inconsistently among processes.";
+            return "Number of elements of this variable is inconsistent among processes.";
         case NC_EMULTIDEFINE_VAR_BEGIN:
             return "Starting file offset of this variable is inconsistent among processes.";
         case NC_EMULTIDEFINE_NUMRECS:
@@ -292,6 +298,8 @@ ncmpi_strerror(int err)
             return "Variable's fill mode is inconsistent among processes.";
         case NC_EMULTIDEFINE_VAR_FILL_VALUE:
             return "Variable's fill value is inconsistent among processes.";
+        case NC_EMULTIDEFINE_CMODE:
+            return "File create mode is inconsistent among processes.";
 
         default:
             /* check netCDF-3 and netCDF-4 errors */
@@ -304,7 +312,7 @@ ncmpi_strerror(int err)
 int ncmpii_handle_error(int   mpi_errorcode, /* returned value from MPI call */
                         char *err_msg)       /* extra error message */
 {
-    int rank, errorclass, errorStringLen;
+    int errorclass, errorStringLen;
     char errorString[MPI_MAX_ERROR_STRING];
 
     /* check for specific error codes understood by PnetCDF */
@@ -359,11 +367,16 @@ int ncmpii_handle_error(int   mpi_errorcode, /* returned value from MPI call */
 
     /* other errors that currently have no corresponding PnetCDF error codes */
 
-    /* we report the world rank */
-    MPI_Comm_rank(MPI_COMM_WORLD, &rank);
     MPI_Error_string(mpi_errorcode, errorString, &errorStringLen);
     if (err_msg == NULL) err_msg = "";
+#ifdef PNETCDF_DEBUG
+    /* report the world rank */
+    int rank;
+    MPI_Comm_rank(MPI_COMM_WORLD, &rank);
     printf("rank %d: MPI error (%s) : %s\n", rank, err_msg, errorString);
+#else
+    printf("MPI error (%s) : %s\n", err_msg, errorString);
+#endif
 
     return NC_EFILE; /* other unknown file I/O error */
 }
@@ -612,9 +625,22 @@ nc_strerror(int ncerr1)
 }
 /** \} */
 
-char* ncmpii_err_code_name(int err)
+/*----< ncmpi_strerrno() >---------------------------------------------------*/
+/* print the NC error code name */
+const char *
+ncmpi_strerrno(int err)
 {
-    static char unknown_str[32];
+    static char unknown_str[64];
+
+    if (err > 0) { /* system error */
+        const char *cp = (const char *) strerror(err);
+        if (cp == NULL)
+            sprintf(unknown_str,"Unknown error code %d",err);
+        else
+            sprintf(unknown_str,"System error code %d (%s)",err,cp);
+        return unknown_str;
+    }
+
     switch (err) {
         case (NC_NOERR):			return "NC_NOERR";
         case (NC_EBADID):			return "NC_EBADID";
@@ -726,6 +752,8 @@ char* ncmpii_err_code_name(int err)
         case (NC_ENULLSTART):			return "NC_ENULLSTART";
         case (NC_ENULLCOUNT):			return "NC_ENULLCOUNT";
         case (NC_EINVAL_CMODE):			return "NC_EINVAL_CMODE";
+        case (NC_EINVAL_OMODE):			return "NC_EINVAL_OMODE";
+        case (NC_EPENDING):			return "NC_EPENDING";
         case (NC_ETYPESIZE):			return "NC_ETYPESIZE";
         case (NC_ETYPE_MISMATCH):		return "NC_ETYPE_MISMATCH";
         case (NC_ETYPESIZE_MISMATCH):		return "NC_ETYPESIZE_MISMATCH";
@@ -755,6 +783,7 @@ char* ncmpii_err_code_name(int err)
         case (NC_EMULTIDEFINE_FILL_MODE):	return "NC_EMULTIDEFINE_FILL_MODE";
         case (NC_EMULTIDEFINE_VAR_FILL_MODE):	return "NC_EMULTIDEFINE_VAR_FILL_MODE";
         case (NC_EMULTIDEFINE_VAR_FILL_VALUE):	return "NC_EMULTIDEFINE_VAR_FILL_VALUE";
+        case (NC_EMULTIDEFINE_CMODE):		return "NC_EMULTIDEFINE_CMODE";
         default:
               sprintf(unknown_str,"Unknown code %d",err);
     }
