@@ -2,10 +2,10 @@
  *  Copyright (C) 2003, Northwestern University and Argonne National Laboratory
  *  See COPYRIGHT notice in top-level directory.
  */
-/* $Id: filetype.c 2335 2016-03-03 14:55:12Z wkliao $ */
+/* $Id: filetype.c 2603 2016-11-08 19:04:56Z wkliao $ */
 
 #if HAVE_CONFIG_H
-# include "ncconfig.h"
+# include <ncconfig.h>
 #endif
 
 #include <stdio.h>
@@ -66,10 +66,10 @@ NC_start_count_stride_ck(const NC         *ncp,
     if (varp->ndims == 0) return NC_NOERR; /* 'scalar' variable */
 
     /* negative start[] is illegal */
-    if (start[0] < 0) DEBUG_RETURN_ERROR(NC_EINVALCOORDS)
+    if (start == NULL || start[0] < 0) DEBUG_RETURN_ERROR(NC_EINVALCOORDS)
 
     if (IS_RECVAR(varp)) {
-        if (!fIsSet(ncp->flags, NC_64BIT_DATA) && /* not CDF-5 */
+        if (ncp->format < 5 && /* not CDF-5 */
             start[0] > X_UINT_MAX) /* sanity check */
             DEBUG_RETURN_ERROR(NC_EINVALCOORDS)
 
@@ -78,10 +78,16 @@ NC_start_count_stride_ck(const NC         *ncp,
 
         /* for record variable, [0] is the NC_UNLIMITED dimension */
         if (rw_flag == READ_REQ) { /* read cannot go beyond current numrecs */
-            if (start[0] >= ncp->numrecs)
-                DEBUG_RETURN_ERROR(NC_EINVALCOORDS)
-
+#ifdef RELAX_COORD_BOUND
+            if (start[0] > ncp->numrecs) DEBUG_RETURN_ERROR(NC_EINVALCOORDS)
+#else
+            if (start[0] >= ncp->numrecs) DEBUG_RETURN_ERROR(NC_EINVALCOORDS)
+#endif
             if (count != NULL) {
+#ifdef RELAX_COORD_BOUND
+                if (start[0] == ncp->numrecs && count[0] > 0)
+                    DEBUG_RETURN_ERROR(NC_EINVALCOORDS)
+#endif
                 if (stride == NULL) { /* for vara APIs */
                     if (start[0] + count[0] > ncp->numrecs)
                         DEBUG_RETURN_ERROR(NC_EEDGE)
@@ -121,8 +127,16 @@ NC_start_count_stride_ck(const NC         *ncp,
     }
 
     for (; i<varp->ndims; i++) {
+#ifdef RELAX_COORD_BOUND
+        if (start[i] < 0 || start[i] > varp->shape[i])
+            DEBUG_RETURN_ERROR(NC_EINVALCOORDS)
+
+        if (count != NULL && start[i] == varp->shape[i] && count[i] > 0)
+            DEBUG_RETURN_ERROR(NC_EINVALCOORDS)
+#else
         if (start[i] < 0 || start[i] >= varp->shape[i])
             DEBUG_RETURN_ERROR(NC_EINVALCOORDS)
+#endif
 
         if (varp->shape[i] < 0) DEBUG_RETURN_ERROR(NC_EEDGE)
 
@@ -148,7 +162,10 @@ NC_start_count_stride_ck(const NC         *ncp,
 }
 
 /*----< ncmpii_get_offset() >------------------------------------------------*/
-/* returns the file offset of the last byte accessed of this request */
+/* returns the file offset of the last byte accessed of this request
+ * If counts is NULL, this is equivalent to the starting offset of this
+ * request
+ */
 int
 ncmpii_get_offset(NC               *ncp,
                   NC_var           *varp,
@@ -919,7 +936,7 @@ ncmpii_vars_create_filetype(NC               *ncp,
 int
 ncmpii_file_set_view(NC           *ncp,
                      MPI_File      fh,
-                     MPI_Offset   *offset,
+                     MPI_Offset   *offset,  /* IN/OUT */
                      MPI_Datatype  filetype)
 {
     int rank, err, mpireturn, status=NC_NOERR;
