@@ -2,10 +2,10 @@
  *  Copyright (C) 2015, Northwestern University and Argonne National Laboratory
  *  See COPYRIGHT notice in top-level directory.
  */
-/* $Id: fill.c 2320 2016-02-24 13:18:32Z wkliao $ */
+/* $Id: fill.c 2722 2016-12-18 06:20:38Z wkliao $ */
 
 #if HAVE_CONFIG_H
-# include "ncconfig.h"
+# include <ncconfig.h>
 #endif
 
 #ifdef HAVE_STDLIB_H
@@ -13,6 +13,7 @@
 #endif
 #include <stdio.h>
 #include <string.h>
+#include <assert.h>
 
 #include <mpi.h>
 
@@ -57,25 +58,25 @@ static unsigned char FILL_UINT[4]   = {0xFF, 0xFF, 0xFF, 0xFF};
 static unsigned char FILL_INT64[8]  = {0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02};
 static unsigned char FILL_UINT64[8] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFE};
 
-/*----< inq_default_fill_value() >-------------------------------------------*/
+/*----< ncmpii_inq_default_fill_value() >------------------------------------*/
 /* copy the default fill value to the memory space pointed by fill_value */
-static int
-inq_default_fill_value(int type, void *fill_value)
+int
+ncmpii_inq_default_fill_value(int xtype, void *fillp)
 {
-    if (fill_value == NULL) return NC_NOERR;
+    if (fillp == NULL) return NC_NOERR;
 
-    switch(type) {
-        case NC_CHAR   :               *(char*)fill_value = NC_FILL_CHAR;   break;
-        case NC_BYTE   :        *(signed char*)fill_value = NC_FILL_BYTE;   break;
-        case NC_SHORT  :              *(short*)fill_value = NC_FILL_SHORT;  break;
-        case NC_INT    :                *(int*)fill_value = NC_FILL_INT;    break;
-        case NC_FLOAT  :              *(float*)fill_value = NC_FILL_FLOAT;  break;
-        case NC_DOUBLE :             *(double*)fill_value = NC_FILL_DOUBLE; break;
-        case NC_UBYTE  :      *(unsigned char*)fill_value = NC_FILL_UBYTE;  break;
-        case NC_USHORT :     *(unsigned short*)fill_value = NC_FILL_USHORT; break;
-        case NC_UINT   :       *(unsigned int*)fill_value = NC_FILL_UINT;   break;
-        case NC_INT64  :          *(long long*)fill_value = NC_FILL_INT64;  break;
-        case NC_UINT64 : *(unsigned long long*)fill_value = NC_FILL_UINT64; break;
+    switch(xtype) {
+        case NC_CHAR   :               *(char*)fillp = NC_FILL_CHAR;   break;
+        case NC_BYTE   :        *(signed char*)fillp = NC_FILL_BYTE;   break;
+        case NC_SHORT  :              *(short*)fillp = NC_FILL_SHORT;  break;
+        case NC_INT    :                *(int*)fillp = NC_FILL_INT;    break;
+        case NC_FLOAT  :              *(float*)fillp = NC_FILL_FLOAT;  break;
+        case NC_DOUBLE :             *(double*)fillp = NC_FILL_DOUBLE; break;
+        case NC_UBYTE  :      *(unsigned char*)fillp = NC_FILL_UBYTE;  break;
+        case NC_USHORT :     *(unsigned short*)fillp = NC_FILL_USHORT; break;
+        case NC_UINT   :       *(unsigned int*)fillp = NC_FILL_UINT;   break;
+        case NC_INT64  :          *(long long*)fillp = NC_FILL_INT64;  break;
+        case NC_UINT64 : *(unsigned long long*)fillp = NC_FILL_UINT64; break;
         default : DEBUG_RETURN_ERROR(NC_EBADTYPE)
     }
     return NC_NOERR;
@@ -225,31 +226,77 @@ ncmpi_fill_var_rec(int        ncid,
 {
     int     indx, err;
     NC     *ncp;
-    NC_var *varp;
+    NC_var *varp=NULL;
 
     /* check if ncid is valid */
     err = ncmpii_NC_check_id(ncid, &ncp);
-    if (err != NC_NOERR) return err;
+    if (err != NC_NOERR) DEBUG_RETURN_ERROR(err)
 
-    if (NC_readonly(ncp)) DEBUG_RETURN_ERROR(NC_EPERM) /* read-only */
+    /* check file's write permission */
+    if (NC_readonly(ncp)) {
+        DEBUG_ASSIGN_ERROR(err, NC_EPERM)
+        goto err_check;
+    }
 
-    /* This must be called in data mode */
-    if (NC_indef(ncp)) DEBUG_RETURN_ERROR(NC_EINDEFINE)                                
-
-    /* must be called in collective data mode */
-    if (NC_indep(ncp)) DEBUG_RETURN_ERROR(NC_EINDEP)
+    /* must be called in data mode */
+    if (NC_indef(ncp)) {
+        DEBUG_ASSIGN_ERROR(err, NC_EINDEFINE)
+        goto err_check;
+    }
 
+    /* check whether variable ID is valid */
     err = ncmpii_NC_lookupvar(ncp, varid, &varp);
-    if (err != NC_NOERR) return err;
+    if (err != NC_NOERR) {
+        DEBUG_TRACE_ERROR
+        goto err_check;
+    }
 
     /* error if this is not a record variable */
-    if (!IS_RECVAR(varp)) DEBUG_RETURN_ERROR(NC_ENOTRECVAR)
+    if (!IS_RECVAR(varp)) {
+        DEBUG_ASSIGN_ERROR(err, NC_ENOTRECVAR)
+        goto err_check;
+    }
 
     /* check if _FillValue attribute is defined */
     indx = ncmpii_NC_findattr(&varp->attrs, _FillValue);
 
     /* error if the fill mode of this variable is not on */
-    if (varp->no_fill && indx == -1) DEBUG_RETURN_ERROR(NC_ENOTFILL)
+    if (varp->no_fill && indx == -1) {
+        DEBUG_ASSIGN_ERROR(err, NC_ENOTFILL)
+        goto err_check;
+    }
+
+err_check:
+    if (ncp->safe_mode) { /* consistency check */
+        int root_varid, status, mpireturn;
+        MPI_Offset root_recno;
+
+        /* check if varid is consistent across all processes */
+        root_varid = varid;
+        TRACE_COMM(MPI_Bcast)(&root_varid, 1, MPI_INT, 0, ncp->nciop->comm);
+        if (mpireturn != MPI_SUCCESS)
+            return ncmpii_handle_error(mpireturn, "MPI_Bcast");
+        if (err == NC_NOERR && root_varid != varid)
+            DEBUG_ASSIGN_ERROR(err, NC_EMULTIDEFINE_FNC_ARGS)
+
+        /* check if recno is consistent across all processes */
+        root_recno = recno;
+        TRACE_COMM(MPI_Bcast)(&root_recno, 1, MPI_OFFSET, 0, ncp->nciop->comm);
+        if (mpireturn != MPI_SUCCESS)
+            return ncmpii_handle_error(mpireturn, "MPI_Bcast");
+        if (err == NC_NOERR && root_recno != recno)
+            DEBUG_ASSIGN_ERROR(err, NC_EMULTIDEFINE_FNC_ARGS)
+
+        /* find min error code across processes */
+        TRACE_COMM(MPI_Allreduce)(&err, &status, 1, MPI_INT, MPI_MIN, ncp->nciop->comm);
+        if (mpireturn != MPI_SUCCESS)
+            return ncmpii_handle_error(mpireturn, "MPI_Allreduce");
+
+        if (err == NC_NOERR) err = status;
+    }
+    if (err != NC_NOERR) return err;
+
+    assert(varp != NULL);
 
     return ncmpii_fill_var_rec(ncp, varp, recno);
 }
@@ -257,36 +304,51 @@ ncmpi_fill_var_rec(int        ncid,
 /*----< ncmpi_set_fill() >---------------------------------------------------*/
 /* this API is collective, must be called in define mode, contrary to netCDF
  * where nc_set_fill() can also be called in data mode. The reason of PnetCDF
- * enforcing this requirement is because PnetCDF only fills fix-sized variables
- * at ncmpi_enddef() and record variables in ncmpi_fill_var_rec().
+ * enforcing this requirement is because PnetCDF only fills fixed-size
+ * variables at ncmpi_enddef() and record variables in ncmpi_fill_var_rec().
  */
 int
 ncmpi_set_fill(int  ncid,
                int  fill_mode,
                int *old_fill_mode)
 {
-    int i, status=NC_NOERR, mpireturn, oldmode;
+    int i, err, mpireturn, oldmode;
     NC *ncp;
 
-    status = ncmpii_NC_check_id(ncid, &ncp);
-    if (status != NC_NOERR) return status;
+    /* check whether ncid is valid */
+    err = ncmpii_NC_check_id(ncid, &ncp);
+    if (err != NC_NOERR) DEBUG_RETURN_ERROR(err)
 
-    if (NC_readonly(ncp)) DEBUG_RETURN_ERROR(NC_EPERM) /* read-only */
+    /* check whether file's write permission */
+    if (NC_readonly(ncp)) {
+        DEBUG_ASSIGN_ERROR(err, NC_EPERM)
+        goto err_check;
+    }
 
     /* check if called in define mode */
-    if (!NC_indef(ncp)) DEBUG_RETURN_ERROR(NC_ENOTINDEFINE)
+    if (!NC_indef(ncp)) {
+        DEBUG_ASSIGN_ERROR(err, NC_ENOTINDEFINE)
+        goto err_check;
+    }
 
+err_check:
     if (ncp->safe_mode) {
-        int root_fill_mode=fill_mode;
+        int status, root_fill_mode=fill_mode;
+
         TRACE_COMM(MPI_Bcast)(&root_fill_mode, 1, MPI_INT, 0, ncp->nciop->comm);
         if (mpireturn != MPI_SUCCESS)
             return  ncmpii_handle_error(mpireturn, "MPI_Bcast"); 
-        if (fill_mode != root_fill_mode) {
+        if (err == NC_NOERR && fill_mode != root_fill_mode)
             /* dataset's fill mode is inconsistent with root's */
-            printf("Warning: fill mode set in %s() is inconsistent\n", __func__);
-            DEBUG_ASSIGN_ERROR(status, NC_EMULTIDEFINE_FILL_MODE)
-        }
+            DEBUG_ASSIGN_ERROR(err, NC_EMULTIDEFINE_FILL_MODE)
+
+        /* find min error code across processes */
+        TRACE_COMM(MPI_Allreduce)(&err, &status, 1, MPI_INT, MPI_MIN, ncp->nciop->comm);
+        if (mpireturn != MPI_SUCCESS)
+            return ncmpii_handle_error(mpireturn, "MPI_Allreduce");
+        if (err == NC_NOERR) err = status;
     }
+    if (err != NC_NOERR) return err;
 
     oldmode = fIsSet(ncp->flags, NC_NOFILL) ? NC_NOFILL : NC_FILL;
 
@@ -301,13 +363,13 @@ ncmpi_set_fill(int  ncid,
 
     /* loop thru all variables defined so far to set/overwrite its fill mode */
     for (i=0; i<ncp->vars.ndefined; i++)
-        ncp->vars.value[i]->no_fill = (fill_mode == NC_NOFILL) ? 1 : 0;
+        ncp->vars.value[i]->no_fill = (char)((fill_mode == NC_NOFILL) ? 1 : 0);
 
     /* once the file's fill mode is set, any new variables defined after this
      * call will check NC_dofill(ncp) and set their no_fill accordingly. See
      * ncmpi_def_var() */
 
-    return status;
+    return NC_NOERR;
 }
 
 /*----< ncmpi_def_var_fill() >------------------------------------------------*/
@@ -318,64 +380,74 @@ ncmpi_def_var_fill(int   ncid,
                    int   no_fill,    /* 1: no fill, 0: fill */
                    void *fill_value) /* when NULL, use default fill value */
 {
-    int err, status=NC_NOERR, mpireturn, free_fill_value=0;
+    int err;
     NC *ncp;
-    NC_var *varp;
+    NC_var *varp=NULL;
 
+    /* check whether ncid is valid */
     err = ncmpii_NC_check_id(ncid, &ncp);
-    if (err != NC_NOERR) return err;
+    if (err != NC_NOERR) DEBUG_RETURN_ERROR(err)
 
-    if (NC_readonly(ncp)) DEBUG_RETURN_ERROR(NC_EPERM) /* read-only */
-
-    /* must be called in define mode */
-    if (!NC_indef(ncp)) DEBUG_RETURN_ERROR(NC_ENOTINDEFINE)
-
-    /* find the pointer to this variable's object */
-    err = ncmpii_NC_lookupvar(ncp, varid, &varp);
-    if (err != NC_NOERR) return err;
-
-    if (ncp->safe_mode) {
-        int root_no_fill=no_fill;
-        TRACE_COMM(MPI_Bcast)(&root_no_fill, 1, MPI_INT, 0, ncp->nciop->comm);
-        if (mpireturn != MPI_SUCCESS)
-            return  ncmpii_handle_error(mpireturn, "MPI_Bcast"); 
-
-        if (no_fill != root_no_fill) {
-            /* variable's fill mode is inconsistent with root's */
-            printf("Warning: variable (%s) fill mode (%d) set in %s() is inconsistent\n",
-                   varp->name->cp, no_fill, __func__);
-            if (status == NC_NOERR) DEBUG_ASSIGN_ERROR(status, NC_EMULTIDEFINE_VAR_FILL_MODE)
-        }
-
-        /* check if fill_value is consistent among processes */
-        void *root_fill_value = NCI_Malloc((size_t)varp->xsz);
-        if (fill_value == NULL) {
-            /* user intends to use default fill value */
-            fill_value = NCI_Malloc((size_t)varp->xsz);
-            err = inq_default_fill_value(varp->type, fill_value);
-            if (err != NC_NOERR) {
-                NCI_Free(fill_value);
-                NCI_Free(root_fill_value);
-                return err;
-            }
-            free_fill_value=1;
-        }
-        memcpy(root_fill_value, fill_value, (size_t)varp->xsz);
-            
-        TRACE_COMM(MPI_Bcast)(root_fill_value, varp->xsz, MPI_BYTE, 0, ncp->nciop->comm);
-        if (memcmp(fill_value, root_fill_value, (size_t)varp->xsz)) {
-            /* variable's fill value is inconsistent with root's */
-            printf("Warning: variable (%s) fill value set in %s() is inconsistent\n",
-                   varp->name->cp, __func__);
-            if (status == NC_NOERR) DEBUG_ASSIGN_ERROR(status, NC_EMULTIDEFINE_VAR_FILL_VALUE)
-        }
-        if (free_fill_value) {
-            NCI_Free(fill_value);
-            fill_value = NULL;
-        }
-        NCI_Free(root_fill_value);
+    /* check whether file's write permission */
+    if (NC_readonly(ncp)) {
+        DEBUG_ASSIGN_ERROR(err, NC_EPERM)
+        goto err_check;
     }
 
+    /* check if called in define mode */
+    if (!NC_indef(ncp)) {
+        DEBUG_ASSIGN_ERROR(err, NC_ENOTINDEFINE)
+        goto err_check;
+    }
+
+    /* check whether variable ID is valid */
+    err = ncmpii_NC_lookupvar(ncp, varid, &varp);
+    if (err != NC_NOERR) {
+        DEBUG_TRACE_ERROR
+        goto err_check;
+    }
+
+err_check:
+    if (ncp->safe_mode) {
+        int root_ids[3], my_fill_null, status, mpireturn;
+
+        /* check if varid, no_fill, fill_value, are consistent across all processes */
+        my_fill_null = (fill_value == NULL) ? 1 : 0;;
+        root_ids[0] = varid;
+        root_ids[1] = no_fill;
+        root_ids[2] = my_fill_null;
+        TRACE_COMM(MPI_Bcast)(&root_ids, 3, MPI_INT, 0, ncp->nciop->comm);
+        if (mpireturn != MPI_SUCCESS)
+            return ncmpii_handle_error(mpireturn, "MPI_Bcast");
+        if (err == NC_NOERR && (root_ids[0] != varid ||
+            root_ids[1] != no_fill || root_ids[2] != my_fill_null))
+            DEBUG_ASSIGN_ERROR(err, NC_EMULTIDEFINE_FNC_ARGS)
+
+        /* check fill_value, if not NULL, is consistent among processes */
+        if (varp!= NULL && root_ids[2] == 0) {
+            double root_fill_value; /* max nc_type space: 8 bytes */
+            if (fill_value != NULL)
+                memcpy(&root_fill_value, fill_value, (size_t)varp->xsz);
+            TRACE_COMM(MPI_Bcast)(&root_fill_value, varp->xsz, MPI_BYTE, 0, ncp->nciop->comm);
+            if (mpireturn != MPI_SUCCESS)
+                return ncmpii_handle_error(mpireturn, "MPI_Bcast");
+            if (err == NC_NOERR && fill_value != NULL &&
+                memcmp(fill_value, &root_fill_value, (size_t)varp->xsz))
+                /* variable's fill value is inconsistent with root's */
+                DEBUG_ASSIGN_ERROR(err, NC_EMULTIDEFINE_VAR_FILL_VALUE)
+        }
+
+        /* find min error code across processes */
+        TRACE_COMM(MPI_Allreduce)(&err, &status, 1, MPI_INT, MPI_MIN, ncp->nciop->comm);
+        if (mpireturn != MPI_SUCCESS)
+            return ncmpii_handle_error(mpireturn, "MPI_Allreduce");
+
+        if (err == NC_NOERR) err = status;
+    }
+    if (err != NC_NOERR) return err;
+
+    assert(varp != NULL);
+
     if (no_fill)
         varp->no_fill = 1;
     else
@@ -394,7 +466,7 @@ ncmpi_def_var_fill(int   ncid,
         if (err != NC_NOERR) return err;
     }
 
-    return status;
+    return NC_NOERR;
 }
 
 /*----< ncmpi_inq_var_fill() >-----------------------------------------------*/
@@ -416,14 +488,14 @@ ncmpi_inq_var_fill(int   ncid,
     err = ncmpii_NC_lookupvar(ncp, varid, &varp);
     if (err != NC_NOERR) return err;
 
-    *no_fill = varp->no_fill;
+    if (no_fill != NULL) *no_fill = varp->no_fill;
 
     if (fill_value != NULL) {
         err = ncmpi_get_att(ncid, varid, _FillValue, fill_value);
         if (err != NC_NOERR && err != NC_ENOTATT)
             return err;
         if (err == NC_ENOTATT) {
-            err = inq_default_fill_value(varp->type, fill_value);
+            err = ncmpii_inq_default_fill_value(varp->type, fill_value);
             if (err != NC_NOERR) return err;
         }
     }
@@ -431,6 +503,39 @@ ncmpi_inq_var_fill(int   ncid,
     return NC_NOERR;
 }
 
+/*----< ncmpii_inq_var_fill() >----------------------------------------------*/
+int
+ncmpii_inq_var_fill(NC_var *varp,
+                    void   *fill_value) /* OUT: user-defined or
+                                                default fill value */
+{
+    int i;
+    size_t nchars;
+    NC_attrarray *attrs;
+
+    assert(varp != NULL); /* NC_GLOBAL varid is illegal in this context */
+
+    nchars = strlen("_FillValue");
+
+    /* Check if _FillValue is defined for this variable */
+    attrs = &varp->attrs;
+    for (i=0; i<attrs->ndefined; i++) {
+        if (attrs->value[i]->name->nchars == (MPI_Offset)nchars &&
+            strncmp(attrs->value[i]->name->cp, "_FillValue", nchars) == 0) {
+            memcpy(fill_value, attrs->value[i]->xvalue, (size_t)varp->xsz);
+            return NC_NOERR;
+        }
+    }
+
+    /* NetCDF 4.4.1 and prior does not use global attribute _FillValue if
+     * it is not defined for the variable. Default fill values are used.
+     * See fill_NC_var() in putget.m4.
+     */
+
+    /* return default _FillValue */
+    return ncmpii_inq_default_fill_value(varp->type, fill_value);
+}
+
 #ifdef FILL_ONE_VAR_AT_A_TIME
 /*----< fillerup() >---------------------------------------------------------*/
 /* fill the newly created fixed-size variables */
@@ -526,13 +631,13 @@ fill_added_recs(NC *ncp, NC *old_ncp)
 static int
 fillerup_aggregate(NC *ncp, NC *old_ncp)
 {
-    int i, j, k, rank, nprocs, start_vid, nsegs, recno;
+    int i, j, k, rank, nprocs, start_vid, recno;
     int nVarsFill, *blocklengths;
     int mpireturn, err, status=NC_NOERR;
     char *buf_ptr, *noFill;
     void *buf;
-    size_t buf_len;
-    MPI_Offset var_len, nrecs, start, *count;
+    size_t nsegs;
+    MPI_Offset buf_len, var_len, nrecs, start, *count;
     MPI_Aint *offset;
     MPI_Datatype filetype;
     MPI_File fh;
@@ -550,28 +655,36 @@ fillerup_aggregate(NC *ncp, NC *old_ncp)
         nrecs = NC_get_numrecs(old_ncp);
     }
 
+    noFill = (char*) NCI_Malloc((size_t)(ncp->vars.ndefined - start_vid));
+    nVarsFill = 0;
+
+#ifdef _CHECK_FILL_MODE_CONSISTENCY
     /* Because fill mode is not part of file header, we must broadcast root's
      * variables' fill modes and overwrite local's if an inconsistency is found
      * Note ncp->vars.ndefined is already made consistent by this point.
      */
-    noFill = (char*) NCI_Malloc(ncp->vars.ndefined - start_vid);
     for (i=start_vid; i<ncp->vars.ndefined; i++)
         noFill[i-start_vid] = ncp->vars.value[i]->no_fill;
-
     TRACE_COMM(MPI_Bcast)(noFill, (ncp->vars.ndefined - start_vid), MPI_BYTE, 0,
                           ncp->nciop->comm);
-    nVarsFill = 0;
-    for (i=start_vid; i<ncp->vars.ndefined; i++) { /* overwrite local's mode */
+    for (i=start_vid; i<ncp->vars.ndefined; i++) {
+        /* overwrite local's mode */
         ncp->vars.value[i]->no_fill = noFill[i-start_vid];
         if (!noFill[i-start_vid]) nVarsFill++;
     }
+#else
+    for (i=start_vid; i<ncp->vars.ndefined; i++) {
+        noFill[i-start_vid] = ncp->vars.value[i]->no_fill;
+        if (!noFill[i-start_vid]) nVarsFill++;
+    }
+#endif
     if (nVarsFill == 0) { /* no variables in fill mode */
         NCI_Free(noFill);
         return NC_NOERR;
     }
 
     /* find the number of write segments (upper bound) */
-    nsegs = ncp->vars.ndefined + ncp->vars.num_rec_vars * nrecs;
+    nsegs = (size_t)(ncp->vars.ndefined + ncp->vars.num_rec_vars * nrecs);
     count  = (MPI_Offset*) NCI_Malloc(nsegs * sizeof(MPI_Offset));
     offset = (MPI_Aint*)   NCI_Malloc(nsegs * sizeof(MPI_Aint));
 
@@ -611,7 +724,7 @@ fillerup_aggregate(NC *ncp, NC *old_ncp)
         /* add up the buffer size */
         buf_len += count[j] * varp->xsz;
 
-        j++;
+        j++; /* increase j even when count[j] is zero */
     }
 
     /* loop thru all record variables to find the aggregated write amount */
@@ -648,14 +761,21 @@ fillerup_aggregate(NC *ncp, NC *old_ncp)
             /* add up the buffer size */
             buf_len += count[j] * varp->xsz;
 
-            j++;
+            j++; /* increase j even when count[j] is zero */
         }
     }
     /* j is now the number of valid write segments */
 
+    if (status == NC_NOERR && j == 0) {
+        NCI_Free(noFill);
+        NCI_Free(count);
+        NCI_Free(offset);
+        return NC_NOERR;
+    }
+
     /* allocate one contiguous buffer space for all writes */
-    blocklengths = (int*) NCI_Malloc(j * sizeof(int));
-    buf = NCI_Malloc(buf_len);
+    blocklengths = (int*) NCI_Malloc((size_t)j * sizeof(int));
+    buf = NCI_Malloc((size_t)buf_len);
     buf_ptr = (char*)buf;
 
     /* fill write buffers for fixed-size variables first */
@@ -722,20 +842,26 @@ fillerup_aggregate(NC *ncp, NC *old_ncp)
     /* k is the number of valid write requests */
     NCI_Free(noFill);
 
-    /* create the fileview: a list of contiguous segment for each variable */
+    if (k == 0) {
+        filetype = MPI_BYTE;
+    }
+    else {
+        /* create fileview: a list of contiguous segment for each variable */
 #ifdef HAVE_MPI_TYPE_CREATE_HINDEXED
-    mpireturn = MPI_Type_create_hindexed(k, blocklengths, offset, MPI_BYTE,
-                                         &filetype);
+        mpireturn = MPI_Type_create_hindexed(k, blocklengths, offset, MPI_BYTE,
+                                             &filetype);
 #else
-    mpireturn = MPI_Type_hindexed(k, blocklengths, offset, MPI_BYTE, &filetype);
+        mpireturn = MPI_Type_hindexed(k, blocklengths, offset, MPI_BYTE,
+                                      &filetype);
 #endif
-    if (mpireturn != MPI_SUCCESS) {
-        int err = ncmpii_handle_error(mpireturn, "MPI_Type_hindexed");
-        /* return the first encountered error if there is any */
-        if (status == NC_NOERR) status = err;
+        if (mpireturn != MPI_SUCCESS) {
+            err = ncmpii_handle_error(mpireturn, "MPI_Type_hindexed");
+            /* return the first encountered error if there is any */
+            if (status == NC_NOERR) status = err;
+        }
+        else
+            MPI_Type_commit(&filetype);
     }
-    else
-        MPI_Type_commit(&filetype);
 
     NCI_Free(blocklengths);
     NCI_Free(count);
@@ -745,10 +871,14 @@ fillerup_aggregate(NC *ncp, NC *old_ncp)
 
     TRACE_IO(MPI_File_set_view)(fh, 0, MPI_BYTE, filetype, "native",
                                 MPI_INFO_NULL);
-    MPI_Type_free(&filetype);
+    if (k > 0) MPI_Type_free(&filetype);
+
+    if (buf_len != (int)buf_len)
+        if (status == NC_NOERR)
+            status = NC_EINTOVERFLOW;
 
     /* write to variable collectively */
-    TRACE_IO(MPI_File_write_at_all)(fh, 0, buf, buf_len, MPI_BYTE, &mpistatus);
+    TRACE_IO(MPI_File_write_at_all)(fh, 0, buf, (int)buf_len, MPI_BYTE, &mpistatus);
 
     NCI_Free(buf);
 
