@@ -2,10 +2,10 @@
  *  Copyright (C) 2003, Northwestern University and Argonne National Laboratory
  *  See COPYRIGHT notice in top-level directory.
  */
-/* $Id: header.c 2322 2016-02-26 22:47:06Z wkliao $ */
+/* $Id: header.c 2657 2016-11-27 22:11:00Z wkliao $ */
 
 #if HAVE_CONFIG_H
-# include "ncconfig.h"
+# include <ncconfig.h>
 #endif
 
 #include <assert.h>
@@ -61,9 +61,9 @@ static int hdr_get_NC_vararray(bufferinfo *gbp, NC_vararray *ncap);
 /*
  * "magic number" at beginning of file: 0x43444601 (big endian)
  */
-static const schar ncmagic1[] = {'C', 'D', 'F', 0x01};
-static const schar ncmagic2[] = {'C', 'D', 'F', 0x02};
-static const schar ncmagic5[] = {'C', 'D', 'F', 0x05};
+static const char ncmagic1[] = {'C', 'D', 'F', 0x01};
+static const char ncmagic2[] = {'C', 'D', 'F', 0x02};
+static const char ncmagic5[] = {'C', 'D', 'F', 0x05};
 
 /*
  * Recompute the shapes of all variables
@@ -90,7 +90,7 @@ ncmpii_NC_computeshapes(NC *ncp)
 
     for ( /*NADA*/; vpp < end; vpp++) {
         /* (*vpp)->len is recomputed from dimensions in ncmpii_NC_var_shape64() */
-        status = ncmpii_NC_var_shape64(ncp, *vpp, &ncp->dims);
+        status = ncmpii_NC_var_shape64(*vpp, &ncp->dims);
 
         if (status != NC_NOERR) return status ;
 
@@ -386,11 +386,11 @@ ncmpii_hdr_len_NC(const NC *ncp)
 
     assert(ncp != NULL);
 
-    if (fIsSet(ncp->flags, NC_64BIT_DATA)) {        /* CDF-5 */
+    if (ncp->format == 5) {        /* CDF-5 */
         sizeof_t     = X_SIZEOF_INT64; /* 8-byte integer for all integers */
         sizeof_off_t = X_SIZEOF_INT64; /* 8-byte integer for var begin */
     }
-    else if (fIsSet(ncp->flags, NC_64BIT_OFFSET)) { /* CDF-2 */
+    else if (ncp->format == 2) { /* CDF-2 */
         sizeof_t     = X_SIZEOF_INT; /* 4-byte integer in CDF-1 */
         sizeof_off_t = X_SIZEOF_INT64; /* 8-byte integer for var begin */
     }
@@ -426,22 +426,22 @@ hdr_put_NC_name(bufferinfo      *pbp,
      * NON_NEG    = <non-negative INT> |  // CDF-1 and CDF-2
      *              <non-negative INT64>  // CDF-5
      */
-    int status;
+    int err;
 
     /* copy nelems */
     if (pbp->version == 5)
-        status = ncmpix_put_uint64((void**)(&pbp->pos), ncstrp->nchars);
+        err = ncmpix_put_uint64((void**)(&pbp->pos), (uint64)ncstrp->nchars);
     else {
-        if (ncstrp->nchars != (uint)ncstrp->nchars) DEBUG_RETURN_ERROR(NC_EINTOVERFLOW)
-        status = ncmpix_put_uint32((void**)(&pbp->pos), (uint)ncstrp->nchars);
+        if (ncstrp->nchars != (uint)ncstrp->nchars)
+            DEBUG_RETURN_ERROR(NC_EINTOVERFLOW)
+        err = ncmpix_put_uint32((void**)(&pbp->pos), (uint)ncstrp->nchars);
     }
-    if (status != NC_NOERR) return status;
+    if (err != NC_NOERR) return err;
 
     /* copy namestring */
-    status = ncmpix_pad_putn_text(&pbp->pos, ncstrp->nchars, ncstrp->cp);
-    if (status != NC_NOERR) return status;
+    err = ncmpix_pad_putn_text(&pbp->pos, ncstrp->nchars, ncstrp->cp);
 
-    return NC_NOERR;
+    return err;
 }
 
 /*----< hdr_put_NC_attrV() >-------------------------------------------------*/
@@ -478,9 +478,11 @@ hdr_put_NC_attrV(bufferinfo    *pbp,
     memcpy(pbp->pos, attrp->xvalue, (size_t)sz);
     pbp->pos = (void *)((char *)pbp->pos + sz);
 
-    /* zero-padding is per buffer, not per element */
-    memset(pbp->pos, 0, (size_t)padding);
-    pbp->pos = (void *)((char *)pbp->pos + padding);
+    if (padding > 0) {
+        /* zero-padding is per buffer, not per element */
+        memset(pbp->pos, 0, (size_t)padding);
+        pbp->pos = (void *)((char *)pbp->pos + padding);
+    }
 
     return NC_NOERR;
 }
@@ -497,23 +499,22 @@ hdr_put_NC_dim(bufferinfo   *pbp,
      * NON_NEG    = <non-negative INT> |  // CDF-1 and CDF-2
      *              <non-negative INT64>  // CDF-5
      */
-    int status;
+    int err;
 
     /* copy name */
-    status = hdr_put_NC_name(pbp, dimp->name);
-    if (status != NC_NOERR) return status;
+    err = hdr_put_NC_name(pbp, dimp->name);
+    if (err != NC_NOERR) return err;
 
     /* copy dim_length */
     if (pbp->version == 5)
-        status = ncmpix_put_uint64((void**)(&pbp->pos), dimp->size);
+        err = ncmpix_put_uint64((void**)(&pbp->pos), (uint64)dimp->size);
     else {
         /* TODO: Isn't checking dimension size already done in def_dim()? */
         if (dimp->size != (uint)dimp->size) DEBUG_RETURN_ERROR(NC_EINTOVERFLOW)
-        status = ncmpix_put_uint32((void**)(&pbp->pos), (uint)dimp->size);
+        err = ncmpix_put_uint32((void**)(&pbp->pos), (uint)dimp->size);
     }
-    if (status != NC_NOERR) return status;
 
-    return NC_NOERR;
+    return err;
 }
 
 /*----< hdr_put_NC_dimarray() >----------------------------------------------*/
@@ -555,9 +556,9 @@ hdr_put_NC_dimarray(bufferinfo        *pbp,
 
         /* copy nelems */
         if (pbp->version == 5)
-            status = ncmpix_put_uint64((void**)(&pbp->pos), ncap->ndefined);
+            status = ncmpix_put_uint64((void**)(&pbp->pos), (uint64)ncap->ndefined);
         else
-            status = ncmpix_put_uint32((void**)(&pbp->pos), ncap->ndefined);
+            status = ncmpix_put_uint32((void**)(&pbp->pos), (uint)ncap->ndefined);
         if (status != NC_NOERR) return status;
 
         /* copy [dim ...] */
@@ -590,12 +591,12 @@ hdr_put_NC_attr(bufferinfo    *pbp,
     if (status != NC_NOERR) return status;
 
     /* copy nc_type */
-    status = ncmpix_put_uint32((void**)(&pbp->pos), attrp->type);
+    status = ncmpix_put_uint32((void**)(&pbp->pos), (uint)attrp->type);
     if (status != NC_NOERR) return status;
 
     /* copy nelems */
     if (pbp->version == 5)
-        status = ncmpix_put_uint64((void**)(&pbp->pos), attrp->nelems);
+        status = ncmpix_put_uint64((void**)(&pbp->pos), (uint64)attrp->nelems);
     else {
         if (attrp->nelems != (uint)attrp->nelems) DEBUG_RETURN_ERROR(NC_EINTOVERFLOW)
         status = ncmpix_put_uint32((void**)(&pbp->pos), (uint)attrp->nelems);
@@ -648,9 +649,9 @@ hdr_put_NC_attrarray(bufferinfo         *pbp,
 
         /* copy nelems */
         if (pbp->version == 5)
-            status = ncmpix_put_uint64((void**)(&pbp->pos), ncap->ndefined);
+            status = ncmpix_put_uint64((void**)(&pbp->pos), (uint64)ncap->ndefined);
         else
-            status = ncmpix_put_uint32((void**)(&pbp->pos), ncap->ndefined);
+            status = ncmpix_put_uint32((void**)(&pbp->pos), (uint)ncap->ndefined);
         if (status != NC_NOERR) return status;
 
         /* copy [attr ...] */
@@ -692,17 +693,17 @@ hdr_put_NC_var(bufferinfo   *pbp,
 
     /* copy nelems */
     if (pbp->version == 5)
-        status = ncmpix_put_uint64((void**)(&pbp->pos), varp->ndims);
+        status = ncmpix_put_uint64((void**)(&pbp->pos), (uint64)varp->ndims);
     else
-        status = ncmpix_put_uint32((void**)(&pbp->pos), varp->ndims);
+        status = ncmpix_put_uint32((void**)(&pbp->pos), (uint)varp->ndims);
     if (status != NC_NOERR) return status;
 
     /* copy [dimid ...] */
     for (i=0; i<varp->ndims; i++) {
         if (pbp->version == 5)
-            status = ncmpix_put_uint64((void**)(&pbp->pos), varp->dimids[i]);
+            status = ncmpix_put_uint64((void**)(&pbp->pos), (uint64)varp->dimids[i]);
         else
-            status = ncmpix_put_uint32((void**)(&pbp->pos), varp->dimids[i]);
+            status = ncmpix_put_uint32((void**)(&pbp->pos), (uint)varp->dimids[i]);
         if (status != NC_NOERR) return status;
     }
 
@@ -711,7 +712,7 @@ hdr_put_NC_var(bufferinfo   *pbp,
     if (status != NC_NOERR) return status;
 
     /* copy nc_type */
-    status = ncmpix_put_uint32((void**)(&pbp->pos), varp->type);
+    status = ncmpix_put_uint32((void**)(&pbp->pos), (uint)varp->type);
     if (status != NC_NOERR) return status;
 
     /* copy vsize */
@@ -719,7 +720,7 @@ hdr_put_NC_var(bufferinfo   *pbp,
      * in CDF-5, it is a 64-bit integer
      */
     if (pbp->version == 5)
-        status = ncmpix_put_uint64((void**)(&pbp->pos), varp->len);
+        status = ncmpix_put_uint64((void**)(&pbp->pos), (uint64)varp->len);
     else {
         /* Special case, when there is no record variable, the last fixed-size
          * variable can be larger than 2 GiB if its file starting offset is
@@ -741,7 +742,7 @@ hdr_put_NC_var(bufferinfo   *pbp,
         status = ncmpix_put_uint32((void**)(&pbp->pos), (uint)varp->begin);
     }
     else
-        status = ncmpix_put_uint64((void**)(&pbp->pos), varp->begin);
+        status = ncmpix_put_uint64((void**)(&pbp->pos), (uint64)varp->begin);
     if (status != NC_NOERR) return status;
 
     return NC_NOERR;
@@ -788,9 +789,9 @@ hdr_put_NC_vararray(bufferinfo        *pbp,
 
         /* copy nelems */
         if (pbp->version == 5)
-            status = ncmpix_put_uint64((void**)(&pbp->pos), ncap->ndefined);
+            status = ncmpix_put_uint64((void**)(&pbp->pos), (uint64)ncap->ndefined);
         else
-            status = ncmpix_put_uint32((void**)(&pbp->pos), ncap->ndefined);
+            status = ncmpix_put_uint32((void**)(&pbp->pos), (uint)ncap->ndefined);
         if (status != NC_NOERR) return status;
 
         /* copy [var ...] */
@@ -818,8 +819,6 @@ ncmpii_hdr_put_NC(NC   *ncp,
     putbuf.pos       = buf;
     putbuf.base      = buf;
     putbuf.size      = ncp->xsz;
-    putbuf.put_size  = 0;
-    putbuf.get_size  = 0;
     putbuf.safe_mode = ncp->safe_mode;
 
     /* netCDF file format:
@@ -828,28 +827,29 @@ ncmpii_hdr_put_NC(NC   *ncp,
      */
 
     /* copy "magic", 4 characters */
-    if (ncp->flags & NC_64BIT_DATA) {
+    if (ncp->format == 5) {
         putbuf.version = 5;
-        status = ncmpix_putn_schar_schar(&putbuf.pos, sizeof(ncmagic5), ncmagic5);
+        status = ncmpix_putn_text(&putbuf.pos, sizeof(ncmagic5), ncmagic5);
     }
-    else if (ncp->flags & NC_64BIT_OFFSET) {
+    else if (ncp->format == 2) {
         putbuf.version = 2;
-        status = ncmpix_putn_schar_schar(&putbuf.pos, sizeof(ncmagic2), ncmagic2);
+        status = ncmpix_putn_text(&putbuf.pos, sizeof(ncmagic2), ncmagic2);
     }
     else {
         putbuf.version = 1;
-        status = ncmpix_putn_schar_schar(&putbuf.pos, sizeof(ncmagic1), ncmagic1);
+        status = ncmpix_putn_text(&putbuf.pos, sizeof(ncmagic1), ncmagic1);
     }
     if (status != NC_NOERR) return status;
 
     /* copy numrecs, number of records */
     nrecs = ncp->numrecs;
-    if (ncp->flags & NC_64BIT_DATA)
-        status = ncmpix_put_uint64((void**)(&putbuf.pos), nrecs);
-    else {
+    if (ncp->format < 5) {
         if (nrecs != (uint)nrecs) DEBUG_RETURN_ERROR(NC_EINTOVERFLOW)
         status = ncmpix_put_uint32((void**)(&putbuf.pos), (uint)nrecs);
     }
+    else {
+        status = ncmpix_put_uint64((void**)(&putbuf.pos), (uint64)nrecs);
+    }
     if (status != NC_NOERR) return status;
 
     /* copy dim_list */
@@ -864,9 +864,6 @@ ncmpii_hdr_put_NC(NC   *ncp,
     status = hdr_put_NC_vararray(&putbuf, &ncp->vars);
     if (status != NC_NOERR) return status;
 
-    ncp->nciop->put_size += putbuf.put_size;
-    ncp->nciop->get_size += putbuf.get_size;
-
     return NC_NOERR;
 }
 
@@ -906,7 +903,6 @@ hdr_fetch(bufferinfo *gbp) {
      * all data in the buffer has been consumed */
     if (slack == gbp->size) slack = 0;
 
-    if (gbp->size != (size_t)gbp->size) DEBUG_RETURN_ERROR(NC_EINTOVERFLOW)
     if (gbp->size != (int)gbp->size) DEBUG_RETURN_ERROR(NC_EINTOVERFLOW)
 
     memset(gbp->base, 0, (size_t)gbp->size);
@@ -926,9 +922,9 @@ hdr_fetch(bufferinfo *gbp) {
             if (err == NC_EFILE) DEBUG_ASSIGN_ERROR(err, NC_EREAD)
         }
         else {
-            int get_size;
+            int get_size; /* actual read amount can be smaller */
             MPI_Get_count(&mpistatus, MPI_BYTE, &get_size);
-            gbp->get_size += get_size;
+            gbp->nciop->get_size += get_size;
         }
     }
     /* we might have had to backtrack */
@@ -939,7 +935,7 @@ hdr_fetch(bufferinfo *gbp) {
         if (err != NC_NOERR) return err;
     }
 
-    /* broadcast root's read to other processes */
+    /* broadcast root's read (full or partial header) to other processes */
     TRACE_COMM(MPI_Bcast)(gbp->base, (int)gbp->size, MPI_BYTE, 0, gbp->nciop->comm);
 
     return err;
@@ -1047,27 +1043,28 @@ hdr_get_nc_type(bufferinfo *gbp,
         type != NC_INT64   &&
         type != NC_UINT64
        )
-        DEBUG_RETURN_ERROR(NC_EINVAL)
+        DEBUG_RETURN_ERROR(NC_EBADTYPE)
 
     *typep = (nc_type) type;
     return NC_NOERR;
 }
 
 /*----< ncmpix_len_nctype() >------------------------------------------------*/
+/* return the length of external data type */
 int
 ncmpix_len_nctype(nc_type type) {
     switch(type) {
         case NC_BYTE:
         case NC_CHAR:
         case NC_UBYTE:  return X_SIZEOF_CHAR;
-        case NC_SHORT:
-        case NC_USHORT: return X_SIZEOF_SHORT;
-        case NC_INT:
-        case NC_UINT:   return X_SIZEOF_INT;
+        case NC_SHORT:  return X_SIZEOF_SHORT;
+        case NC_USHORT: return X_SIZEOF_USHORT;
+        case NC_INT:    return X_SIZEOF_INT;
+        case NC_UINT:   return X_SIZEOF_UINT;
         case NC_FLOAT:  return X_SIZEOF_FLOAT;
         case NC_DOUBLE: return X_SIZEOF_DOUBLE;
-        case NC_INT64:
-        case NC_UINT64: return X_SIZEOF_INT64;
+        case NC_INT64:  return X_SIZEOF_INT64;
+        case NC_UINT64: return X_SIZEOF_UINT64;
         default: fprintf(stderr,"ncmpix_len_nctype bad type %d\n",type);
                  assert(0);
     }
@@ -1090,24 +1087,24 @@ hdr_get_NC_name(bufferinfo  *gbp,
      * NON_NEG    = <non-negative INT> |  // CDF-1 and CDF-2
      *              <non-negative INT64>  // CDF-5
      */
-    int status;
-    MPI_Offset  nchars, nbytes, padding, bufremain, strcount;
+    int err;
+    char *cpos;
     NC_string *ncstrp;
-    char *cpos, pad[X_ALIGN-1];
     MPI_Aint pos_addr, base_addr;
+    MPI_Offset  nchars, nbytes, padding, bufremain, strcount;
 
     /* get nelems */
     if (gbp->version == 5) {
         uint64 tmp;
-        status = hdr_get_uint64(gbp, &tmp);
+        err = hdr_get_uint64(gbp, &tmp);
         nchars = (MPI_Offset)tmp;
     }
     else {
         uint tmp;
-        status = hdr_get_uint32(gbp, &tmp);
+        err = hdr_get_uint32(gbp, &tmp);
         nchars = (MPI_Offset)tmp;
     }
-    if (status != NC_NOERR) return status;
+    if (err != NC_NOERR) return err;
 
     /* Allocate a NC_string structure large enough to hold nchars characters */
     ncstrp = ncmpii_new_NC_string((size_t)nchars, NULL);
@@ -1130,7 +1127,8 @@ hdr_get_NC_name(bufferinfo  *gbp,
     while (nbytes > 0) {
         if (bufremain > 0) {
             strcount = MIN(bufremain, nbytes);
-            if (strcount != (size_t)strcount) DEBUG_RETURN_ERROR(NC_EINTOVERFLOW)
+            if (strcount != (size_t)strcount)
+                DEBUG_RETURN_ERROR(NC_EINTOVERFLOW)
             memcpy(cpos, gbp->pos, (size_t)strcount);
             nbytes -= strcount;
             gbp->pos = (void *)((char *)gbp->pos + strcount);
@@ -1138,10 +1136,10 @@ hdr_get_NC_name(bufferinfo  *gbp,
             cpos += strcount;
             bufremain -= strcount;
         } else {
-            status = hdr_fetch(gbp);
-            if (status != NC_NOERR) {
+            err = hdr_fetch(gbp);
+            if (err != NC_NOERR) {
                 ncmpii_free_NC_string(ncstrp);
-                return status;
+                return err;
             }
             bufremain = gbp->size;
         }
@@ -1149,8 +1147,13 @@ hdr_get_NC_name(bufferinfo  *gbp,
 
     /* handle the padding */
     if (padding > 0) {
+        /* CDF specification: Header padding uses null (\x00) bytes. */
+        char pad[X_ALIGN-1];
         memset(pad, 0, X_ALIGN-1);
         if (memcmp(gbp->pos, pad, (size_t)padding) != 0) {
+#ifdef PNETCDF_DEBUG
+            fprintf(stderr,"Error in file %s func %s line %d: NetCDF header non-zero padding found\n",__FILE__,__func__,__LINE__);
+#endif
             ncmpii_free_NC_string(ncstrp);
             DEBUG_RETURN_ERROR(NC_EINVAL)
         }
@@ -1252,11 +1255,22 @@ hdr_get_NC_dimarray(bufferinfo  *gbp,
     /* TODO: we should allow ndefined > 2^32, considering change the data type
      * of ndefined from int to MPI_Offset */
 
+    ncap->unlimited_id = -1;
+
     if (ndefined == 0) {
-        if (type != NC_DIMENSION && type != NC_UNSPECIFIED)
+        if (type != NC_DIMENSION && type != NC_UNSPECIFIED) {
+#ifdef PNETCDF_DEBUG
+            fprintf(stderr,"Error in file %s func %s line %d: NetCDF header format for NC_DIMENSION\n",__FILE__,__func__,__LINE__);
+#endif
             DEBUG_RETURN_ERROR(NC_EINVAL)
+        }
     } else {
-        if (type != NC_DIMENSION) DEBUG_RETURN_ERROR(NC_EINVAL)
+        if (type != NC_DIMENSION) {
+#ifdef PNETCDF_DEBUG
+            fprintf(stderr,"Error in file %s func %s line %d: NetCDF header format for NC_DIMENSION\n",__FILE__,__func__,__LINE__);
+#endif
+            DEBUG_RETURN_ERROR(NC_EINVAL)
+        }
 
         ncap->value = (NC_dim **) NCI_Malloc((size_t)ndefined * sizeof(NC_dim*));
         if (ncap->value == NULL) DEBUG_RETURN_ERROR(NC_ENOMEM)
@@ -1271,6 +1285,8 @@ hdr_get_NC_dimarray(bufferinfo  *gbp,
                 ncmpii_free_NC_dimarray(ncap);
                 return status;
             }
+            if (ncap->value[i]->size == NC_UNLIMITED)
+                ncap->unlimited_id = i; /* ID of unlimited dimension */
         }
     }
 
@@ -1295,9 +1311,7 @@ hdr_get_NC_attrV(bufferinfo *gbp,
      * doubles = [DOUBLE ...]
      * padding = <0, 1, 2, or 3 bytes to next 4-byte boundary>
      */
-    int status;
     void *value = attrp->xvalue;
-    char pad[X_ALIGN-1];
     MPI_Offset nbytes, padding, bufremain, attcount;
     MPI_Aint pos_addr, base_addr;
 
@@ -1316,7 +1330,8 @@ hdr_get_NC_attrV(bufferinfo *gbp,
     while (nbytes > 0) {
         if (bufremain > 0) {
             attcount = MIN(bufremain, nbytes);
-            if (attcount != (size_t)attcount) DEBUG_RETURN_ERROR(NC_EINTOVERFLOW)
+            if (attcount != (size_t)attcount)
+                DEBUG_RETURN_ERROR(NC_EINTOVERFLOW)
             memcpy(value, gbp->pos, (size_t)attcount);
             nbytes -= attcount;
             gbp->pos = (void *)((char *)gbp->pos + attcount);
@@ -1324,16 +1339,24 @@ hdr_get_NC_attrV(bufferinfo *gbp,
             value = (void *)((char *)value + attcount);
             bufremain -= attcount;
         } else {
-            status = hdr_fetch(gbp);
-            if (status != NC_NOERR) return status;
+            int err;
+            err = hdr_fetch(gbp);
+            if (err != NC_NOERR) return err;
             bufremain = gbp->size;
         }
     }
 
     /* handle the padding */
     if (padding > 0) {
+        /* CDF specification: Header padding uses null (\x00) bytes. */
+        char pad[X_ALIGN-1];
         memset(pad, 0, X_ALIGN-1);
-        if (memcmp(gbp->pos, pad, (size_t)padding) != 0) DEBUG_RETURN_ERROR(NC_EINVAL)
+        if (memcmp(gbp->pos, pad, (size_t)padding) != 0) {
+#ifdef PNETCDF_DEBUG
+            fprintf(stderr,"Error in file %s func %s line %d: NetCDF header non-zero padding found\n",__FILE__,__func__,__LINE__);
+#endif
+            DEBUG_RETURN_ERROR(NC_EINVAL)
+        }
         gbp->pos = (void *)((char *)gbp->pos + padding);
         gbp->index += padding;
     }
@@ -1450,10 +1473,19 @@ hdr_get_NC_attrarray(bufferinfo   *gbp,
     ncap->ndefined = (int)ndefined;
 
     if (ndefined == 0) {
-        if (type != NC_ATTRIBUTE && type != NC_UNSPECIFIED)
+        if (type != NC_ATTRIBUTE && type != NC_UNSPECIFIED) {
+#ifdef PNETCDF_DEBUG
+            fprintf(stderr,"Error in file %s func %s line %d: NetCDF header format for NC_ATTRIBUTE\n",__FILE__,__func__,__LINE__);
+#endif
             DEBUG_RETURN_ERROR(NC_EINVAL)
+        }
     } else {
-        if (type != NC_ATTRIBUTE) DEBUG_RETURN_ERROR(NC_EINVAL)
+        if (type != NC_ATTRIBUTE) {
+#ifdef PNETCDF_DEBUG
+            fprintf(stderr,"Error in file %s func %s line %d: NetCDF header format for NC_ATTRIBUTE\n",__FILE__,__func__,__LINE__);
+#endif
+            DEBUG_RETURN_ERROR(NC_EINVAL)
+        }
 
         ncap->value = (NC_attr **)NCI_Malloc((size_t)ndefined * sizeof(NC_attr*));
         if (ncap->value == NULL) DEBUG_RETURN_ERROR(NC_ENOMEM)
@@ -1674,10 +1706,19 @@ hdr_get_NC_vararray(bufferinfo  *gbp,
      * of ndefined from int to MPI_Offset */
 
     if (ndefined == 0) { /* no variable defined */
-        if (type != NC_VARIABLE && type != NC_UNSPECIFIED)
+        if (type != NC_VARIABLE && type != NC_UNSPECIFIED) {
+#ifdef PNETCDF_DEBUG
+            fprintf(stderr,"Error in file %s func %s line %d: NetCDF header format for NC_VARIABLE\n",__FILE__,__func__,__LINE__);
+#endif
             DEBUG_RETURN_ERROR(NC_EINVAL)
+        }
     } else {
-        if (type != NC_VARIABLE) DEBUG_RETURN_ERROR(NC_EINVAL)
+        if (type != NC_VARIABLE) {
+#ifdef PNETCDF_DEBUG
+            fprintf(stderr,"Error in file %s func %s line %d: NetCDF header format for NC_VARIABLE\n",__FILE__,__func__,__LINE__);
+#endif
+            DEBUG_RETURN_ERROR(NC_EINVAL)
+        }
 
         ncap->value = (NC_var **) NCI_Malloc((size_t)ndefined * sizeof(NC_var*));
         if (ncap->value == NULL) DEBUG_RETURN_ERROR(NC_ENOMEM)
@@ -1716,7 +1757,7 @@ ncmpii_hdr_get_NC(NC *ncp)
 {
     int status;
     bufferinfo getbuf;
-    schar magic[sizeof(ncmagic1)];
+    char magic[sizeof(ncmagic1)];
     MPI_Offset nrecs = 0;
     MPI_Aint pos_addr, base_addr;
 
@@ -1725,8 +1766,6 @@ ncmpii_hdr_get_NC(NC *ncp)
     /* Initialize the get buffer that stores the header read from the file */
     getbuf.nciop     = ncp->nciop;
     getbuf.offset    = 0;   /* read from start of the file */
-    getbuf.put_size  = 0;   /* amount of writes so far in bytes */
-    getbuf.get_size  = 0;   /* amount of reads  so far in bytes */
     getbuf.safe_mode = ncp->safe_mode;
 
     /* CDF-5's minimum header size is 4 bytes more than CDF-1 and CDF-2's */
@@ -1746,38 +1785,50 @@ ncmpii_hdr_get_NC(NC *ncp)
 
     /* First get the file format information, magic */
     memset(magic, 0, sizeof(magic));
-    status = ncmpix_getn_schar_schar((const void **)(&getbuf.pos),
-                                     sizeof(magic), magic);
+    status = ncmpix_getn_text((const void **)(&getbuf.pos), sizeof(magic),
+                              magic);
     if (status != NC_NOERR) return status;
     getbuf.index += (MPI_Offset)sizeof(magic);
 
-    /* don't need to worry about CDF-1 or CDF-2
-     * if the first bits are not 'CDF'
-     */
+    /* check if the first three bytes are 'C','D','F' */
     if (memcmp(magic, ncmagic1, sizeof(ncmagic1)-1) != 0) {
+        /* check if is HDF5 file */
+        char signature[8], *hdf5_signature="\211HDF\r\n\032\n";
+        memcpy(signature, magic, 4);
+        ncmpix_getn_text((const void **)(&getbuf.pos), 4, signature+4);
+        if (memcmp(signature, hdf5_signature, 8) == 0) {
+            DEBUG_ASSIGN_ERROR(status, NC_ENOTNC3)
+            if (ncp->safe_mode)
+                fprintf(stderr,"Error: file %s is HDF5 format\n",ncp->nciop->path);
+        }
+        else
+            DEBUG_ASSIGN_ERROR(status, NC_ENOTNC)
         NCI_Free(getbuf.base);
-        DEBUG_RETURN_ERROR(NC_ENOTNC)
+        return status;
     }
 
     /* check version number in last byte of magic */
     if (magic[sizeof(ncmagic1)-1] == 0x1) {
         getbuf.version = 1;
         fSet(ncp->flags, NC_32BIT);
+        ncp->format = 1;
     } else if (magic[sizeof(ncmagic1)-1] == 0x2) {
         getbuf.version = 2;
         fSet(ncp->flags, NC_64BIT_OFFSET);
+        ncp->format = 2;
 #if SIZEOF_MPI_OFFSET < 8
-            /* take the easy way out: if we can't support all CDF-2
-             * files, return immediately */
-            NCI_Free(getbuf.base);
-            DEBUG_RETURN_ERROR(NC_ESMALL)
+        /* take the easy way out: if we can't support all CDF-2
+         * files, return immediately */
+        NCI_Free(getbuf.base);
+        DEBUG_RETURN_ERROR(NC_ESMALL)
 #endif
     } else if (magic[sizeof(ncmagic1)-1] == 0x5) {
         getbuf.version = 5;
         fSet(ncp->flags, NC_64BIT_DATA);
+        ncp->format = 5;
 #if SIZEOF_MPI_OFFSET < 8
-            NCI_Free(getbuf.base);
-            DEBUG_RETURN_ERROR(NC_ESMALL)
+        NCI_Free(getbuf.base);
+        DEBUG_RETURN_ERROR(NC_ESMALL)
 #endif
     } else {
         NCI_Free(getbuf.base);
@@ -1855,9 +1906,6 @@ ncmpii_hdr_get_NC(NC *ncp)
 
     NCI_Free(getbuf.base);
 
-    ncp->nciop->put_size += getbuf.put_size;
-    ncp->nciop->get_size += getbuf.get_size;
-
     return status;
 }
 
@@ -1935,6 +1983,30 @@ ncmpii_comp_dims(int          safe_mode,
 
     local_dim->ndefined = root_dim->ndefined;
 
+#ifndef SEARCH_NAME_LINEARLY
+    if (status != NC_NOERR) {
+        /* dims are not consistent, must rebuild dim name lookup table */
+        for (i=0; i<HASH_TABLE_SIZE; i++) {
+            if (local_dim->nameT[i].num > 0)
+                NCI_Free(local_dim->nameT[i].list);
+            local_dim->nameT[i].num = 0;
+            local_dim->nameT[i].list = NULL;
+        }
+
+        /* populate dim name lookup table */
+        for (i=0; i<local_dim->ndefined; i++) {
+            /* hash the dim name into a key for name lookup */
+            int key = HASH_FUNC(local_dim->value[i]->name->cp);
+            NC_nametable *nameT = &local_dim->nameT[key];
+            if (nameT->num % NC_NAME_TABLE_CHUNK == 0)
+                nameT->list = (int*) NCI_Realloc(nameT->list,
+                              (size_t)(nameT->num+NC_NAME_TABLE_CHUNK) * sizeof(int));
+            nameT->list[nameT->num] = i;
+            nameT->num++;
+        }
+    }
+#endif
+
     return status;
 }
 
@@ -2086,8 +2158,10 @@ ncmpii_comp_attrs(int           safe_mode,
             for (j=0; j<v1->nelems; j++) {
                 /* floating-point inequality here but we genuinely do
                  * expect all processors to set bit-for-bit identical
-                 * headers */
+                 * headers
                 if (fa[j] != fb[j]) {
+                 */
+                if (memcmp(fa+j, fb+j, sizeof(float))) {
                     msg = "%s attribute \"%s\"[%d] FLOAT (root=%f, local=%f)\n";
                     ATTR_WARN_J(msg, name, j, fa[j], fb[j])
                     DEBUG_ASSIGN_ERROR(err, NC_EMULTIDEFINE_ATTR_VAL)
@@ -2101,8 +2175,10 @@ ncmpii_comp_attrs(int           safe_mode,
             for (j=0; j<v1->nelems; j++) {
                 /* floating-point inequality here but we genuinely do
                  * expect all processors to set bit-for-bit identical
-                 * headers */
+                 * headers
                 if (da[j] != db[j]) {
+                 */
+                if (memcmp(da+j, db+j, sizeof(double))) {
                     msg = "%s attribute \"%s\"[%d] DOUBLE (root=%f, local=%f)\n";
                     ATTR_WARN_J(msg, name, j, da[j], db[j])
                     DEBUG_ASSIGN_ERROR(err, NC_EMULTIDEFINE_ATTR_VAL)
@@ -2254,6 +2330,30 @@ ncmpii_comp_vars(int          safe_mode,
 
     local_var->ndefined = root_var->ndefined;
 
+#ifndef SEARCH_NAME_LINEARLY
+    if (status != NC_NOERR) {
+        /* vars are not consistent, must rebuild var name lookup table */
+        for (i=0; i<HASH_TABLE_SIZE; i++) {
+            if (local_var->nameT[i].num > 0)
+                NCI_Free(local_var->nameT[i].list);
+            local_var->nameT[i].num = 0;
+            local_var->nameT[i].list = NULL;
+        }
+
+        /* populate var name lookup table */
+        for (i=0; i<local_var->ndefined; i++) {
+            /* hash the var name into a key for name lookup */
+            int key = HASH_FUNC(local_var->value[i]->name->cp);
+            NC_nametable *nameT = &local_var->nameT[key];
+            if (nameT->num % NC_NAME_TABLE_CHUNK == 0)
+                nameT->list = (int*) NCI_Realloc(nameT->list,
+                              (size_t)(nameT->num+NC_NAME_TABLE_CHUNK) * sizeof(int));
+            nameT->list[nameT->num] = i;
+            nameT->num++;
+        }
+    }
+#endif
+
     return status;
 }
 
@@ -2263,13 +2363,17 @@ ncmpii_comp_vars(int          safe_mode,
  * header object, ncp, with root's header if any inconsistency is detected.
  * This function is called independently and should not contain any MPI
  * communication calls.
+ *
+ * Possible error codes returned:
+ * NC_ENOTNC3, NC_ENOTNC, NC_ESMALL, and all inconsistency errors
+ * NC_EMULTIDEFINE_XXX
  */
 int
 ncmpii_hdr_check_NC(bufferinfo *getbuf, /* header from root */
                     NC         *ncp)
 {
-    int rank, err, status=NC_NOERR;
-    schar magic[sizeof(ncmagic1)];
+    int err, status=NC_NOERR;
+    char magic[sizeof(ncmagic1)]; /* root's file format signature */
     MPI_Offset nrecs=0, chunksize=NC_DEFAULT_CHUNKSIZE;
     MPI_Aint pos_addr, base_addr;
     NC *root_ncp;
@@ -2277,12 +2381,9 @@ ncmpii_hdr_check_NC(bufferinfo *getbuf, /* header from root */
     assert(ncp != NULL);
     assert(getbuf != NULL);
 
-    MPI_Comm_rank(MPI_COMM_WORLD, &rank);
-
-    /* check header's magic */
+    /* check the file format signature in root's header */
     memset(magic, 0, sizeof(magic));
-    err = ncmpix_getn_schar_schar((const void **)(&getbuf->pos),
-                                  sizeof(magic), magic);
+    err = ncmpix_getn_text((const void **)(&getbuf->pos), sizeof(magic), magic);
     if (err != NC_NOERR) {
         /* Fatal error, as root's header is significant */
         if (ncp->safe_mode) fprintf(stderr,"Error: CDF magic number from root's header\n");
@@ -2293,58 +2394,77 @@ ncmpii_hdr_check_NC(bufferinfo *getbuf, /* header from root */
     /* check if the first 3 letters are "CDF" */
     if (memcmp(magic, ncmagic1, sizeof(ncmagic1)-1) != 0) {
         /* Fatal error, as root's header is significant */
-        if (ncp->safe_mode)
-            fprintf(stderr,"Error: root's header indicates not a CDF file\n");
-        DEBUG_RETURN_ERROR(NC_ENOTNC) /* should not continue */
+        /* check if is HDF5 file */
+        char signature[8], *hdf5_signature="\211HDF\r\n\032\n";
+        memcpy(signature, magic, 4);
+        ncmpix_getn_text((const void **)(&getbuf->pos), 4, signature+4);
+        if (memcmp(signature, hdf5_signature, 8) == 0) {
+            DEBUG_ASSIGN_ERROR(status, NC_ENOTNC3)
+            if (ncp->safe_mode)
+                fprintf(stderr,"Error: root's header indicates an HDF5 file\n");
+        }
+        else {
+            DEBUG_ASSIGN_ERROR(status, NC_ENOTNC)
+            if (ncp->safe_mode)
+                fprintf(stderr,"Error: root's header indicates not a CDF file\n");
+        }
+        return status; /* should not continue */
     }
 
     /* allocate a header object and fill it with root's header */
     root_ncp = ncmpii_new_NC(&chunksize);
 
-    /* consistency of magic numbers should have already been checked during
+    /* in safe_mode, consistency of magic numbers have already been checked in
      * ncmpi_create()
      */
     if (magic[sizeof(ncmagic1)-1] == 0x5) {
         fSet(root_ncp->flags, NC_64BIT_DATA);
+        root_ncp->format = 5;
+        getbuf->version = 5;
     }
     else if (magic[sizeof(ncmagic1)-1] == 0x2) {
         fSet(root_ncp->flags, NC_64BIT_OFFSET);
+        root_ncp->format = 2;
+        getbuf->version = 2;
     }
     else if (magic[sizeof(ncmagic1)-1] != 0x1) {
+        getbuf->version = 1;
+        root_ncp->format = 1;
         /* Fatal error, as root's header is significant */
         if (ncp->safe_mode)
             fprintf(stderr,"Error: root's header indicates not CDF 1/2/5 format\n");
         DEBUG_RETURN_ERROR(NC_ENOTNC) /* should not continue */
     }
 
-    /* check version number in last byte of magic */
-    int local_ver, root_ver;
-         if (ncp->flags & NC_64BIT_DATA)   local_ver = 0x5;
-    else if (ncp->flags & NC_64BIT_OFFSET) local_ver = 0x2;
-    else                                   local_ver = 0x1;
+    if (! ncp->safe_mode) {
+        /* check local's version number in last byte of magic against root's */
+        int root_ver = magic[sizeof(ncmagic1)-1];
 
-    root_ver = magic[sizeof(ncmagic1)-1];
-    if (local_ver != root_ver) {
-        if (ncp->safe_mode)
+        if (ncp->format != root_ver) {
+#ifdef PNETCDF_DEBUG
             printf("%s CDF file format (local=CDF-%d, root=CDF-%d)\n",
-                   WARN_STR, local_ver, root_ver);
+                   WARN_STR, ncp->format, root_ver);
+#endif
 
-        /* overwrite the local header object with root's */
-             if (local_ver == 0x5) fClr(ncp->flags, NC_64BIT_DATA);
-        else if (local_ver == 0x2) fClr(ncp->flags, NC_64BIT_OFFSET);
+            /* overwrite the local header object with root's */
+            if      (ncp->format == 5) fClr(ncp->flags, NC_64BIT_DATA);
+            else if (ncp->format == 2) fClr(ncp->flags, NC_64BIT_OFFSET);
 
-             if (root_ver  == 0x5) fSet(ncp->flags, NC_64BIT_DATA);
-        else if (root_ver  == 0x2) fSet(ncp->flags, NC_64BIT_OFFSET);
+            if      (root_ver == 5) fSet(ncp->flags, NC_64BIT_DATA);
+            else if (root_ver == 2) fSet(ncp->flags, NC_64BIT_OFFSET);
 
-        /* this inconsistency is not fatal */
-        DEBUG_ASSIGN_ERROR(status, NC_EMULTIDEFINE_OMODE)
+            /* this inconsistency is not fatal */
+            DEBUG_ASSIGN_ERROR(status, NC_EMULTIDEFINE_CMODE)
+        }
+        getbuf->version = root_ver;
+        ncp->format     = root_ver;
     }
-    getbuf->version = root_ver; /* getbuf's version has not been set before */
 
 #if SIZEOF_MPI_OFFSET < 8
-    if (root_ver > 1) {
+    if (getbuf->version > 1) {
         /* for NC_64BIT_DATA or NC_64BIT_OFFSET, MPI_Offset must be 8 bytes */
-        if (ncp->safe_mode) fprintf(stderr,"Error: cannot support CDF-2 and CDF-5 on this machine\n");
+        if (ncp->safe_mode)
+            fprintf(stderr,"Error: cannot support CDF-2 and CDF-5 on this machine\n");
         DEBUG_RETURN_ERROR(NC_ESMALL) /* should not continue */
     }
 #endif
@@ -2478,8 +2598,14 @@ int ncmpii_write_header(NC *ncp)
     if (NC_indep(ncp))
         fh = ncp->nciop->independent_fh;
 
+    /* update file header size, as this subroutine may be called from a rename
+     * API (var or attribute) and the new name is smaller/bigger which changes
+     * the header size. We recalculate ncp->xsz by getting the un-aligned size
+     * occupied by the file header */
+    ncp->xsz = ncmpii_hdr_len_NC(ncp);
+
     MPI_Comm_rank(ncp->nciop->comm, &rank);
-    if (rank == 0) {
+    if (rank == 0) { /* only root writes to file header */
         MPI_Status mpistatus;
         void *buf = NCI_Malloc((size_t)ncp->xsz); /* header's write buffer */
 
@@ -2499,9 +2625,7 @@ int ncmpii_write_header(NC *ncp)
             }
         }
         else {
-            int put_size;
-            MPI_Get_count(&mpistatus, MPI_BYTE, &put_size);
-            ncp->nciop->put_size += put_size;
+            ncp->nciop->put_size += ncp->xsz;
         }
         NCI_Free(buf);
     }
@@ -2518,9 +2642,6 @@ int ncmpii_write_header(NC *ncp)
         }
     }
 
-    /* update file header size */
-    ncp->xsz = ncmpii_hdr_len_NC(ncp);
-
     if (NC_doFsync(ncp)) { /* NC_SHARE is set */
         TRACE_IO(MPI_File_sync)(fh);
         if (mpireturn != MPI_SUCCESS) {
