@@ -2,10 +2,10 @@
  *  Copyright (C) 2003, Northwestern University and Argonne National Laboratory
  *  See COPYRIGHT notice in top-level directory.
  */
-/* $Id: mpincio.c 2323 2016-02-26 23:04:46Z wkliao $ */
+/* $Id: mpincio.c 2676 2016-12-05 03:11:14Z wkliao $ */
 
 #if HAVE_CONFIG_H
-# include "ncconfig.h"
+# include <ncconfig.h>
 #endif
 
 #ifdef HAVE_ACCESS
@@ -51,6 +51,10 @@
 
 #define MAX_NC_ID 1024
 
+/* ncid corresponds to the index of IDalloc. The content of IDalloc[ncid] is
+ * either 0 or 1. 1 means ncid is corresponds to an opened file. 0 for
+ * otherwise. (not thread-safe)
+ */
 static unsigned char IDalloc[MAX_NC_ID];
 
 inline void
@@ -95,6 +99,9 @@ static
 void ncmpiio_extract_hints(ncio     *nciop,
                            MPI_Info  info)
 {
+    char value[MPI_MAX_INFO_VAL];
+    int  flag;
+
     /* value 0 indicates the hint is not set */
     nciop->hints.h_align                = 0;
     nciop->hints.v_align                = 0;
@@ -106,61 +113,58 @@ void ncmpiio_extract_hints(ncio     *nciop,
 #endif
 
     /* extract NC hints */
-    if (info != MPI_INFO_NULL) {
-        char value[MPI_MAX_INFO_VAL];
-        int  flag;
+    if (info == MPI_INFO_NULL) return;
 
-        MPI_Info_get(info, "nc_header_align_size", MPI_MAX_INFO_VAL-1, value,
-                     &flag);
-        if (flag) nciop->hints.h_align = atoll(value);
+    MPI_Info_get(info, "nc_header_align_size", MPI_MAX_INFO_VAL-1, value,
+                 &flag);
+    if (flag) nciop->hints.h_align = strtoll(value,NULL,10);
 
-        MPI_Info_get(info, "nc_var_align_size",    MPI_MAX_INFO_VAL-1, value,
-                     &flag);
-        if (flag) nciop->hints.v_align = atoll(value);
+    MPI_Info_get(info, "nc_var_align_size",    MPI_MAX_INFO_VAL-1, value,
+                 &flag);
+    if (flag) nciop->hints.v_align = strtoll(value,NULL,10);
 
-        MPI_Info_get(info, "nc_record_align_size", MPI_MAX_INFO_VAL-1,
-                     value, &flag);
-        if (flag) nciop->hints.r_align = atoll(value);
+    MPI_Info_get(info, "nc_record_align_size", MPI_MAX_INFO_VAL-1,
+                 value, &flag);
+    if (flag) nciop->hints.r_align = strtoll(value,NULL,10);
 
-        MPI_Info_get(info, "nc_header_read_chunk_size", MPI_MAX_INFO_VAL-1,
-                     value, &flag);
-        if (flag) nciop->hints.header_read_chunk_size = atoll(value);
+    MPI_Info_get(info, "nc_header_read_chunk_size", MPI_MAX_INFO_VAL-1,
+                 value, &flag);
+    if (flag) nciop->hints.header_read_chunk_size = strtoll(value,NULL,10);
 
 #ifdef ENABLE_SUBFILING
-        MPI_Info_get(info, "pnetcdf_subfiling", MPI_MAX_INFO_VAL-1,
-                     value, &flag);
-        if (flag && strcasecmp(value, "disable") == 0)
-            nciop->hints.subfile_mode = 0;
+    MPI_Info_get(info, "pnetcdf_subfiling", MPI_MAX_INFO_VAL-1,
+                 value, &flag);
+    if (flag && strcasecmp(value, "disable") == 0)
+        nciop->hints.subfile_mode = 0;
 
-        MPI_Info_get(info, "nc_num_subfiles", MPI_MAX_INFO_VAL-1,
-                     value, &flag);
-        if (flag) nciop->hints.num_subfiles = atoi(value);
+    MPI_Info_get(info, "nc_num_subfiles", MPI_MAX_INFO_VAL-1,
+                 value, &flag);
+    if (flag) nciop->hints.num_subfiles = (int)strtol(value,NULL,10);
 #endif
 
-        /* nc_header_align_size, nc_var_align_size, and nciop->hints.r_align
-         * take effect when a file is created or opened and later adding more
-         * header or variable data */
+    /* nc_header_align_size, nc_var_align_size, and nciop->hints.r_align
+     * take effect when a file is created or opened and later adding more
+     * header or variable data */
 
-        if (nciop->hints.h_align < 0)
-            nciop->hints.h_align = 0;
-        if (nciop->hints.v_align < 0)
-            nciop->hints.v_align = 0;
-        if (nciop->hints.r_align < 0)
-            nciop->hints.r_align = 0;
-        if (nciop->hints.header_read_chunk_size < 0)
-            nciop->hints.header_read_chunk_size = 0;
+    if (nciop->hints.h_align < 0)
+        nciop->hints.h_align = 0;
+    if (nciop->hints.v_align < 0)
+        nciop->hints.v_align = 0;
+    if (nciop->hints.r_align < 0)
+        nciop->hints.r_align = 0;
+    if (nciop->hints.header_read_chunk_size < 0)
+        nciop->hints.header_read_chunk_size = 0;
 #ifdef ENABLE_SUBFILING
-        if (nciop->hints.num_subfiles < 0)
-            nciop->hints.num_subfiles = 0;
-        /* override subfile hints if env var is set */
-        char *num_sf_env;
-        num_sf_env = getenv("NC_NUM_SUBFILES");
-        if (num_sf_env != NULL)
-            nciop->hints.num_subfiles = atoi(num_sf_env);
-        if (nciop->hints.subfile_mode == 0)
-            nciop->hints.num_subfiles = 0;
+    if (nciop->hints.num_subfiles < 0)
+        nciop->hints.num_subfiles = 0;
+    /* override subfile hints if env var is set */
+    char *num_sf_env;
+    num_sf_env = getenv("NC_NUM_SUBFILES");
+    if (num_sf_env != NULL)
+        nciop->hints.num_subfiles = (int)strtol(num_sf_env,NULL,10);
+    if (nciop->hints.subfile_mode == 0)
+        nciop->hints.num_subfiles = 0;
 #endif
-    }
 }
 
 /*----< ncmpiio_create() >---------------------------------------------------*/
@@ -175,13 +179,13 @@ ncmpiio_create(MPI_Comm     comm,
     int i, rank, mpireturn, err;
     int mpiomode = MPI_MODE_RDWR | MPI_MODE_CREATE;
 
-    assert(path != NULL);
-    assert(*path != '\0');
     /* checking path consistency is expected done in MPI-IO */
 
     MPI_Comm_rank(comm, &rank);
 
-    /* NC_CLOBBER is the default mode, even if it is not used in cmode */
+    /* NC_CLOBBER is the default mode, even if it is not used in cmode.
+     * Note ioflags has been checked for consistency before entering this API.
+     */
     if (fIsSet(ioflags, NC_NOCLOBBER)) {
         /* check if file exists: NetCDF requires NC_EEXIST returned if the file
          * already exists and NC_NOCLOBBER mode is used in create
@@ -241,16 +245,17 @@ ncmpiio_create(MPI_Comm     comm,
     /* ignore if NC_NOWRITE set by user */
     fSet(ioflags, NC_WRITE);
 
-    nciop = ncmpiio_new(path, ioflags); /* allocate buffer */
-    if (nciop == NULL)
-        DEBUG_RETURN_ERROR(NC_ENOMEM)
+    /* allocate ncio object */
+    nciop = ncmpiio_new(path, ioflags);
+    if (nciop == NULL) DEBUG_RETURN_ERROR(NC_ENOMEM)
 
     nciop->mpiomode  = MPI_MODE_RDWR;
     nciop->mpioflags = 0;
 
-    /* extract MPI-IO hints */
+    /* intialize hints and extract PnetCDF-level hints */
     ncmpiio_extract_hints(nciop, info);
 
+    /* open file in parallel */
     TRACE_IO(MPI_File_open)(comm, (char *)path, mpiomode, info,
                             &nciop->collective_fh);
     if (mpireturn != MPI_SUCCESS) {
@@ -287,7 +292,7 @@ ncmpiio_create(MPI_Comm     comm,
         DEBUG_RETURN_ERROR(NC_ENFILE)
     }
 
-    *((int *)&nciop->fd) = i;
+    nciop->fd = i;
     IDalloc[i] = 1;
 
     /* collective I/O mode is the default mode */
@@ -299,7 +304,9 @@ ncmpiio_create(MPI_Comm     comm,
         return ncmpii_handle_error(mpireturn, "MPI_Comm_dup");
 
     /* get the file info actually used by MPI-IO (maybe alter user's info) */
-    MPI_File_get_info(nciop->collective_fh, &nciop->mpiinfo);
+    mpireturn = MPI_File_get_info(nciop->collective_fh, &nciop->mpiinfo);
+    if (mpireturn != MPI_SUCCESS)
+        return ncmpii_handle_error(mpireturn, "MPI_File_get_info");
 
     ncp->nciop = nciop;
     return NC_NOERR;
@@ -317,8 +324,11 @@ ncmpiio_open(MPI_Comm     comm,
     int i, mpireturn;
     int mpiomode = fIsSet(ioflags, NC_WRITE) ? MPI_MODE_RDWR : MPI_MODE_RDONLY;
 
-    assert(path != NULL);
-    assert(*path != '\0');
+    /* Note ioflags has been checked for consistency before entering this API.
+     */
+
+    assert(ncp != NULL);
+
     /* checking path consistency is expected done in MPI-IO */
 
     /* When open an non-existing file for read, we can either call access() to
@@ -338,40 +348,45 @@ ncmpiio_open(MPI_Comm     comm,
     }
 #endif
 
+    /* allocate ncio object */
     nciop = ncmpiio_new(path, ioflags);
-    if (nciop == NULL)
-        DEBUG_RETURN_ERROR(NC_ENOMEM)
+    if (nciop == NULL) DEBUG_RETURN_ERROR(NC_ENOMEM)
 
     nciop->mpiomode  = mpiomode;
     nciop->mpioflags = 0;
 
-    /* extract MPI-IO hints */
+    /* intialize hints and extract PnetCDF-level hints */
     ncmpiio_extract_hints(nciop, info);
 
-    TRACE_IO(MPI_File_open)(comm, (char *)path, mpiomode,
-                            info, &nciop->collective_fh);
+    /* open file in parallel */
+    TRACE_IO(MPI_File_open)(comm, (char *)path, mpiomode, info,
+                            &nciop->collective_fh);
     if (mpireturn != MPI_SUCCESS) {
         ncmpiio_free(nciop);
         return ncmpii_handle_error(mpireturn, "MPI_File_open");
     }
 
+    /* check if max number of opened files is reached */
     for (i=0; i<MAX_NC_ID && IDalloc[i] != 0; i++);
     if (i == MAX_NC_ID) {
         ncmpiio_free(nciop);
         DEBUG_RETURN_ERROR(NC_ENFILE)
     }
-    *((int *)&nciop->fd) = i;
+    nciop->fd = i;
     IDalloc[i] = 1;
 
+    /* default mode is collective */
     set_NC_collectiveFh(nciop);
 
-    /* duplicate communicator as user may free it later */
+    /* duplicate MPI communicator as user may free it later */
     mpireturn = MPI_Comm_dup(comm, &(nciop->comm));
     if (mpireturn != MPI_SUCCESS)
         return ncmpii_handle_error(mpireturn, "MPI_Comm_dup");
 
     /* get the file info used by MPI-IO */
-    MPI_File_get_info(nciop->collective_fh, &nciop->mpiinfo);
+    mpireturn = MPI_File_get_info(nciop->collective_fh, &nciop->mpiinfo);
+    if (mpireturn != MPI_SUCCESS)
+        return ncmpii_handle_error(mpireturn, "MPI_File_get_info");
 
     ncp->nciop = nciop;
     return NC_NOERR;
@@ -420,7 +435,7 @@ ncmpiio_close(ncio *nciop, int doUnlink) {
         if (mpireturn != MPI_SUCCESS)
             return ncmpii_handle_error(mpireturn, "MPI_File_close");
     }
-    IDalloc[*((int *)&nciop->fd)] = 0;
+    IDalloc[nciop->fd] = 0;
 
     if (doUnlink) {
         TRACE_IO(MPI_File_delete)((char *)nciop->path, nciop->mpiinfo);
@@ -441,7 +456,7 @@ ncmpiio_move(ncio *const nciop,
 {
     int rank, nprocs, bufcount, mpireturn, err, status=NC_NOERR, min_st;
     void *buf;
-    int chunk_size = 1048576; /* move 1 MB per process at a time */
+    int chunk_size=1048576; /* move 1 MB per process at a time */
     MPI_Status mpistatus;
 
     MPI_Comm_size(nciop->comm, &nprocs);
@@ -451,6 +466,8 @@ ncmpiio_move(ncio *const nciop,
      * we use that instead of 1 MB */
     if (nciop->striping_unit > 0) chunk_size = nciop->striping_unit;
 
+    /* buf will be used as a temporal buffer to move data in chunks, i.e.
+     * read a chunk and later write to the new location */
     buf = NCI_Malloc((size_t)chunk_size);
     if (buf == NULL) DEBUG_RETURN_ERROR(NC_ENOMEM)
 
@@ -460,6 +477,8 @@ ncmpiio_move(ncio *const nciop,
 
     /* move the variable starting from its tail toward its beginning */
     while (nbytes > 0) {
+        int get_size=0;
+
         /* calculate how much to move at each time */
         bufcount = chunk_size;
         if (nbytes < (MPI_Offset)nprocs * chunk_size) {
@@ -475,6 +494,9 @@ ncmpiio_move(ncio *const nciop,
             nbytes -= chunk_size*nprocs;
         }
 
+        /* explicitly initialize mpistatus object to 0, see comments below */
+        memset(&mpistatus, 0, sizeof(MPI_Status));
+
         /* read the original data @ from+nbytes+rank*chunk_size */
         TRACE_IO(MPI_File_read_at_all)(nciop->collective_fh,
                                        from+nbytes+rank*chunk_size,
@@ -484,7 +506,12 @@ ncmpiio_move(ncio *const nciop,
             if (err == NC_EFILE) DEBUG_ASSIGN_ERROR(status, NC_EREAD)
         }
         else {
-            int get_size;
+            /* for zero-length read, MPI_Get_count may report incorrect result
+             * for some MPICH version, due to the uninitialized MPI_Status
+             * object passed to MPI-IO calls. Thus we initialize it above to
+             * work around. Otherwise we can just use:
+            nciop->get_size += bufcount;
+             */
             MPI_Get_count(&mpistatus, MPI_BYTE, &get_size);
             nciop->get_size += get_size;
         }
@@ -495,15 +522,37 @@ ncmpiio_move(ncio *const nciop,
         status = min_st;
         if (status != NC_NOERR) break;
 
-        /* write to new location @ to+nbytes+rank*chunk_size */
+        /* write to new location @ to+nbytes+rank*chunk_size
+         *
+         * Ideally, we should write the amount of get_size returned from a call
+         * to MPI_Get_count in the below MPI write. This is in case some
+         * variables are defined but never been written. The value returned by
+         * MPI_Get_count is supposed to be the actual amount read by the MPI
+         * read call. If partial data (or none) is available for read, then we
+         * should just write that amount. Note this MPI write is collective,
+         * and thus all processes must participate the call even if get_size
+         * is 0. However, in some MPICH versions MPI_Get_count fails to report
+         * the correct value due to an internal error that fails to initialize
+         * the MPI_Status object. Therefore, the solution can be either to
+         * explicitly initialize the status object to zeros, or to just use
+         * bufcount for write. Note that the latter will write the variables
+         * that have not been written before. Below uses the former option.
+         */
         TRACE_IO(MPI_File_write_at_all)(nciop->collective_fh,
                                         to+nbytes+rank*chunk_size,
-                                        buf, bufcount, MPI_BYTE, &mpistatus);
+                                        buf, get_size /* bufcount */,
+                                        MPI_BYTE, &mpistatus);
         if (mpireturn != MPI_SUCCESS) {
             err = ncmpii_handle_error(mpireturn, "MPI_File_write_at_all");
             if (err == NC_EFILE) DEBUG_ASSIGN_ERROR(status, NC_EWRITE)
         }
         else {
+            /* for zero-length read, MPI_Get_count may report incorrect result
+             * for some MPICH version, due to the uninitialized MPI_Status
+             * object passed to MPI-IO calls. Thus we initialize it above to
+             * work around. Otherwise we can just use:
+            nciop->put_size += bufcount;
+             */
             int put_size;
             MPI_Get_count(&mpistatus, MPI_BYTE, &put_size);
             nciop->put_size += put_size;
