@@ -2,10 +2,10 @@
  *  Copyright (C) 2003, Northwestern University and Argonne National Laboratory
  *  See COPYRIGHT notice in top-level directory.
  */
-/* $Id: nc.c 2326 2016-03-01 17:21:03Z wkliao $ */
+/* $Id: nc.c 2686 2016-12-07 20:02:41Z wkliao $ */
 
 #if HAVE_CONFIG_H
-# include "ncconfig.h"
+# include <ncconfig.h>
 #endif
 
 #ifdef HAVE_STDLIB_H
@@ -25,11 +25,13 @@
 #include "subfile.h"
 #endif
 
-/* list of open netcdf's */
+/* list of opened netcdf file objects (not thread-safe) */
 static NC *NClist = NULL;
 
-/* This is the default create format for ncmpi_create and nc__create. */
-static int default_create_format = NC_FORMAT_CLASSIC;
+/* This is the default create format for ncmpi_create and nc__create.
+ * The use of this file scope variable is not thread-safe.
+ */
+static int ncmpi_default_create_format = NC_FORMAT_CLASSIC;
 
 /* These have to do with version numbers. */
 #define MAGIC_NUM_LEN 4
@@ -43,7 +45,6 @@ static int default_create_format = NC_FORMAT_CLASSIC;
 static int move_data_r(NC *ncp, NC *old);
 static int move_vars_r(NC *ncp, NC *old);
 static int NC_check_def(MPI_Comm comm, void *buf, MPI_Offset nn);
-static int nc_set_fill(int ncid, int fillmode, int *old_mode_ptr);
 #endif
 
 /*----< ncmpii_add_to_NCList() >---------------------------------------------*/
@@ -82,6 +83,7 @@ ncmpii_del_from_NCList(NC *ncp)
     ncp->prev = NULL;
 }
 
+#ifdef _CHECK_HEADER_IN_DETAIL
 /*----< NC_check_header() >--------------------------------------------------*/
 /*
  * Check the consistency of defined header metadata across all processes and
@@ -93,72 +95,75 @@ NC_check_header(NC         *ncp,
                 void       *buf,
                 MPI_Offset  local_xsz) /* size of buf */
 {
-    void *cmpbuf;
-    int rank, g_status, status=NC_NOERR, mpireturn;
-    bufferinfo gbp;
-
-    MPI_Comm_rank(ncp->nciop->comm, &rank);
+    int h_size, rank, g_status, status=NC_NOERR, mpireturn;
 
     /* root's header size has been broadcasted in NC_begin() and saved in
      * ncp->xsz.
      */
-    if (rank == 0)
-        cmpbuf = buf;
-    else
-        cmpbuf = (void*) NCI_Malloc((size_t)ncp->xsz);
 
-    /* process 0 broadcasts its header
-     * TODO: currently the header size cannot be larger than 2^31 bytes,
+    /* TODO: When root process 0 broadcasts its header,
+     * currently the header size cannot be larger than 2^31 bytes,
      * due to the 2nd argument, count, of MPI_Bcast being of type int.
      * Possible solution is to broadcast in chunks of 2^31 bytes.
      */
-    if (ncp->xsz != (int)ncp->xsz) {
-        if (rank > 0) NCI_Free(cmpbuf);
+    h_size = (int)ncp->xsz;
+    if (ncp->xsz != h_size)
         DEBUG_RETURN_ERROR(NC_EINTOVERFLOW)
+
+    MPI_Comm_rank(ncp->nciop->comm, &rank);
+
+    if (rank == 0) {
+        TRACE_COMM(MPI_Bcast)(buf, h_size, MPI_BYTE, 0, ncp->nciop->comm);
     }
+    else {
+        bufferinfo gbp;
+        void *cmpbuf = (void*) NCI_Malloc((size_t)h_size);
 
-    TRACE_COMM(MPI_Bcast)(cmpbuf, (int)ncp->xsz, MPI_BYTE, 0, ncp->nciop->comm);
+        TRACE_COMM(MPI_Bcast)(cmpbuf, h_size, MPI_BYTE, 0, ncp->nciop->comm);
 
-    if (rank > 0 && (ncp->xsz != local_xsz || memcmp(buf, cmpbuf, (size_t)ncp->xsz))) {
-        /* now part of this process's header is not consistent with root's
-         * check and report the inconsistent part
-         */
+        if (h_size != local_xsz || memcmp(buf, cmpbuf, h_size)) {
+            /* now part of this process's header is not consistent with root's
+             * check and report the inconsistent part
+             */
 
-        gbp.nciop  = ncp->nciop; /* will not be used in ncmpii_hdr_check_NC() */
-        gbp.offset = 0;          /* will not be used in ncmpii_hdr_check_NC() */
-        gbp.size   = ncp->xsz;   /* entire header is in the buffer, cmpbuf */
-        gbp.index  = 0;
-        gbp.pos    = gbp.base = cmpbuf;
+            /* Note that gbp.nciop and gbp.offset below will not be used in
+             * ncmpii_hdr_check_NC() */
+            gbp.nciop  = ncp->nciop;
+            gbp.offset = 0;
+            gbp.size   = h_size;   /* entire header is in the buffer, cmpbuf */
+            gbp.index  = 0;
+            gbp.pos    = gbp.base = cmpbuf;
 
-        /* find the inconsistent part of the header, report the difference, and
-         * overwrite the local header object with root's. ncmpii_hdr_check_NC()
-         * should not have any MPI communication calls.
-         */
-        status = ncmpii_hdr_check_NC(&gbp, ncp);
+            /* find the inconsistent part of the header, report the difference,
+             * and overwrite the local header object with root's.
+             * ncmpii_hdr_check_NC() should not have any MPI communication
+             * calls.
+             */
+            status = ncmpii_hdr_check_NC(&gbp, ncp);
 
-        /* header consistency is only checked on non-root processes. The
-         * returned status can be a fatal error or header inconsistency error,
-         * (fatal errors are due to object allocation), but never NC_NOERR.
-         */
+            /* header consistency is only checked on non-root processes. The
+             * returned status can be a fatal error or header inconsistency
+             * error, (fatal errors are due to object allocation), but never
+             * NC_NOERR.
+             */
+        }
+        NCI_Free(cmpbuf);
     }
 
     if (ncp->safe_mode) {
         TRACE_COMM(MPI_Allreduce)(&status, &g_status, 1, MPI_INT, MPI_MIN,
                                   ncp->nciop->comm);
         if (mpireturn != MPI_SUCCESS) {
-            if (rank > 0) NCI_Free(cmpbuf);
             return ncmpii_handle_error(mpireturn, "MPI_Allreduce"); 
         }
-
         if (g_status != NC_NOERR) { /* some headers are inconsistent */
             if (status == NC_NOERR) DEBUG_ASSIGN_ERROR(status, NC_EMULTIDEFINE)
         }
     }
 
-    if (rank > 0) NCI_Free(cmpbuf);
-
     return status;
 }
+#endif
 
 
 #if 0
@@ -218,7 +223,7 @@ ncmpii_NC_check_id(int   ncid,
     if (ncid >= 0) {
         for (ncp = NClist; ncp != NULL; ncp = ncp->next) {
             if (ncp->nciop->fd == ncid) {
-                *ncpp = ncp;
+                if (ncpp != NULL) *ncpp = ncp;
                 return NC_NOERR; /* normal return */
             }
         }
@@ -260,9 +265,11 @@ inline void
 ncmpii_free_NC(NC *ncp)
 {
     if (ncp == NULL) return;
+
     ncmpii_free_NC_dimarray(&ncp->dims);
     ncmpii_free_NC_attrarray(&ncp->attrs);
     ncmpii_free_NC_vararray(&ncp->vars);
+
     NCI_Free(ncp);
 }
 
@@ -283,42 +290,16 @@ ncmpii_new_NC(const MPI_Offset *chunkp)
 /*----< ncmpi_set_default_format() >-----------------------------------------*/
 /* This function sets a default create file format.
  * Valid formats are NC_FORMAT_CLASSIC, NC_FORMAT_CDF2, and NC_FORMAT_CDF5
- * This API is collective.
+ * This API is NOT collective, as there is no way to check against an MPI
+ * communicator. It should be called by all MPI processes that intend to
+ * create a file later. Consistency check will have to be done in other APIs.
  */
 int
 ncmpi_set_default_format(int format, int *old_formatp)
 {
-    int safe_mode=0, mpireturn, status=NC_NOERR;
-    char *env_str;
-
     /* Return existing format if desired. */
-    if (old_formatp)
-        *old_formatp = default_create_format;
-
-    env_str = getenv("PNETCDF_SAFE_MODE");
-    if (env_str != NULL) {
-        if (*env_str == '0') safe_mode = 0;
-        else                 safe_mode = 1;
-    }
-
-    if (safe_mode) {
-        /* check if format is consistent with root's */
-        int root_format=format;
-
-        TRACE_COMM(MPI_Bcast)(&root_format, 1, MPI_INT, 0, MPI_COMM_WORLD);
-        if (mpireturn != MPI_SUCCESS)
-            return ncmpii_handle_error(mpireturn, "MPI_Bcast");
-
-        if (root_format != format) {
-            int rank;
-            MPI_Comm_rank(MPI_COMM_WORLD, &rank);
-            /* formats are inconsistent, overwrite local format with root's */
-            printf("rank %d: Warning - inconsistent file format, overwrite with root's\n",rank);
-            format = root_format;
-            DEBUG_ASSIGN_ERROR(status, NC_EMULTIDEFINE_OMODE)
-        }
-    }
-
+    if (old_formatp != NULL)
+        *old_formatp = ncmpi_default_create_format;
 
     /* Make sure only valid format is set. */
     if (format != NC_FORMAT_CLASSIC &&
@@ -326,9 +307,9 @@ ncmpi_set_default_format(int format, int *old_formatp)
         format != NC_FORMAT_CDF5) {
         DEBUG_RETURN_ERROR(NC_EINVAL)
     }
-    default_create_format = format;
+    ncmpi_default_create_format = format;
 
-    return status;
+    return NC_NOERR;
 }
 
 /* returns a value suitable for a create flag.  Will return one or more of the
@@ -339,7 +320,7 @@ ncmpi_inq_default_format(int *formatp)
 {
     if (formatp == NULL) DEBUG_RETURN_ERROR(NC_EINVAL)
 
-    *formatp = default_create_format;
+    *formatp = ncmpi_default_create_format;
     return NC_NOERR;
 }
 
@@ -439,27 +420,36 @@ NC_begins(NC         *ncp,
           MPI_Offset  v_minfree,/* free space for fixed variable section */
           MPI_Offset  r_align)  /* alignment for record variable section */
 {
-    int i, j, rank, cdf_format, mpireturn;
+    int i, j, rank, mpireturn;
     MPI_Offset end_var=0;
     NC_var *last = NULL;
     NC_var *first_var = NULL;       /* first "non-record" var */
 
-    /* cdf_format determines the size of variable's "begin" in the header */
-    if (fIsSet(ncp->flags, NC_64BIT_DATA))
-        cdf_format = 5;  /* CDF-5 */
-    else if (fIsSet(ncp->flags, NC_64BIT_OFFSET))
-        cdf_format = 2;  /* CDF-2 */
-    else
-        cdf_format = 1;  /* CDF-1 */
+    /* CDF file format determines the size of variable's "begin" in the header */
 
-    /* get the true header size (un-aligned one) */
+    /* get the true header size (not header extent) */
     MPI_Comm_rank(ncp->nciop->comm, &rank);
-    if (rank ==0) ncp->xsz = ncmpii_hdr_len_NC(ncp);
+    ncp->xsz = ncmpii_hdr_len_NC(ncp);
 
-    /* only root's header size matters */
-    TRACE_COMM(MPI_Bcast)(&ncp->xsz, 1, MPI_OFFSET, 0, ncp->nciop->comm);
-    if (mpireturn != MPI_SUCCESS)
-        return ncmpii_handle_error(mpireturn, "MPI_Bcast"); 
+    if (ncp->safe_mode) { /* this consistency check is redundant as metadata is
+                             kept consistent at all time when safe mode is on */
+        int err, status;
+        MPI_Offset root_xsz = ncp->xsz;
+
+        /* only root's header size matters */
+        TRACE_COMM(MPI_Bcast)(&root_xsz, 1, MPI_OFFSET, 0, ncp->nciop->comm);
+        if (mpireturn != MPI_SUCCESS)
+            return ncmpii_handle_error(mpireturn, "MPI_Bcast"); 
+
+        err = NC_NOERR;
+        if (root_xsz != ncp->xsz) err = NC_EMULTIDEFINE;
+
+        /* find min error code across processes */
+        TRACE_COMM(MPI_Allreduce)(&err, &status, 1, MPI_INT, MPI_MIN, ncp->nciop->comm);
+        if (mpireturn != MPI_SUCCESS)
+            return ncmpii_handle_error(mpireturn, "MPI_Allreduce");
+        if (status != NC_NOERR) DEBUG_RETURN_ERROR(status)
+    }
 
     /* This function is called in ncmpi_enddef(), which can happen either when
      * creating a new file or opening an existing file with metadata modified.
@@ -495,7 +485,7 @@ NC_begins(NC         *ncp,
         if (first_var == NULL) first_var = ncp->vars.value[i];
 
         /* for CDF-1 check if over the file size limit 32-bit integer */
-        if (cdf_format == 1 && end_var > X_OFF_MAX)
+        if (ncp->format == 1 && end_var > X_OFF_MAX)
             DEBUG_RETURN_ERROR(NC_EVARSIZE)
 
         /* this will pad out non-record variables with zero to the
@@ -569,7 +559,7 @@ NC_begins(NC         *ncp,
             continue;
 
         /* X_OFF_MAX is the max of 32-bit integer */
-        if (cdf_format == 1 && end_var > X_OFF_MAX)
+        if (ncp->format == 1 && end_var > X_OFF_MAX)
             DEBUG_RETURN_ERROR(NC_EVARSIZE)
 
         /* A few attempts at aligning record variables have failed
@@ -593,7 +583,7 @@ NC_begins(NC         *ncp,
         end_var += ncp->vars.value[i]->len;
         /* end_var is the end offset of record variable i */
 
-        /* check if record size must fit in 32-bits */
+        /* check if record size must fit in 32-bits (for CDF-1) */
 #if SIZEOF_OFF_T == SIZEOF_SIZE_T && SIZEOF_SIZE_T == 4
         if (ncp->recsize > X_UINT_MAX - ncp->vars.value[i]->len)
             DEBUG_RETURN_ERROR(NC_EVARSIZE)
@@ -639,15 +629,74 @@ NC_begins(NC         *ncp,
 
 #define NC_NUMRECS_OFFSET 4
 
+/*----< ncmpii_write_numrecs() >-----------------------------------------------*/
+/* root process writes the new record number into file.
+ * This function is called by:
+ * 1. ncmpii_sync_numrecs
+ * 2. collective nonblocking wait API, if the new number of records is bigger
+ */
+int
+ncmpii_write_numrecs(NC         *ncp,
+                     MPI_Offset  new_numrecs)
+{
+    int rank, mpireturn, err;
+    MPI_File fh;
+
+    /* root process writes numrecs in file */
+    MPI_Comm_rank(ncp->nciop->comm, &rank);
+    if (rank > 0) return NC_NOERR;
+
+    /* return now if there is no record variabled defined */
+    if (ncp->vars.num_rec_vars == 0) return NC_NOERR;
+
+    fh = ncp->nciop->collective_fh;
+    if (NC_indep(ncp))
+        fh = ncp->nciop->independent_fh;
+
+    if (new_numrecs > ncp->numrecs || NC_ndirty(ncp)) {
+        int len;
+        char pos[8], *buf=pos;
+        MPI_Offset max_numrecs;
+        MPI_Status mpistatus;
+
+        max_numrecs = MAX(new_numrecs, ncp->numrecs);
+
+        if (ncp->format < 5) {
+            if (max_numrecs != (int)max_numrecs)
+                DEBUG_RETURN_ERROR(NC_EINTOVERFLOW)
+            len = X_SIZEOF_SIZE_T;
+            err = ncmpix_put_uint32((void**)&buf, (uint)max_numrecs);
+            if (err != NC_NOERR) DEBUG_RETURN_ERROR(err)
+        }
+        else {
+            len = X_SIZEOF_INT64;
+            err = ncmpix_put_uint64((void**)&buf, (uint64)max_numrecs);
+            if (err != NC_NOERR) DEBUG_RETURN_ERROR(err)
+        }
+        /* ncmpix_put_xxx advances the 1st argument with size len */
+
+        /* root's file view always includes the entire file header */
+        TRACE_IO(MPI_File_write_at)(fh, NC_NUMRECS_OFFSET, (void*)pos, len,
+                                    MPI_BYTE, &mpistatus);
+        if (mpireturn != MPI_SUCCESS) {
+            err = ncmpii_handle_error(mpireturn, "MPI_File_write_at");
+            if (err == NC_EFILE) DEBUG_RETURN_ERROR(NC_EWRITE)
+        }
+        else {
+            ncp->nciop->put_size += len;
+        }
+    }
+    return NC_NOERR;
+}
+
 /*----< ncmpii_sync_numrecs() >-----------------------------------------------*/
 /* Synchronize the number of records in memory and write numrecs to file.
  * This function is called by:
  * 1. ncmpi_sync_numrecs(): by the user
  * 2. ncmpi_sync(): by the user
  * 3. ncmpii_end_indep_data(): exit from independent data mode
- * 4. all blocking collective put APIs (getput.m4) when writing record variable
- * 5. collective nonblocking wait API (ncmpii_wait_getput)
- * 6. ncmpii_close(): file close and currently in independent data mode
+ * 4. all blocking collective put APIs when writing record variables
+ * 5. ncmpii_close(): file close and currently in independent data mode
  *
  * This function is collective.
  */
@@ -655,13 +704,15 @@ int
 ncmpii_sync_numrecs(NC         *ncp,
                     MPI_Offset  new_numrecs)
 {
-    int rank, status=NC_NOERR, mpireturn, err;
-    MPI_File fh;
+    int status=NC_NOERR, mpireturn;
     MPI_Offset max_numrecs;
 
     assert(!NC_readonly(ncp));
     assert(!NC_indef(ncp)); /* can only be called by APIs in data mode */
 
+    /* return now if there is no record variabled defined */
+    if (ncp->vars.num_rec_vars == 0) return NC_NOERR;
+
     /* find the max new_numrecs among all processes
      * Note new_numrecs may be smaller than ncp->numrecs
      */
@@ -670,63 +721,24 @@ ncmpii_sync_numrecs(NC         *ncp,
     if (mpireturn != MPI_SUCCESS)
         return ncmpii_handle_error(mpireturn, "MPI_Allreduce");
 
-    fh = ncp->nciop->collective_fh;
-    if (NC_indep(ncp))
-        fh = ncp->nciop->independent_fh;
-
-    /* root process writes numrecs in file */
-    MPI_Comm_rank(ncp->nciop->comm, &rank);
-    if (rank == 0 && /* Only root process writes to file header */
-        (max_numrecs > ncp->numrecs || NC_ndirty(ncp))) {
-         /* For collective data mode, we check max_numrecs against root's
-          * ncp->numrecs because root's numrecs has not been updated.
-          * For independent data mode, we check NC_ndirty bit, because root's
-          * numrecs may have been updated and in this case NC_ndirty bit has
-          * been set to dirty. */
-        int len;
-        char pos[8], *buf=pos;
-        MPI_Status mpistatus;
-
-        if (ncp->flags & NC_64BIT_DATA) {
-            len = X_SIZEOF_INT64;
-            err = ncmpix_put_uint64((void**)&buf, max_numrecs);
-            if (status == NC_NOERR) status = err;
-        }
-        else {
-            if (max_numrecs != (int)max_numrecs) DEBUG_ASSIGN_ERROR(status, NC_EINTOVERFLOW)
-            len = X_SIZEOF_SIZE_T;
-            err = ncmpix_put_uint32((void**)&buf, (uint)max_numrecs);
-            if (status == NC_NOERR) status = err;
-        }
-        /* ncmpix_put_xxx advances the 1st argument with size len */
-
-        /* root's file view always includes the entire file header */
-
-        TRACE_IO(MPI_File_write_at)(fh, NC_NUMRECS_OFFSET, (void*)pos, len,
-                                    MPI_BYTE, &mpistatus);
-        if (mpireturn != MPI_SUCCESS) {
-            err = ncmpii_handle_error(mpireturn, "MPI_File_write_at");
-            if (status == NC_NOERR && err == NC_EFILE) DEBUG_ASSIGN_ERROR(status, NC_EWRITE)
-        }
-        else {
-            int put_size;
-            MPI_Get_count(&mpistatus, MPI_BYTE, &put_size);
-            ncp->nciop->put_size += put_size;
-        }
-    }
-    /* update numrecs in all processes's memory only if the new one is larger.
-     * Note new_numrecs may be smaller than ncp->numrecs
-     */
-    if (max_numrecs > ncp->numrecs) ncp->numrecs = max_numrecs;
+    /* root process writes max_numrecs to file */
+    status = ncmpii_write_numrecs(ncp, max_numrecs);
 
     if (ncp->safe_mode == 1) {
         /* broadcast root's status, because only root writes to the file */
         int root_status = status;
         TRACE_COMM(MPI_Bcast)(&root_status, 1, MPI_INT, 0, ncp->nciop->comm);
+        if (mpireturn != MPI_SUCCESS)
+            return ncmpii_handle_error(mpireturn, "MPI_Bcast");
         /* root's write has failed, which is serious */
         if (root_status == NC_EWRITE) DEBUG_ASSIGN_ERROR(status, NC_EWRITE)
     }
 
+    /* update numrecs in all processes's memory only if the new one is larger.
+     * Note new_numrecs may be smaller than ncp->numrecs
+     */
+    if (max_numrecs > ncp->numrecs) ncp->numrecs = max_numrecs;
+
     /* clear numrecs dirty bit */
     fClr(ncp->flags, NC_NDIRTY);
 
@@ -768,14 +780,22 @@ static int
 write_NC(NC *ncp)
 {
     void *buf;
-    int status, mpireturn, err, max_err, rank;
+    int status, mpireturn, err, rank;
     MPI_Offset local_xsz;
 
     assert(!NC_readonly(ncp));
 
-    /* ncp->xsz is root's header size, we need to calculate local's */
+    /* In NC_begins(), root's ncp->xsz, root's header size, has been
+     * broadcasted, so ncp->xsz is now root's header size. To check any
+     * inconsistency in file header, we need to calculate local's header
+     * size by calling ncmpii_hdr_len_NC()./
+     */
     local_xsz = ncmpii_hdr_len_NC(ncp);
 
+    /* Note valgrind will complain about uninitialized buf below, but buf will
+     * be first filled with header of size ncp->xsz and later write to file.
+     * So, no need to change to NCI_Calloc for the sake of valgrind.
+     */
     buf = NCI_Malloc((size_t)local_xsz); /* buffer for local header object */
 
     /* copy the entire local header object to buffer */
@@ -785,6 +805,46 @@ write_NC(NC *ncp)
         return status;
     }
 
+    MPI_Comm_rank(ncp->nciop->comm, &rank);
+
+#ifdef _DIFF_HEADER
+    if (ncp->safe_mode == 0) {
+        int h_size=(int)ncp->xsz;
+        void *root_header;
+
+        /* check header against root's */
+        if (rank == 0)
+            root_header = buf;
+        else
+            root_header = (void*) NCI_Malloc((size_t)h_size);
+
+        TRACE_COMM(MPI_Bcast)(root_header, h_size, MPI_BYTE, 0, ncp->nciop->comm);
+        if (mpireturn != MPI_SUCCESS)
+            return ncmpii_handle_error(mpireturn, "MPI_Bcast"); 
+
+        if (rank > 0) {
+            if (h_size != local_xsz || memcmp(buf, root_buf, h_size))
+                status = NC_EMULTIDEFINE;
+            NCI_Free(root_buf);
+        }
+
+        /* report error if header is inconsistency */
+        TRACE_COMM(MPI_Allreduce)(&status, &err, 1, MPI_INT, MPI_MIN, ncp->nciop->comm);
+        if (mpireturn != MPI_SUCCESS) {
+            ncmpii_handle_error(mpireturn,"MPI_Allreduce");
+            DEBUG_RETURN_ERROR(NC_EMPI)
+        }
+        if (err != NC_NOERR) {
+            /* TODO: this error return is harsh. Maybe relax for inconsistent
+             * attribute contents? */
+            if (status == NC_NOERR) status = err;
+            NCI_Free(buf);
+            return status;
+        }
+    }
+#endif
+
+#ifdef _CHECK_HEADER_IN_DETAIL
     /* check the header consistency across all processes and sync header.
      * When safe_mode is on:
      *   The returned status on root can be either NC_NOERR (all headers are
@@ -797,6 +857,7 @@ write_NC(NC *ncp)
      *   error (>-250), or inconsistency error (-250 to -269).
      * For fatal error, we should stop. For others, we can continue.
      */
+    int max_err;
     status = NC_check_header(ncp, buf, local_xsz);
 
     /* check for fatal error */
@@ -811,20 +872,21 @@ write_NC(NC *ncp)
             DEBUG_RETURN_ERROR(NC_EMPI)
         }
     }
-
     if (max_err == 1) { /* some processes encounter a fatal error */
         NCI_Free(buf);
         return status;
     }
+#endif
     /* For non-fatal error, we continue to write header to the file, as now the
      * header object in memory has been sync-ed across all processes. */
 
     /* only rank 0's header gets written to the file */
-    MPI_Comm_rank(ncp->nciop->comm, &rank);
     if (rank == 0) {
         /* rank 0's fileview already includes the file header */
         MPI_Status mpistatus;
-        if (ncp->xsz != (int)ncp->xsz) DEBUG_ASSIGN_ERROR(status, NC_EINTOVERFLOW)
+        if (ncp->xsz != (int)ncp->xsz)
+            DEBUG_ASSIGN_ERROR(status, NC_EINTOVERFLOW)
+
         TRACE_IO(MPI_File_write_at)(ncp->nciop->collective_fh, 0, buf,
                                     (int)ncp->xsz, MPI_BYTE, &mpistatus);
         if (mpireturn != MPI_SUCCESS) {
@@ -833,9 +895,7 @@ write_NC(NC *ncp)
             if (err == NC_EFILE) DEBUG_ASSIGN_ERROR(status, NC_EWRITE)
         }
         else {
-            int put_size;
-            MPI_Get_count(&mpistatus, MPI_BYTE, &put_size);
-            ncp->nciop->put_size += put_size;
+            ncp->nciop->put_size += ncp->xsz;
         }
     }
 
@@ -940,7 +1000,9 @@ move_vars_r(NC *ncp, NC *old) {
 #endif
 
 /*
- * Given a valid ncp, return NC_EVARSIZE if any variable has a bad len
+ * Given a valid ncp, check all variables for their sizes against the maximal
+ * allowable sizes. Different CDF formation versions have different maximal
+ * sizes. This function returns NC_EVARSIZE if any variable has a bad len
  * (product of non-rec dim sizes too large), else return NC_NOERR.
  */
 static int
@@ -950,76 +1012,81 @@ ncmpii_NC_check_vlens(NC *ncp)
     /* maximum permitted variable size (or size of one record's worth
        of a record variable) in bytes.  This is different for format 1
        and format 2. */
-    MPI_Offset vlen_max;
-    MPI_Offset ii;
-    MPI_Offset large_vars_count;
-    MPI_Offset rec_vars_count;
+    MPI_Offset ii, vlen_max, rec_vars_count;
+    MPI_Offset large_fix_vars_count, large_rec_vars_count;
     int last = 0;
 
-    if(ncp->vars.ndefined == 0)
-       return NC_NOERR;
+    if (ncp->vars.ndefined == 0)
+        return NC_NOERR;
 
-    if ((ncp->flags & NC_64BIT_DATA) && SIZEOF_OFF_T > 4)
-       return NC_NOERR;
+    if (ncp->format >= 5) /* CDF-5 */
+        return NC_NOERR;
 
-    if ((ncp->flags & NC_64BIT_OFFSET) && SIZEOF_OFF_T > 4) {
-       /* CDF2 format and LFS */
-       vlen_max = X_UINT_MAX - 3; /* "- 3" handles rounded-up size */
-    } else {
-       /* CDF1 format */
-       vlen_max = X_INT_MAX - 3;
-    }
-    /* Loop through vars, first pass is for non-record variables.   */
-    large_vars_count = 0;
+    /* only CDF-1 and CDF-2 need to continue */
+
+    if (ncp->flags & NC_64BIT_OFFSET) /* CDF2 format */
+        vlen_max = X_UINT_MAX - 3; /* "- 3" handles rounded-up size */
+    else
+        vlen_max = X_INT_MAX - 3; /* CDF1 format */
+
+    /* Loop through vars, first pass is for non-record variables */
+    large_fix_vars_count = 0;
     rec_vars_count = 0;
     vpp = ncp->vars.value;
     for (ii = 0; ii < ncp->vars.ndefined; ii++, vpp++) {
-       if( !IS_RECVAR(*vpp) ) {
-           last = 0;
-           if( ncmpii_NC_check_vlen(*vpp, vlen_max) == 0 ) {
-               large_vars_count++;
-               last = 1;
-           }
-       } else {
-         rec_vars_count++;
-       }
+        if (!IS_RECVAR(*vpp)) {
+            last = 0;
+            if (ncmpii_NC_check_vlen(*vpp, vlen_max) == 0) {
+                /* check this variable's shape product against vlen_max */
+                large_fix_vars_count++;
+                last = 1;
+            }
+        } else {
+            rec_vars_count++;
+        }
     }
     /* OK if last non-record variable size too large, since not used to
        compute an offset */
-    if( large_vars_count > 1) { /* only one "too-large" variable allowed */
-      DEBUG_RETURN_ERROR(NC_EVARSIZE)
+    if (large_fix_vars_count > 1)  /* only one "too-large" variable allowed */
+        DEBUG_RETURN_ERROR(NC_EVARSIZE)
+
+    /* The only "too-large" variable must be the last one defined */
+    if (large_fix_vars_count == 1 && last == 0)
+        DEBUG_RETURN_ERROR(NC_EVARSIZE)
+
+    if (rec_vars_count == 0) return NC_NOERR;
+
+    /* if there is a "too-large" fixed-size variable, no record variable is
+     * allowed */
+    if (large_fix_vars_count == 1)
+        DEBUG_RETURN_ERROR(NC_EVARSIZE)
+
+    /* Loop through vars, second pass is for record variables.   */
+    large_rec_vars_count = 0;
+    vpp = ncp->vars.value;
+    for (ii = 0; ii < ncp->vars.ndefined; ii++, vpp++) {
+        if (IS_RECVAR(*vpp)) {
+            last = 0;
+            if (ncmpii_NC_check_vlen(*vpp, vlen_max) == 0) {
+                /* check this variable's shape product against vlen_max */
+                large_rec_vars_count++;
+                last = 1;
+            }
+        }
     }
+
+    /* For CDF-2, no record variable can require more than 2^32 - 4 bytes of
+     * storage for each record's worth of data, unless it is the last record
+     * variable. See
+     * http://www.unidata.ucar.edu/software/netcdf/docs/file_structure_and_performance.html#offset_format_limitations
+     */
+    if (large_rec_vars_count > 1)  /* only one "too-large" variable allowed */
+        DEBUG_RETURN_ERROR(NC_EVARSIZE)
+
     /* and it has to be the last one */
-    if( large_vars_count == 1 && last == 0) {
-      DEBUG_RETURN_ERROR(NC_EVARSIZE)
-    }
-    if( rec_vars_count > 0 ) {
-       /* and if it's the last one, there can't be any record variables */
-       if( large_vars_count == 1 && last == 1) {
-           DEBUG_RETURN_ERROR(NC_EVARSIZE)
-       }
-       /* Loop through vars, second pass is for record variables.   */
-       large_vars_count = 0;
-       vpp = ncp->vars.value;
-       for (ii = 0; ii < ncp->vars.ndefined; ii++, vpp++) {
-           if( IS_RECVAR(*vpp) ) {
-               last = 0;
-               if( ncmpii_NC_check_vlen(*vpp, vlen_max) == 0 ) {
-                   large_vars_count++;
-                   last = 1;
-               }
-           }
-       }
-       /* OK if last record variable size too large, since not used to
-          compute an offset */
-       if( large_vars_count > 1) { /* only one "too-large" variable allowed */
-           DEBUG_RETURN_ERROR(NC_EVARSIZE)
-       }
-       /* and it has to be the last one */
-       if( large_vars_count == 1 && last == 0) {
-           DEBUG_RETURN_ERROR(NC_EVARSIZE)
-       }
-    }
+    if (large_rec_vars_count == 1 && last == 0)
+        DEBUG_RETURN_ERROR(NC_EVARSIZE)
+
     return NC_NOERR;
 }
 
@@ -1099,7 +1166,7 @@ ncmpii_NC_enddef(NC         *ncp,
     }
 #endif
 
-    /* check on dimension lengths */
+    /* check whether sizes of all variables are legal */
     status = ncmpii_NC_check_vlens(ncp);
     CHECK_ERROR(status)
 
@@ -1108,6 +1175,9 @@ ncmpii_NC_enddef(NC         *ncp,
      * file offset as well as the offsets of record variables.
      * When ncp->old != NULL, this enddef is called after a redef. In this
      * case, we re-used all variable offsets as many as possible.
+     *
+     * Note in NC_begins, root broadcasts ncp->xsz, the file header size, to
+     * all processes.
      */
     status = NC_begins(ncp, h_align, h_minfree, v_align, v_minfree, r_align);
     CHECK_ERROR(status)
@@ -1116,7 +1186,7 @@ ncmpii_NC_enddef(NC         *ncp,
     if (ncp->nc_num_subfiles > 1) {
         /* get ncp info for the subfile */
         status = ncmpii_NC_check_id(ncp->ncid_sf, &ncp_sf);
-        CHECK_ERROR(status)
+        if (status != NC_NOERR) DEBUG_RETURN_ERROR(status)
 
         status = NC_begins(ncp_sf, h_align, h_minfree, v_align, v_minfree,
                            r_align);
@@ -1160,8 +1230,8 @@ ncmpii_NC_enddef(NC         *ncp,
     } /* ... ncp->old != NULL */
 
     /* first sync header objects in memory across all processes, and then root
-     * writes the header to file. Note safe_mode error check is already done
-     * in write_NC() */
+     * writes the header to file. Note safe_mode error check will be done in
+     * write_NC() */
     status = write_NC(ncp);
 
     /* we should continue to exit define mode, even if header is inconsistent
@@ -1184,8 +1254,10 @@ ncmpii_NC_enddef(NC         *ncp,
         ncp->vars.num_rec_vars += IS_RECVAR(ncp->vars.value[i]);
 
     /* fill variables according to their fill mode settings */
-    err = ncmpii_fill_vars(ncp);
-    if (status == NC_NOERR) status = err;
+    if (ncp->vars.ndefined > 0) {
+        err = ncmpii_fill_vars(ncp);
+        if (status == NC_NOERR) status = err;
+    }
 
     if (ncp->old != NULL) {
         ncmpii_free_NC(ncp->old);
@@ -1218,6 +1290,9 @@ ncmpii_inq_env_align_hints(MPI_Offset *h_align,
                            MPI_Offset *h_chunk,
                            MPI_Offset *r_align)
 {
+    /* TODO: env variables have been checked in ncmpi_create and ncmpi_open
+     * Maybe this can be part of that
+     */
     char *env_str;
 
     *h_align = 0;
@@ -1226,21 +1301,26 @@ ncmpii_inq_env_align_hints(MPI_Offset *h_align,
     *r_align = 0;
 
     if ((env_str = getenv("PNETCDF_HINTS")) != NULL) {
-        char *key = strtok(env_str, ";");
+        char *env_str_cpy, *key;
+        env_str_cpy = (char*) NCI_Malloc(strlen(env_str)+1);
+        strcpy(env_str_cpy, env_str);
+        key = strtok(env_str_cpy, ";");
         while (key != NULL) {
             char *val = strchr(key, '=');
+            if (val == NULL) continue; /* ill-formed hint */
             *val = '\0';
             val++;
             if (strcasecmp(key, "nc_header_align_size") == 0)
-                *h_align = atoll(val);
+                *h_align = strtoll(val,NULL,10);
             else if (strcasecmp(key, "nc_var_align_size") == 0)
-                *v_align = atoll(val);
+                *v_align = strtoll(val,NULL,10);
             else if (strcasecmp(key, "nc_header_read_chunk_size") == 0)
-                *h_chunk = atoll(val);
+                *h_chunk = strtoll(val,NULL,10);
             else if (strcasecmp(key, "nc_record_align_size") == 0)
-                *r_align = atoll(val);
+                *r_align = strtoll(val,NULL,10);
             key = strtok(NULL, ";");
         }
+        NCI_Free(env_str_cpy);
     }
     return 1;
 }
@@ -1266,7 +1346,7 @@ ncmpii_enddef(NC *ncp)
     MPI_Info_get(ncp->nciop->mpiinfo, "striping_unit", MPI_MAX_INFO_VAL-1,
                  value, &flag);
     striping_unit = 0;
-    if (flag) striping_unit = atoi(value);
+    if (flag) striping_unit = (int)strtol(value,NULL,10);
     ncp->nciop->striping_unit = striping_unit;
 
     all_var_size = 0;  /* sum of all defined variables */
@@ -1343,7 +1423,7 @@ ncmpii__enddef(NC         *ncp,
     MPI_Info_get(ncp->nciop->mpiinfo, "striping_unit", MPI_MAX_INFO_VAL-1,
                  value, &flag);
     striping_unit = 0;
-    if (flag) striping_unit = atoi(value);
+    if (flag) striping_unit = (int)strtol(value,NULL,10);
     ncp->nciop->striping_unit = striping_unit;
 
     all_var_size = 0;  /* sum of all defined variables */
@@ -1438,13 +1518,31 @@ ncmpii_close(NC *ncp)
 
     /* We can cancel or complete all outstanding nonblocking I/O.
      * For now, cancelling makes more sense. */
-    if (ncp->head != NULL) {
 #ifdef COMPLETE_NONBLOCKING_IO
-        ncmpii_wait(ncp, INDEP_IO, NC_REQ_ALL, NULL, NULL);
-#else
-        ncmpii_cancel(ncp, NC_REQ_ALL, NULL, NULL);
-#endif
+    if (ncp->numGetReqs > 0) {
+        ncmpii_wait(ncp, INDEP_IO, NC_GET_REQ_ALL, NULL, NULL);
+        if (status == NC_NOERR ) status = NC_EPENDING;
     }
+    if (ncp->numPutReqs > 0) {
+        ncmpii_wait(ncp, INDEP_IO, NC_PUT_REQ_ALL, NULL, NULL);
+        if (status == NC_NOERR ) status = NC_EPENDING;
+    }
+#else
+    if (ncp->numGetReqs > 0) {
+        int rank;
+        MPI_Comm_rank(ncp->nciop->comm, &rank);
+        printf("PnetCDF warning: %d nonblocking get requests still pending on process %d. Cancelling ...\n",ncp->numGetReqs,rank);
+        ncmpii_cancel(ncp, NC_GET_REQ_ALL, NULL, NULL);
+        if (status == NC_NOERR ) status = NC_EPENDING;
+    }
+    if (ncp->numPutReqs > 0) {
+        int rank;
+        MPI_Comm_rank(ncp->nciop->comm, &rank);
+        printf("PnetCDF warning: %d nonblocking put requests still pending on process %d. Cancelling ...\n",ncp->numPutReqs,rank);
+        ncmpii_cancel(ncp, NC_PUT_REQ_ALL, NULL, NULL);
+        if (status == NC_NOERR ) status = NC_EPENDING;
+    }
+#endif
 
     /* If the user wants a stronger data consistency by setting NC_SHARE */
     if (fIsSet(ncp->nciop->ioflags, NC_SHARE))
@@ -1476,8 +1574,7 @@ ncmpi_inq(int  ncid,
     NC *ncp;
 
     status = ncmpii_NC_check_id(ncid, &ncp);
-    if (status != NC_NOERR)
-        return status;
+    if (status != NC_NOERR) DEBUG_RETURN_ERROR(status)
 
     if (ndimsp != NULL)
         *ndimsp = (int) ncp->dims.ndefined;
@@ -1486,7 +1583,8 @@ ncmpi_inq(int  ncid,
     if (nattsp != NULL)
         *nattsp = (int) ncp->attrs.ndefined;
     if (xtendimp != NULL)
-        *xtendimp = ncmpii_find_NC_Udim(&ncp->dims, NULL);
+        /* *xtendimp = ncmpii_find_NC_Udim(&ncp->dims, NULL); */
+        *xtendimp = ncp->dims.unlimited_id;
 
     return NC_NOERR;
 }
@@ -1499,12 +1597,11 @@ ncmpi_inq_version(int ncid, int *nc_mode)
     NC *ncp;
 
     status = ncmpii_NC_check_id(ncid, &ncp);
-    if (status != NC_NOERR)
-        return status;
+    if (status != NC_NOERR) DEBUG_RETURN_ERROR(status)
 
-    if (fIsSet(ncp->flags, NC_64BIT_DATA))
+    if (ncp->format == 5)
         *nc_mode = NC_64BIT_DATA;
-    else if (fIsSet(ncp->flags, NC_64BIT_OFFSET))
+    else if (ncp->format == 2)
         *nc_mode = NC_64BIT_OFFSET;
     else
         *nc_mode = NC_CLASSIC_MODEL;
@@ -1520,7 +1617,7 @@ ncmpi_inq_ndims(int ncid, int *ndimsp)
     NC *ncp;
 
     status = ncmpii_NC_check_id(ncid, &ncp);
-    if (status != NC_NOERR) return status;
+    if (status != NC_NOERR) DEBUG_RETURN_ERROR(status)
 
     if (ndimsp != NULL)
         *ndimsp = (int) ncp->dims.ndefined;
@@ -1535,7 +1632,7 @@ ncmpi_inq_nvars(int ncid, int *nvarsp)
     NC *ncp;
 
     status = ncmpii_NC_check_id(ncid, &ncp);
-    if (status != NC_NOERR) return status;
+    if (status != NC_NOERR) DEBUG_RETURN_ERROR(status)
 
     if (nvarsp != NULL)
         *nvarsp = (int) ncp->vars.ndefined;
@@ -1550,7 +1647,7 @@ ncmpi_inq_natts(int ncid, int *nattsp)
     NC *ncp;
 
     status = ncmpii_NC_check_id(ncid, &ncp);
-    if (status != NC_NOERR) return status;
+    if (status != NC_NOERR) DEBUG_RETURN_ERROR(status)
 
     if (nattsp != NULL)
         *nattsp = (int) ncp->attrs.ndefined;
@@ -1565,10 +1662,12 @@ ncmpi_inq_unlimdim(int ncid, int *xtendimp)
     NC *ncp;
 
     status = ncmpii_NC_check_id(ncid, &ncp);
-    if (status != NC_NOERR) return status;
+    if (status != NC_NOERR) DEBUG_RETURN_ERROR(status)
 
+    /* TODO: it makes no sense xtendimp being NULL */
     if (xtendimp != NULL)
-        *xtendimp = ncmpii_find_NC_Udim(&ncp->dims, NULL);
+        /* *xtendimp = ncmpii_find_NC_Udim(&ncp->dims, NULL); */
+        *xtendimp = ncp->dims.unlimited_id;
 
     return NC_NOERR;
 }
@@ -1582,8 +1681,7 @@ ncmpi_inq_num_rec_vars(int ncid, int *nvarsp)
 
     /* get ncp object */
     status = ncmpii_NC_check_id(ncid, &ncp);
-    if (status != NC_NOERR)
-        return status;
+    if (status != NC_NOERR) DEBUG_RETURN_ERROR(status)
 
     if (nvarsp != NULL) {
         if (NC_indef(ncp)) {
@@ -1608,8 +1706,7 @@ ncmpi_inq_num_fix_vars(int ncid, int *nvarsp)
 
     /* get ncp object */
     status = ncmpii_NC_check_id(ncid, &ncp);
-    if (status != NC_NOERR)
-        return status;
+    if (status != NC_NOERR) DEBUG_RETURN_ERROR(status)
 
     if (nvarsp != NULL) {
         if (NC_indef(ncp)) {
