@@ -2,7 +2,7 @@
  *  Copyright (C) 2003, Northwestern University and Argonne National Laboratory
  *  See COPYRIGHT notice in top-level directory.
  */
-/* $Id: nc.h 2319 2016-02-04 08:04:01Z wkliao $ */
+/* $Id: nc.h 2723 2016-12-18 06:21:12Z wkliao $ */
 #ifndef _NC_H_
 #define _NC_H_
 
@@ -70,7 +70,7 @@ enum API_KIND {
 #endif
 
 /* ncmpi_create/ncmpi_open set up header to be 'chunksize' big and to grow
- * by 'chunksize' as new items adde. This used to be 4k. 256k lets us read
+ * by 'chunksize' as new items added. This used to be 4k. 256k lets us read
  * in an entire climate header in one go */
 #define NC_DEFAULT_CHUNKSIZE 262144
 
@@ -115,9 +115,9 @@ typedef enum {
     NC_UNSPECIFIED =  0,
 /*  NC_BITFIELD    =  7, */
 /*  NC_STRING      =  8, */
-    NC_DIMENSION   = 10,
-    NC_VARIABLE    = 11,
-    NC_ATTRIBUTE   = 12
+    NC_DIMENSION   = 10,  /* \x00 \x00 \x00 \x0A */
+    NC_VARIABLE    = 11,  /* \x00 \x00 \x00 \x0B */
+    NC_ATTRIBUTE   = 12   /* \x00 \x00 \x00 \x0C */
 } NCtype;
 
 
@@ -165,14 +165,46 @@ typedef struct {
 #endif
 } NC_dim;
 
+#define NC_NAME_TABLE_CHUNK 16
+#define HASH_TABLE_SIZE 256
+/*
+#define HASH_FUNC(x) ncmpii_jenkins_one_at_a_time_hash(x)
+#define HASH_FUNC(x) ncmpii_additive_hash(x)
+#define HASH_FUNC(x) ncmpii_rotating_hash(x)
+#define HASH_FUNC(x) ncmpii_Pearson_hash(x)
+*/
+#define HASH_FUNC(x) ncmpii_Bernstein_hash(x)
+/* Look like Bernstein's hashing function performs the best */
+
+/* For the initial naive implementation of hashing:
+ * #define HASH_FUNC(x) (unsigned char)x[0]
+ * if used this simple hash function, HASH_TABLE_SIZE must be 256 which is the
+ * number of possible keys can be stored in an unsigned char
+ */
+
+typedef struct NC_nametable {
+    int  num;  /* number of elements added in the list array. Its value starts
+                  with zero and incremented with new name is created. When its
+                  value becomes a multiple of NC_NAME_TABLE_CHUNK, list will be
+                  reallocated to a space of size (num + NC_NAME_TABLE_CHUNK) */
+    int *list; /* dimension or variable IDs */
+} NC_nametable;
+
 /* the dimension ID returned from ncmpi_def_dim() is an integer pointer
  * which means the total number of defined dimension allowed in a file
  * is up to 2^31-1. Thus, the member ndefined below should be of type int.
  */
 typedef struct NC_dimarray {
-    int      nalloc;    /* number allocated >= ndefined */
-    int      ndefined;  /* number of defined dimensions */
-    NC_dim **value;
+    int            nalloc;        /* number allocated >= ndefined */
+    int            ndefined;      /* number of defined dimensions */
+    int            unlimited_id;  /* -1 for not defined, otherwise >= 0 */
+    NC_dim       **value;
+    NC_nametable   nameT[HASH_TABLE_SIZE]; /* table for quick name lookup.
+                    * indices 0, 1, ... HASH_TABLE_SIZE-1 are the hash keys.
+                    * nameT[i].num is the number of hash collisions. The IDs of
+                    * dimensions with names producing the same hash key i are
+                    * stored in nameT[i].list[*]
+                    */
 } NC_dimarray;
 
 /* Begin defined in dim.c */
@@ -203,23 +235,6 @@ ncmpii_dup_NC_dimarray(NC_dimarray *ncap, const NC_dimarray *ref);
 extern NC_dim *
 ncmpii_elem_NC_dimarray(const NC_dimarray *ncap, int elem);
 
-extern int
-ncmpi_def_dim(int ncid, const char *name, MPI_Offset size, int *dimidp);
-
-extern int
-ncmpi_rename_dim( int ncid, int dimid, const char *newname);
-
-extern int
-ncmpi_inq_dimid(int ncid, const char *name, int *dimid_ptr);
-
-extern int
-ncmpi_inq_dim(int ncid, int dimid, char *name, MPI_Offset *sizep);
-
-extern int
-ncmpi_inq_dimname(int ncid, int dimid, char *name);
-
-extern int
-ncmpi_inq_dimlen(int ncid, int dimid, MPI_Offset *lenp);
 /* End defined in dim.c */
 
 /*
@@ -271,84 +286,6 @@ ncmpii_dup_NC_attrarray(NC_attrarray *ncap, const NC_attrarray *ref);
 extern NC_attr *
 ncmpii_elem_NC_attrarray(const NC_attrarray *ncap, MPI_Offset elem);
 
-extern int
-ncmpi_put_att_text(int ncid, int varid, const char *name,
-        MPI_Offset nelems, const char *value);
-
-extern int
-ncmpi_get_att_text(int ncid, int varid, const char *name, char *str);
-
-extern int
-ncmpi_put_att_schar(int ncid, int varid, const char *name,
-        nc_type type, MPI_Offset nelems, const signed char *value);
-
-extern int
-ncmpi_get_att_schar(int ncid, int varid, const char *name, signed char *tp);
-
-extern int
-ncmpi_put_att_uchar(int ncid, int varid, const char *name,
-        nc_type type, MPI_Offset nelems, const unsigned char *value);
-
-extern int
-ncmpi_get_att_uchar(int ncid, int varid, const char *name, unsigned char *tp);
-
-extern int
-ncmpi_put_att_short(int ncid, int varid, const char *name,
-        nc_type type, MPI_Offset nelems, const short *value);
-
-extern int
-ncmpi_get_att_short(int ncid, int varid, const char *name, short *tp);
-
-extern int
-ncmpi_put_att_int(int ncid, int varid, const char *name,
-        nc_type type, MPI_Offset nelems, const int *value);
-
-extern int
-ncmpi_get_att_int(int ncid, int varid, const char *name, int *tp);
-
-extern int
-ncmpi_put_att_long(int ncid, int varid, const char *name,
-        nc_type type, MPI_Offset nelems, const long *value);
-
-extern int
-ncmpi_get_att_long(int ncid, int varid, const char *name, long *tp);
-
-extern int
-ncmpi_put_att_float(int ncid, int varid, const char *name,
-        nc_type type, MPI_Offset nelems, const float *value);
-extern int
-ncmpi_get_att_float(int ncid, int varid, const char *name, float *tp);
-extern int
-ncmpi_put_att_double(int ncid, int varid, const char *name,
-        nc_type type, MPI_Offset nelems, const double *value);
-extern int
-ncmpi_get_att_double(int ncid, int varid, const char *name, double *tp);
-
-extern int
-ncmpi_inq_attid(int ncid, int varid, const char *name, int *attnump);
-
-extern int
-ncmpi_inq_atttype(int ncid, int varid, const char *name, nc_type *datatypep);
-
-extern int
-ncmpi_inq_attlen(int ncid, int varid, const char *name, MPI_Offset *lenp);
-
-extern int
-ncmpi_inq_att(int ncid, int varid, const char *name,
-        nc_type *datatypep, MPI_Offset *lenp);
-
-extern int
-ncmpi_copy_att(int ncid_in, int varid_in, const char *name,
-        int ncid_out, int ovarid);
-
-extern int
-ncmpi_rename_att( int ncid, int varid, const char *name, const char *newname);
-
-extern int
-ncmpi_del_att(int ncid, int varid, const char *name);
-
-extern int
-ncmpi_inq_attname(int ncid, int varid, int attnum, char *name);
 /* End defined in attr.c */
 
 /*
@@ -377,10 +314,16 @@ typedef struct {
 
 /* note: we only allow less than 2^31-1 variables defined in a file */
 typedef struct NC_vararray {
-    int      nalloc;      /* number allocated >= ndefined */
-    int      ndefined;    /* number of defined variables */
-    int      num_rec_vars;/* number of defined record variables */
-    NC_var **value;
+    int            nalloc;      /* number allocated >= ndefined */
+    int            ndefined;    /* number of defined variables */
+    int            num_rec_vars;/* number of defined record variables */
+    NC_var       **value;
+    NC_nametable   nameT[HASH_TABLE_SIZE]; /* table for quick name lookup.
+                    * indices 0, 1, ... HASH_TABLE_SIZE-1 are the hash keys.
+                    * nameT[i].num is the number of hash collisions. The IDs of
+                    * variables with names producing the same hash key i are
+                    * stored in nameT[i].list[*]
+                    */
 } NC_vararray;
 
 /* Begin defined in var.c */
@@ -406,7 +349,7 @@ extern int
 ncmpii_dup_NC_vararray(NC_vararray *ncap, const NC_vararray *ref);
 
 extern int
-ncmpii_NC_var_shape64(NC *ncp, NC_var *varp, const NC_dimarray *dims);
+ncmpii_NC_var_shape64(NC_var *varp, const NC_dimarray *dims);
 
 extern int
 ncmpii_NC_check_vlen(NC_var *varp, MPI_Offset vlen_max);
@@ -414,37 +357,6 @@ ncmpii_NC_check_vlen(NC_var *varp, MPI_Offset vlen_max);
 extern int
 ncmpii_NC_lookupvar(NC *ncp, int varid, NC_var **varp);
 
-extern int
-ncmpi_def_var(int ncid, const char *name, nc_type type,
-        int ndims, const int *dimidsp, int *varidp);
-
-extern int
-ncmpi_rename_var(int ncid, int varid, const char *newname);
-
-extern int
-ncmpi_inq_var(int ncid, int varid, char *name, nc_type *typep,
-        int *ndimsp, int *dimids, int *nattsp);
-
-extern int
-ncmpi_inq_varid(int ncid, const char *name, int *varid_ptr);
-
-extern int
-ncmpi_inq_varname(int ncid, int varid, char *name);
-
-extern int
-ncmpi_inq_vartype(int ncid, int varid, nc_type *typep);
-
-extern int
-ncmpi_inq_varndims(int ncid, int varid, int *ndimsp);
-
-extern int
-ncmpi_inq_vardimid(int ncid, int varid, int *dimids);
-
-extern int
-ncmpi_inq_varnatts(int ncid, int varid, int *nattsp);
-
-extern int
-ncmpi_rename_var(int ncid, int varid, const char *newname);
 /* End defined in var.c */
 
 #define IS_RECVAR(vp) \
@@ -454,11 +366,12 @@ ncmpi_rename_var(int ncid, int varid, const char *newname);
  *  The PnetCDF non-blocking I/O request type
  */
 typedef struct NC_req {
-    int            id;
-    int            rw_flag;
+    int            id;          /* even number for write, odd for read */
     void          *buf;         /* the original user buffer */
     void          *xbuf;        /* the buffer used to read/write, may point to
                                    the same address as buf */
+    MPI_Offset     num_recs;    /* number of records requested (1 for
+                                   fixed-size variable) */
     int            buftype_is_contig;
     int            need_swap_back_buf;
     int            abuf_index;  /* index in the abuf occupy_table
@@ -481,9 +394,6 @@ typedef struct NC_req {
     MPI_Datatype   ptype;        /* element data type in buftype */
     MPI_Datatype   imaptype;     /* derived data type constructed from imap */
     int           *status;       /* pointer to user's status */
-    int            num_subreqs;  /* each record is a subrequest */
-    struct NC_req *subreqs;      /* [num_subreq] */
-    struct NC_req *next;
 } NC_req;
 
 #define NC_ABUF_DEFAULT_TABLE_SIZE 128
@@ -503,6 +413,9 @@ typedef struct NC_buf {
     void          *buf;
 } NC_buf;
 
+/* chunk size for allocating read/write nonblocking request lists */
+#define NC_REQUEST_CHUNK 1024
+
 struct NC {
     /* linked list of currently opened netcdf files */
     struct NC *next;
@@ -515,13 +428,14 @@ struct NC {
     struct NC *old;
     /* flags */
 #define NC_INDEP  0x10000   /* in independent data mode, cleared by endindep */
-#define NC_CREAT  0x20000   /* in create phase, cleared by ncenddef */
-#define NC_INDEF  0x80000   /* in define mode, cleared by ncenddef */
+#define NC_CREAT  0x20000   /* in create phase, cleared by enddef */
+#define NC_INDEF  0x80000   /* in define mode, cleared by enddef */
 #define NC_NSYNC  0x100000  /* synchronise numrecs on change */
 #define NC_HSYNC  0x200000  /* synchronise whole header on change */
 #define NC_NDIRTY 0x400000  /* numrecs has changed */
 #define NC_HDIRTY 0x800000  /* header info has changed */
 /* NC_NOFILL is defined in netcdf.h, historical interface */
+    int           format;   /* 1, 2, or 5 corresponding to CDF-1, 2, or 5 */
     int           flags;
     int           safe_mode;    /* 0 or 1, for parameter consistency check */
     int           subfile_mode; /* 0 or 1, for disable/enable subfiling */
@@ -535,14 +449,14 @@ struct NC {
     MPI_Offset    recsize;  /* length of 'record': sum of single record sizes
                                of all the record variables */
     MPI_Offset    numrecs;  /* number of 'records' allocated */
-    int           numGetReqs;  /* number of pending nonblocking get requests */
-    int           numPutReqs;  /* number of pending nonblocking put requests */
     NC_dimarray   dims;     /* dimensions defined */
     NC_attrarray  attrs;    /* global attributes defined */
     NC_vararray   vars;     /* variables defined */
-    NC_req       *head;     /* linked list head of nonblocking requests */
-    NC_req       *tail;     /* tail of the linked list */
-    NC_buf       *abuf;     /* attached buffer, used by bput APIs */
+    int           numGetReqs;  /* number of pending nonblocking get requests */
+    int           numPutReqs;  /* number of pending nonblocking put requests */
+    NC_req       *get_list;    /* list of nonblocking read requests */
+    NC_req       *put_list;    /* list of nonblocking write requests */
+    NC_buf       *abuf;        /* attached buffer, used by bput APIs */
 };
 
 #define NC_readonly(ncp) \
@@ -645,30 +559,6 @@ ncmpii__enddef(NC *ncp, MPI_Offset h_minfree, MPI_Offset v_align,
 extern int
 ncmpii_close(NC *ncp);
 
-extern int
-ncmpi_inq(int ncid, int *ndimsp, int *nvarsp, int *nattsp, int *xtendimp);
-
-extern int
-ncmpi_inq_ndims(int ncid, int *ndimsp);
-
-extern int
-ncmpi_inq_nvars(int ncid, int *nvarsp);
-
-extern int
-ncmpi_inq_natts(int ncid, int *nattsp);
-
-extern int
-ncmpi_inq_unlimdim(int ncid, int *xtendimp);
-
-extern int
-ncmpi_get_default_format(void);
-
-extern int
-ncmpi_inq_num_rec_vars(int ncid, int *nump);
-
-extern int
-ncmpi_inq_num_fix_vars(int ncid, int *nump);
-
 /* End defined in nc.c */
 
 #if 0
@@ -712,8 +602,6 @@ typedef struct bufferinfo {
     void       *pos;      /* current position in buffer */
     MPI_Offset  size;     /* size of the buffer */
     MPI_Offset  index;    /* index of current position in buffer */
-    MPI_Offset  put_size; /* amount of writes so far in bytes */
-    MPI_Offset  get_size; /* amount of reads  so far in bytes */
 } bufferinfo;
 
 extern int
@@ -767,7 +655,6 @@ NC_computeshapes(NC *ncp);
 /* end defined in mpincio.h */
 
 /* begin defined in error.c */
-const char* ncmpi_strerror(int err);
 
 int ncmpii_handle_error(int mpi_errorcode, char *msg);
 
@@ -780,52 +667,59 @@ int ncmpii_handle_error(int mpi_errorcode, char *msg);
  * ../nc_test/tests.h if you change these.
  */
 
-int ncmpii_x_putn_schar (void *xbuf, const void *buf, MPI_Offset nelems,
-                        MPI_Datatype datatype);
-int ncmpii_x_putn_uchar (void *xbuf, const void *buf, MPI_Offset nelems,
-                        MPI_Datatype datatype);
-int ncmpii_x_putn_short (void *xbuf, const void *buf, MPI_Offset nelems,
-                        MPI_Datatype datatype);
-int ncmpii_x_putn_ushort(void *xbuf, const void *buf, MPI_Offset nelems,
-                        MPI_Datatype datatype);
-int ncmpii_x_putn_int   (void *xbuf, const void *buf, MPI_Offset nelems,
-                        MPI_Datatype datatype);
-int ncmpii_x_putn_uint  (void *xbuf, const void *buf, MPI_Offset nelems,
-                        MPI_Datatype datatype);
-int ncmpii_x_putn_float (void *xbuf, const void *buf, MPI_Offset nelems,
-                        MPI_Datatype datatype);
-int ncmpii_x_putn_double(void *xbuf, const void *buf, MPI_Offset nelems,
-                        MPI_Datatype datatype);
-int ncmpii_x_putn_int64 (void *xbuf, const void *buf, MPI_Offset nelems,
-                        MPI_Datatype datatype);
-int ncmpii_x_putn_uint64(void *xbuf, const void *buf, MPI_Offset nelems,
-                        MPI_Datatype datatype);
-int ncmpii_x_getn_schar (const void *xbuf, void *buf, MPI_Offset nelems,
-                        MPI_Datatype datatype);
-int ncmpii_x_getn_uchar (const void *xbuf, void *buf, MPI_Offset nelems,
-                        MPI_Datatype datatype);
-int ncmpii_x_getn_short (const void *xbuf, void *buf, MPI_Offset nelems,
-                        MPI_Datatype datatype);
-int ncmpii_x_getn_ushort(const void *xbuf, void *buf, MPI_Offset nelems,
-                        MPI_Datatype datatype);
-int ncmpii_x_getn_int   (const void *xbuf, void *buf, MPI_Offset nelems,
-                        MPI_Datatype datatype);
-int ncmpii_x_getn_uint  (const void *xbuf, void *buf, MPI_Offset nelems,
-                        MPI_Datatype datatype);
-int ncmpii_x_getn_float (const void *xbuf, void *buf, MPI_Offset nelems,
-                        MPI_Datatype datatype);
-int ncmpii_x_getn_double(const void *xbuf, void *buf, MPI_Offset nelems,
-                        MPI_Datatype datatype);
-int ncmpii_x_getn_int64 (const void *xbuf, void *buf, MPI_Offset nelems,
-                        MPI_Datatype datatype);
-int ncmpii_x_getn_uint64(const void *xbuf, void *buf, MPI_Offset nelems,
-                        MPI_Datatype datatype);
+int ncmpii_x_putn_NC_CHAR  (void *xbuf, const void *buf, MPI_Offset nelems,
+                           MPI_Datatype datatype);
+int ncmpii_x_putn_NC_BYTE  (int cdf_ver,
+                           void *xbuf, const void *buf, MPI_Offset nelems,
+                           MPI_Datatype datatype, void *fillp);
+int ncmpii_x_putn_NC_UBYTE (void *xbuf, const void *buf, MPI_Offset nelems,
+                           MPI_Datatype datatype, void *fillp);
+int ncmpii_x_putn_NC_SHORT (void *xbuf, const void *buf, MPI_Offset nelems,
+                           MPI_Datatype datatype, void *fillp);
+int ncmpii_x_putn_NC_USHORT(void *xbuf, const void *buf, MPI_Offset nelems,
+                           MPI_Datatype datatype, void *fillp);
+int ncmpii_x_putn_NC_INT   (void *xbuf, const void *buf, MPI_Offset nelems,
+                           MPI_Datatype datatype, void *fillp);
+int ncmpii_x_putn_NC_UINT  (void *xbuf, const void *buf, MPI_Offset nelems,
+                           MPI_Datatype datatype, void *fillp);
+int ncmpii_x_putn_NC_FLOAT (void *xbuf, const void *buf, MPI_Offset nelems,
+                           MPI_Datatype datatype, void *fillp);
+int ncmpii_x_putn_NC_DOUBLE(void *xbuf, const void *buf, MPI_Offset nelems,
+                           MPI_Datatype datatype, void *fillp);
+int ncmpii_x_putn_NC_INT64 (void *xbuf, const void *buf, MPI_Offset nelems,
+                           MPI_Datatype datatype, void *fillp);
+int ncmpii_x_putn_NC_UINT64(void *xbuf, const void *buf, MPI_Offset nelems,
+                           MPI_Datatype datatype, void *fillp);
+
+int ncmpii_x_getn_NC_CHAR  (const void *xbuf, void *buf, MPI_Offset nelems,
+                           MPI_Datatype datatype);
+int ncmpii_x_getn_NC_BYTE  (int cdf_ver,
+                           const void *xbuf, void *buf, MPI_Offset nelems,
+                           MPI_Datatype datatype);
+int ncmpii_x_getn_NC_UBYTE (const void *xbuf, void *buf, MPI_Offset nelems,
+                           MPI_Datatype datatype);
+int ncmpii_x_getn_NC_SHORT (const void *xbuf, void *buf, MPI_Offset nelems,
+                           MPI_Datatype datatype);
+int ncmpii_x_getn_NC_USHORT(const void *xbuf, void *buf, MPI_Offset nelems,
+                           MPI_Datatype datatype);
+int ncmpii_x_getn_NC_INT   (const void *xbuf, void *buf, MPI_Offset nelems,
+                           MPI_Datatype datatype);
+int ncmpii_x_getn_NC_UINT  (const void *xbuf, void *buf, MPI_Offset nelems,
+                           MPI_Datatype datatype);
+int ncmpii_x_getn_NC_FLOAT (const void *xbuf, void *buf, MPI_Offset nelems,
+                           MPI_Datatype datatype);
+int ncmpii_x_getn_NC_DOUBLE(const void *xbuf, void *buf, MPI_Offset nelems,
+                           MPI_Datatype datatype);
+int ncmpii_x_getn_NC_INT64 (const void *xbuf, void *buf, MPI_Offset nelems,
+                           MPI_Datatype datatype);
+int ncmpii_x_getn_NC_UINT64(const void *xbuf, void *buf, MPI_Offset nelems,
+                           MPI_Datatype datatype);
 
 int NC_start_count_stride_ck(const NC *ncp, const NC_var *varp,
                 const MPI_Offset *start, const MPI_Offset *count,
                 const MPI_Offset *stride, const int rw_flag);
 
-int ncmpii_need_convert(nc_type nctype,MPI_Datatype mpitype);
+int ncmpii_need_convert(int format, nc_type nctype, MPI_Datatype mpitype);
 
 int ncmpii_need_swap(nc_type nctype,MPI_Datatype mpitype);
 
@@ -842,6 +736,8 @@ int ncmpii_get_offset(NC *ncp, NC_var *varp, const MPI_Offset starts[],
 
 int ncmpii_check_mpifh(NC* ncp, int collective);
 
+int ncmpii_write_numrecs(NC *ncp, MPI_Offset new_numrecs);
+
 int ncmpii_sync_numrecs(NC *ncp, MPI_Offset newnumrecs);
 
 int ncmpii_vars_create_filetype(NC* ncp, NC_var* varp,
@@ -899,12 +795,8 @@ ncmpii_inq_files_opened(int *num, int *ncids);
 extern MPI_Datatype
 ncmpii_nc2mpitype(nc_type type);
 
-extern int
-ncmpii_set_iget_callback(NC *ncp, int reqid, void *tmpBuf, void *userBuf,
-                         int userBufCount, MPI_Datatype userBufType);
-
-extern int
-ncmpii_set_iput_callback(NC *ncp, int reqid, void *tmpPutBuf);
+extern nc_type
+ncmpii_mpi2nctype(MPI_Datatype itype);
 
 extern int
 ncmpii_end_indep_data(NC *ncp); 
@@ -919,9 +811,8 @@ ncmpii_create_imaptype(NC_var *varp, const MPI_Offset *count,
                        MPI_Datatype *imaptype);
 
 extern int
-ncmpii_calc_datatype_elems(NC *ncp, NC_var *varp, const MPI_Offset *start,
-                           const MPI_Offset *count, const MPI_Offset *stride,
-                           int rw_flag, MPI_Datatype buftype,
+ncmpii_calc_datatype_elems(NC_var *varp, const MPI_Offset *count,
+                           MPI_Datatype buftype,
                            MPI_Datatype *ptype, MPI_Offset *bufcount,
                            MPI_Offset *bnelems, MPI_Offset *nbytes,
                            int *el_size, int *buftype_is_contig);
@@ -931,11 +822,40 @@ ncmpii_fill_vars(NC *ncp);
 
 extern int
 ncmpii_sanity_check(int ncid, int varid, const MPI_Offset *start,
-                    const MPI_Offset *count, MPI_Offset bufcount,
-                    enum API_KIND api, int mustInDataMode, int isFlexAPI,
+                    const MPI_Offset *count, const MPI_Offset *stride,
+                    MPI_Offset bufcount, MPI_Datatype buftype,
+                    enum API_KIND api, int isFlexibleAPI, int mustInDataMode,
                     int rw_flag, int io_method, NC **ncp, NC_var **varp);
 
 extern char*
 ncmpii_err_code_name(int err);
 
+extern int
+ncmpii_jenkins_one_at_a_time_hash(const char *str_name);
+
+extern int
+ncmpii_additive_hash(const char *str_name);
+
+extern int
+ncmpii_rotating_hash(const char *str_name);
+
+extern int
+ncmpii_Bernstein_hash(const char *str_name);
+
+extern int
+ncmpii_Pearson_hash(const char *str_name);
+
+extern int
+ncmpii_update_name_lookup_table(NC_nametable *nameT, const int id,
+                                const char *oldname, const char *newname);
+
+extern int
+ncmpii_inq_var_fill(NC_var *varp, void *fill_value);
+
+extern int
+ncmpii_inq_default_fill_value(int type, void *fill_value);
+
+extern int
+ncmpii_getput_zero_req(NC *ncp, int rw_flag);
+
 #endif /* _NC_H_ */
