@@ -2,10 +2,10 @@
  *  Copyright (C) 2013, Northwestern University and Argonne National Laboratory
  *  See COPYRIGHT notice in top-level directory.
  */
-/* $Id: subfile.c 2196 2015-11-27 22:31:05Z wkliao $ */
+/* $Id: subfile.c 2686 2016-12-07 20:02:41Z wkliao $ */
 
 #if HAVE_CONFIG_H
-# include "ncconfig.h"
+# include <ncconfig.h>
 #endif
 #include "subfile.h"
 #ifdef TAU_SSON
@@ -190,8 +190,7 @@ int ncmpii_subfile_close(NC *ncp)
     NC *ncp_sf;
 
     status = ncmpii_NC_check_id(ncp->ncid_sf, &ncp_sf);
-    if (status != NC_NOERR)
-        return status;
+    if (status != NC_NOERR) DEBUG_RETURN_ERROR(status)
 
     status = ncmpii_close(ncp_sf);
     if (status != NC_NOERR)
