@@ -2,7 +2,7 @@
  *  Copyright (C) 2012, Northwestern University and Argonne National Laboratory
  *  See COPYRIGHT notice in top-level directory.
  */
-/* $Id: utf8proc.c 2220 2015-12-11 23:07:44Z wkliao $ */
+/* $Id: utf8proc.c 2475 2016-09-05 23:15:29Z wkliao $ */
 
 /*
  *  Copyright (c) 2006-2007 Jan Behrens, FlexiGuided GmbH, Berlin
@@ -48,12 +48,13 @@
 
 
 #if HAVE_CONFIG_H
-# include "ncconfig.h"
+# include <ncconfig.h>
 #endif
 
 #include "utf8proc.h"
 #include "utf8proc_data.h"
 
+static
 const int8_t utf8proc_utf8class[256] = {
   1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
   1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
@@ -106,6 +107,8 @@ const int8_t utf8proc_utf8class[256] = {
 #define UTF8PROC_BOUNDCLASS_LVT     10
 
 
+#ifdef UNUSED_APIS
+static
 const char *utf8proc_errmsg(ssize_t errcode) {
   switch (errcode) {
     case UTF8PROC_ERROR_NOMEM:
@@ -122,7 +125,9 @@ const char *utf8proc_errmsg(ssize_t errcode) {
     return "An unknown error occurred while processing UTF-8 data.";
   }
 }
+#endif
 
+static
 ssize_t utf8proc_iterate(
   const uint8_t *str, ssize_t slen, int32_t *dst
 ) {
@@ -163,13 +168,17 @@ ssize_t utf8proc_iterate(
   return length;
 }
 
+#ifdef UNUSED_APIS
+static
 bool utf8proc_codepoint_valid(int32_t uc) {
   if (uc < 0 || uc >= 0x110000 ||
     ((uc & 0xFFFF) >= 0xFFFE) || (uc >= 0xD800 && uc < 0xE000) ||
     (uc >= 0xFDD0 && uc < 0xFDF0)) return false;
   else return true;
 }
+#endif
 
+static
 ssize_t utf8proc_encode_char(int32_t uc, uint8_t *dst) {
   if (uc < 0x00) {
     return 0;
@@ -200,6 +209,9 @@ ssize_t utf8proc_encode_char(int32_t uc, uint8_t *dst) {
   } else return 0;
 }
 
+#define UTF8PROC_GET_PROPERTY(uc) (&utf8proc_properties[utf8proc_stage2table[utf8proc_stage1table[uc >> 8] + (uc & 0xFF)]])
+#if 0
+static
 const utf8proc_property_t *utf8proc_get_property(int32_t uc) {
   /*  // ASSERT: uc >= 0 && uc < 0x110000*/
   return utf8proc_properties + (
@@ -208,18 +220,20 @@ const utf8proc_property_t *utf8proc_get_property(int32_t uc) {
     ]
   );
 }
+#endif
 
 #define utf8proc_decompose_lump(replacement_uc) \
   return utf8proc_decompose_char((replacement_uc), dst, bufsize, \
   options & ~UTF8PROC_LUMP, last_boundclass)
 
+static
 ssize_t utf8proc_decompose_char(int32_t uc, int32_t *dst, ssize_t bufsize,
     int options, int *last_boundclass) {
   /*// ASSERT: uc >= 0 && uc < 0x110000*/
   const utf8proc_property_t *property;
   utf8proc_propval_t category;
   int32_t hangul_sindex;
-  property = utf8proc_get_property(uc);
+  property = UTF8PROC_GET_PROPERTY(uc);
   category = property->category;
   hangul_sindex = uc - UTF8PROC_HANGUL_SBASE;
   if (options & (UTF8PROC_COMPOSE|UTF8PROC_DECOMPOSE)) {
@@ -358,6 +372,7 @@ ssize_t utf8proc_decompose_char(int32_t uc, int32_t *dst, ssize_t bufsize,
   return 1;
 }
 
+static
 ssize_t utf8proc_decompose(
   const uint8_t *str, ssize_t slen,
   int32_t *buffer, ssize_t bufsize, int options
@@ -405,8 +420,8 @@ ssize_t utf8proc_decompose(
       const utf8proc_property_t *property1, *property2;
       uc1 = buffer[pos];
       uc2 = buffer[pos+1];
-      property1 = utf8proc_get_property(uc1);
-      property2 = utf8proc_get_property(uc2);
+      property1 = UTF8PROC_GET_PROPERTY(uc1);
+      property2 = UTF8PROC_GET_PROPERTY(uc2);
       if (property1->combining_class > property2->combining_class &&
           property2->combining_class > 0) {
         buffer[pos] = uc2;
@@ -420,6 +435,7 @@ ssize_t utf8proc_decompose(
   return wpos;
 }
 
+static
 ssize_t utf8proc_reencode(int32_t *buffer, ssize_t length, int options) {
   /* UTF8PROC_NULLTERM option will be ignored, 'length' is never ignored
      ASSERT: 'buffer' has one spare byte of free space at the end! */
@@ -464,7 +480,7 @@ ssize_t utf8proc_reencode(int32_t *buffer, ssize_t length, int options) {
     int32_t composition;
     for (rpos = 0; rpos < length; rpos++) {
       current_char = buffer[rpos];
-      current_property = utf8proc_get_property(current_char);
+      current_property = UTF8PROC_GET_PROPERTY(current_char);
       if (starter && current_property->combining_class > max_combining_class) {
 	/*        // combination perhaps possible*/
         int32_t hangul_lindex;
@@ -493,7 +509,7 @@ ssize_t utf8proc_reencode(int32_t *buffer, ssize_t length, int options) {
           }
         }
         if (!starter_property) {
-          starter_property = utf8proc_get_property(*starter);
+          starter_property = UTF8PROC_GET_PROPERTY(*starter);
         }
         if (starter_property->comb1st_index >= 0 &&
             current_property->comb2nd_index >= 0) {
@@ -502,7 +518,7 @@ ssize_t utf8proc_reencode(int32_t *buffer, ssize_t length, int options) {
             current_property->comb2nd_index
           ];
           if (composition >= 0 && (!(options & UTF8PROC_STABLE) ||
-              !(utf8proc_get_property(composition)->comp_exclusion))) {
+              !(UTF8PROC_GET_PROPERTY(composition)->comp_exclusion))) {
             *starter = composition;
             starter_property = NULL;
             continue;
@@ -535,6 +551,7 @@ ssize_t utf8proc_reencode(int32_t *buffer, ssize_t length, int options) {
   }
 }
 
+static
 ssize_t utf8proc_map(
   const uint8_t *str, ssize_t slen, uint8_t **dstptr, int options
 ) {
@@ -564,20 +581,25 @@ ssize_t utf8proc_map(
   return result;
 }
 
+#ifdef UNUSED_APIS
+static
 uint8_t *utf8proc_NFD(const uint8_t *str) {
   uint8_t *retval;
   utf8proc_map(str, 0, &retval, UTF8PROC_NULLTERM | UTF8PROC_STABLE |
     UTF8PROC_DECOMPOSE);
   return retval;
 }
+#endif
 
-uint8_t *utf8proc_NFC(const uint8_t *str) {
+uint8_t* ncmpii_utf8proc_NFC(const uint8_t *str) {
   uint8_t *retval;
   utf8proc_map(str, 0, &retval, UTF8PROC_NULLTERM | UTF8PROC_STABLE |
     UTF8PROC_COMPOSE);
   return retval;
 }
 
+#ifdef UNUSED_APIS
+static
 uint8_t *utf8proc_NFKD(const uint8_t *str) {
   uint8_t *retval;
   utf8proc_map(str, 0, &retval, UTF8PROC_NULLTERM | UTF8PROC_STABLE |
@@ -585,14 +607,16 @@ uint8_t *utf8proc_NFKD(const uint8_t *str) {
   return retval;
 }
 
+static
 uint8_t *utf8proc_NFKC(const uint8_t *str) {
   uint8_t *retval;
   utf8proc_map(str, 0, &retval, UTF8PROC_NULLTERM | UTF8PROC_STABLE |
     UTF8PROC_COMPOSE | UTF8PROC_COMPAT);
   return retval;
 }
+#endif
 
-ssize_t utf8proc_check(const uint8_t *str) {
+ssize_t  ncmpii_utf8proc_check(const uint8_t *str) {
   ssize_t result;
   result = utf8proc_decompose(str, 0, NULL, 0,
 			      UTF8PROC_NULLTERM | UTF8PROC_STABLE);
