@@ -2,10 +2,10 @@
  *  Copyright (C) 2003, Northwestern University and Argonne National Laboratory
  *  See COPYRIGHT notice in top-level directory.
  */
-/* $Id: var.c 2322 2016-02-26 22:47:06Z wkliao $ */
+/* $Id: var.c 2683 2016-12-07 06:42:58Z wkliao $ */
 
 #if HAVE_CONFIG_H
-# include "ncconfig.h"
+# include <ncconfig.h>
 #endif
 
 #ifdef HAVE_STDLIB_H
@@ -45,8 +45,7 @@ ncmpii_free_NC_var(NC_var *varp)
 
 /*----< ncmpii_new_x_NC_var() >-----------------------------------------------*/
 /*
- * Common code for ncmpii_new_NC_var()
- * and ncx_get_NC_var()
+ * Used by ncmpii_new_NC_var() and ncx_get_NC_var()
  */
 NC_var *
 ncmpii_new_x_NC_var(NC_string *strp,
@@ -94,50 +93,124 @@ ncmpii_new_x_NC_var(NC_string *strp,
     return varp;
 }
 
-
 /*----< ncmpii_new_NC_var() >------------------------------------------------*/
 /*
  * Formerly, NC_new_var()
  */
-static NC_var *
-ncmpii_new_NC_var(const char *uname,  /* variable name (NULL terminated) */
-                  nc_type     type,
-                  int         ndims,
-                  const int  *dimids)
+static int
+ncmpii_new_NC_var(NC_vararray  *vcap,
+                  const char   *name,  /* normalized variable name (NULL terminated) */
+                  nc_type       type,
+                  int           ndims,
+                  const int    *dimids,
+                  NC_var      **varp)
 {
     NC_string *strp;
-    NC_var *varp;
-
-    char *name = (char *)utf8proc_NFC((const unsigned char *)uname);
-    if (name == NULL) return NULL;
 
     strp = ncmpii_new_NC_string(strlen(name), name);
-    free(name);
-    if (strp == NULL) return NULL;
+    if (strp == NULL) DEBUG_RETURN_ERROR(NC_ENOMEM)
 
-    varp = ncmpii_new_x_NC_var(strp, ndims);
-    if (varp == NULL ) {
+    *varp = ncmpii_new_x_NC_var(strp, ndims);
+    if (*varp == NULL ) {
         ncmpii_free_NC_string(strp);
-        return NULL;
+        DEBUG_RETURN_ERROR(NC_ENOMEM)
     }
 
-    varp->type = type;
+    (*varp)->type = type;
 
     if (ndims != 0 && dimids != NULL)
-        memcpy(varp->dimids, dimids, (size_t)ndims * SIZEOF_INT);
+        memcpy((*varp)->dimids, dimids, (size_t)ndims * SIZEOF_INT);
 
-    return(varp);
+#ifndef SEARCH_NAME_LINEARLY
+    if (vcap != NULL) { /* insert new var to hash table */
+        int key;
+        NC_nametable *nameT = vcap->nameT; /* var name lookup table */
+
+        /* hash the var name into a key for name lookup */
+        key = HASH_FUNC(name);
+
+        /* allocate or expand the space for nameT[key].list */
+        if (nameT[key].num % NC_NAME_TABLE_CHUNK == 0)
+            nameT[key].list = (int*) NCI_Realloc(nameT[key].list,
+                              (size_t)(nameT[key].num+NC_NAME_TABLE_CHUNK) * sizeof(int));
+
+        /* add the new variable ID to the name lookup table
+         * the new varid will be vcap->ndefined
+         */
+        nameT[key].list[nameT[key].num] = vcap->ndefined;
+        nameT[key].num++;
+    }
+    /* else case is for variable duplication called from dup_NC_var() */
+#endif
+
+    return NC_NOERR;
 }
 
+/*----< ncmpii_update_name_lookup_table() >----------------------------------*/
+/* remove the entry in lookup table for oldname and add a new entry for
+ * newname
+ */
+int
+ncmpii_update_name_lookup_table(NC_nametable *nameT,
+                                const int     id,
+                                const char   *oldname,  /*    normalized */
+                                const char   *unewname) /* un-normalized */
+{
+    int i, key;
+    char *name; /* normalized name string */
+
+    /* remove the old name from the lookup table
+     * hash the var name into a key for name lookup
+     */
+    key = HASH_FUNC(oldname);
+    for (i=0; i<nameT[key].num; i++) {
+        if (nameT[key].list[i] == id) break;
+    }
+    assert(i!=nameT[key].num);
+
+    /* coalesce the id array */
+    for (; i<nameT[key].num-1; i++)
+        nameT[key].list[i] = nameT[key].list[i+1]; 
+
+    /* decrease the number of IDs and free space if necessary */
+    nameT[key].num--;
+    if (nameT[key].num == 0) {
+        NCI_Free(nameT[key].list);
+        nameT[key].list = NULL;
+    }
+
+    /* normalized version of uname */
+    name = (char *)ncmpii_utf8proc_NFC((const unsigned char *)unewname);
+    if (name == NULL) DEBUG_RETURN_ERROR(NC_ENOMEM)
+
+    /* hash the var name into a key for name lookup */
+    key = HASH_FUNC(name);
+    free(name);
+
+    /* add the new name to the lookup table
+     * Note unewname must have already been checked for existence
+     */
+    if (nameT[key].num % NC_NAME_TABLE_CHUNK == 0)
+        nameT[key].list = (int*) NCI_Realloc(nameT[key].list,
+                          (size_t)(nameT[key].num+NC_NAME_TABLE_CHUNK) * sizeof(int));
+    nameT[key].list[nameT[key].num] = id;
+    nameT[key].num++;
+
+    return NC_NOERR;
+}
+
+
 /*----< dup_NC_var() >--------------------------------------------------------*/
 NC_var *
 dup_NC_var(const NC_var *rvarp)
 {
+    int err;
     NC_var *varp;
 
-    varp = ncmpii_new_NC_var(rvarp->name->cp, rvarp->type, rvarp->ndims,
-                             rvarp->dimids);
-    if (varp == NULL) return NULL;
+    /* note that name in rvarp->name->cp is already normalized */
+    err = ncmpii_new_NC_var(NULL, rvarp->name->cp, rvarp->type, rvarp->ndims,
+                            rvarp->dimids, &varp);
+    if (err != NC_NOERR) return NULL;
 
     if (ncmpii_dup_NC_attrarray(&varp->attrs, &rvarp->attrs) != NC_NOERR) {
         ncmpii_free_NC_var(varp);
@@ -182,6 +255,13 @@ ncmpii_free_NC_vararray(NC_vararray *ncap)
     ncap->value    = NULL;
     ncap->nalloc   = 0;
     ncap->ndefined = 0;
+
+    /* free space allocated for var name lookup table */
+    for (i=0; i<HASH_TABLE_SIZE; i++) {
+        if (ncap->nameT[i].num > 0)
+            NCI_Free(ncap->nameT[i].list);
+        ncap->nameT[i].num = 0;
+    }
 }
 
 
@@ -224,6 +304,17 @@ ncmpii_dup_NC_vararray(NC_vararray       *ncap,
 
     ncap->ndefined = ref->ndefined;
 
+    /* duplicate var name lookup table */
+    for (i=0; i<HASH_TABLE_SIZE; i++) {
+        ncap->nameT[i].num = ref->nameT[i].num;
+        ncap->nameT[i].list = NULL;
+        if (ncap->nameT[i].num > 0) {
+            ncap->nameT[i].list = NCI_Malloc((size_t)ncap->nameT[i].num * sizeof(int));
+            memcpy(ncap->nameT[i].list, ref->nameT[i].list,
+                   (size_t)ncap->nameT[i].num * sizeof(int));
+        }
+    }
+
     return NC_NOERR;
 }
 
@@ -286,46 +377,71 @@ elem_NC_vararray(const NC_vararray *ncap,
 /* End vararray per se */
 
 
+#ifdef SEARCH_NAME_LINEARLY
 /*
  * Step thru NC_VARIABLE array, seeking match on name.
- * Return varid or -1 on not found.
- * *varpp is set to the appropriate NC_var.
+ * If found, set the variable ID pointed by vardip, otherwise return NC_ENOTVAR
  * Formerly (sort of)
 NC_hvarid
  */
 static int
-ncmpii_NC_findvar(const NC_vararray  *ncap,
-                  const char         *uname,
-                  NC_var            **varpp)
+ncmpii_NC_findvar(const NC_vararray *ncap,
+                  const char        *name,  /* normalized name */
+                  int               *varidp)
 {
     int varid;
     size_t nchars;
-    char *name;
     NC_var **loc;
 
     assert (ncap != NULL);
 
-    if (ncap->ndefined == 0) return -1;
+    if (ncap->ndefined == 0) return NC_ENOTVAR;
 
     loc = (NC_var **) ncap->value;
 
-    /* normalized version of uname */
-    name = (char *)utf8proc_NFC((const unsigned char *)uname);
-    if (name == NULL) DEBUG_RETURN_ERROR(NC_ENOMEM)
     nchars = strlen(name);
 
     for (varid=0; varid<ncap->ndefined; varid++, loc++) {
         if ((*loc)->name->nchars == nchars &&
             strncmp((*loc)->name->cp, name, nchars) == 0) {
-            if (varpp != NULL)
-                *varpp = *loc;
-            free(name);
-            return (varid); /* Normal return */
+            if (varidp != NULL) *varidp = varid;
+            return NC_NOERR; /* found it */
         }
     }
-    free(name);
-    return (-1); /* not found */
+
+    return NC_ENOTVAR; /* not found */
 }
+#else
+/*----< ncmpii_NC_findvar() >------------------------------------------------*/
+/* Check if the name has been used.
+ * If yes, set the variable ID pointed by vardip, otherwise return NC_ENOTVAR
+ */
+static int
+ncmpii_NC_findvar(const NC_vararray  *ncap,
+                  const char         *name,  /* normalized name */
+                  int                *varidp)
+{
+    int i, key, varid;
+
+    assert (ncap != NULL);
+
+    if (ncap->ndefined == 0) return NC_ENOTVAR;
+
+    /* hash the var name into a key for name lookup */
+    key = HASH_FUNC(name);
+
+    /* check the list using linear search */
+    for (i=0; i<ncap->nameT[key].num; i++) {
+        varid = ncap->nameT[key].list[i];
+        if (strcmp(name, ncap->value[varid]->name->cp) == 0) {
+            if (varidp != NULL) *varidp = varid;
+            return NC_NOERR; /* the name already exists */
+        }
+    }
+
+    return NC_ENOTVAR; /* the name has never been used */
+}
+#endif
 
 /*
  * For a netcdf type
@@ -363,8 +479,7 @@ ncx_szof(nc_type type)
  * set varp->xsz, varp->shape and varp->len of a variable
  */
 int
-ncmpii_NC_var_shape64(NC                *ncp,
-                      NC_var            *varp,
+ncmpii_NC_var_shape64(NC_var            *varp,
                       const NC_dimarray *dims)
 {
     int i;
@@ -428,7 +543,7 @@ out :
      * We will check this in ncmpi_enddef() which calls ncmpii_NC_enddef()
      * which calls ncmpii_NC_check_vlens()
      *
-    if (! fIsSet(ncp->flags, NC_64BIT_DATA) && product >= X_UINT_MAX)
+    if (ncp->format < 5 && product >= X_UINT_MAX)
         DEBUG_RETURN_ERROR(NC_EVARSIZE)
      */
 
@@ -500,64 +615,167 @@ ncmpi_def_var(int         ncid,
               const int  *dimids,
               int        *varidp)
 {
-    int varid, file_ver, status;
-    NC *ncp;
-    NC_var *varp;
+    int err;
+    char *nname=NULL; /* normalized name */
+    NC *ncp=NULL;
+    NC_var *varp=NULL;
 
     /* check if ncid is valid */
-    status = ncmpii_NC_check_id(ncid, &ncp);
-    if (status != NC_NOERR) return status;
+    err = ncmpii_NC_check_id(ncid, &ncp);
+    if (err != NC_NOERR) DEBUG_RETURN_ERROR(err)
 
     /* check if called in define mode */
-    if (!NC_indef(ncp)) DEBUG_RETURN_ERROR(NC_ENOTINDEFINE)
+    if (!NC_indef(ncp)) {
+        DEBUG_ASSIGN_ERROR(err, NC_ENOTINDEFINE)
+        goto err_check;
+    }
+
+    if (name == NULL || *name == 0 || strlen(name) > NC_MAX_NAME) {
+        DEBUG_ASSIGN_ERROR(err, NC_EBADNAME)
+        goto err_check;
+    }
 
     /* check if the name string is legal for netcdf format */
-    file_ver = 1;
-    if (fIsSet(ncp->flags, NC_64BIT_OFFSET))
-        file_ver = 2;
-    else if (fIsSet(ncp->flags, NC_64BIT_DATA))
-        file_ver = 5;
-    status = ncmpii_NC_check_name(name, file_ver);
-    if (status != NC_NOERR) return status;
+    err = ncmpii_NC_check_name(name, ncp->format);
+    if (err != NC_NOERR) {
+        DEBUG_TRACE_ERROR
+        goto err_check;
+    }
 
     /* check if type is a valid netcdf type */
-    status = ncmpii_cktype(file_ver, type);
-    if (status != NC_NOERR) return status;
+    err = ncmpii_cktype(ncp->format, type);
+    if (err != NC_NOERR) {
+        DEBUG_TRACE_ERROR
+        goto err_check;
+    }
 
     /* TODO: make ndims of type MPI_Offset so ndims can be > 2^31-1 in CDF-5
     if ((ndims < 0) || ndims > X_INT_MAX) DEBUG_RETURN_ERROR(NC_EINVAL)
     */
-    if (ndims < 0) DEBUG_RETURN_ERROR(NC_EINVAL)
+    if (ndims < 0) {
+        DEBUG_ASSIGN_ERROR(err, NC_EINVAL)
+        goto err_check;
+    }
 
     /* there is an upperbound for the number of variables defined in a file */
-    if (ncp->vars.ndefined >= NC_MAX_VARS) DEBUG_RETURN_ERROR(NC_EMAXVARS)
+    if (ncp->vars.ndefined >= NC_MAX_VARS) {
+        DEBUG_ASSIGN_ERROR(err, NC_EMAXVARS)
+        goto err_check;
+    }
 
-    /* check whether the variable name has been used */
-    varid = ncmpii_NC_findvar(&ncp->vars, name, &varp);
-    if (varid != -1) DEBUG_RETURN_ERROR(NC_ENAMEINUSE)
+    /* create a normalized character string */
+    nname = (char *)ncmpii_utf8proc_NFC((const unsigned char *)name);
+    if (nname == NULL) {
+        DEBUG_ASSIGN_ERROR(err, NC_ENOMEM)
+        goto err_check;
+    }
+
+    /* check whether new name is already in use, for this API (def_var) the
+     * name should NOT already exist */
+    err = ncmpii_NC_findvar(&ncp->vars, nname, NULL);
+    if (err != NC_ENOTVAR) {
+        DEBUG_ASSIGN_ERROR(err, NC_ENAMEINUSE)
+        goto err_check;
+    }
+    else
+        err = NC_NOERR;
+
+err_check:
+    if (ncp->safe_mode) {
+        int root_ndims, status, mpireturn;
+        char root_name[NC_MAX_NAME];
+
+        /* check if name is consistent among all processes */
+        if (name == NULL || *name == 0)
+            root_name[0] = 0;
+        else
+            strncpy(root_name, name, NC_MAX_NAME);
+        TRACE_COMM(MPI_Bcast)(root_name, NC_MAX_NAME, MPI_CHAR, 0, ncp->nciop->comm);
+        if (mpireturn != MPI_SUCCESS) {
+            if (nname != NULL) free(nname);
+            return ncmpii_handle_error(mpireturn, "MPI_Bcast");
+        }
+        if (err == NC_NOERR && strcmp(root_name, name))
+            DEBUG_ASSIGN_ERROR(err, NC_EMULTIDEFINE_VAR_NAME)
+
+        /* check if type is consistent among all processes */
+        nc_type root_type=type;
+        TRACE_COMM(MPI_Bcast)(&root_type, 1, MPI_INT, 0, ncp->nciop->comm);
+        if (mpireturn != MPI_SUCCESS) {
+            if (nname != NULL) free(nname);
+            return ncmpii_handle_error(mpireturn, "MPI_Bcast");
+        }
+        if (err == NC_NOERR && root_type != type)
+            DEBUG_ASSIGN_ERROR(err, NC_EMULTIDEFINE_VAR_TYPE)
+
+        /* check if ndims is consistent among all processes */
+        root_ndims=ndims;
+        TRACE_COMM(MPI_Bcast)(&root_ndims, 1, MPI_INT, 0, ncp->nciop->comm);
+        if (mpireturn != MPI_SUCCESS) {
+            if (nname != NULL) free(nname);
+            return ncmpii_handle_error(mpireturn, "MPI_Bcast");
+        }
+        if (err == NC_NOERR && root_ndims != ndims)
+            DEBUG_ASSIGN_ERROR(err, NC_EMULTIDEFINE_VAR_NDIMS)
+
+        /* check if dimids is consistent among all processes */
+        if (root_ndims > 0) {
+            int root_dimids[NC_MAX_DIMS];
+            if (dimids != NULL)
+                memcpy(root_dimids, dimids, (size_t)root_ndims*sizeof(int));
+            else
+                memset(root_dimids, 0, (size_t)root_ndims*sizeof(int));
+            TRACE_COMM(MPI_Bcast)(root_dimids, root_ndims, MPI_INT, 0, ncp->nciop->comm);
+            if (mpireturn != MPI_SUCCESS) {
+                if (nname != NULL) free(nname);
+                return ncmpii_handle_error(mpireturn, "MPI_Bcast");
+            }
+            if (err == NC_NOERR && dimids != NULL &&
+                memcmp(root_dimids, dimids, (size_t)root_ndims*sizeof(int)))
+                DEBUG_ASSIGN_ERROR(err, NC_EMULTIDEFINE_VAR_DIMIDS)
+        }
+
+        /* find min error code across processes */
+        TRACE_COMM(MPI_Allreduce)(&err, &status, 1, MPI_INT, MPI_MIN, ncp->nciop->comm);
+        if (mpireturn != MPI_SUCCESS) {
+            if (nname != NULL) free(nname);
+            return ncmpii_handle_error(mpireturn, "MPI_Allreduce");
+        }
+        if (err == NC_NOERR) err = status;
+    }
+
+    if (err != NC_NOERR) {
+        if (nname != NULL) free(nname);
+        return err;
+    }
+
+    assert(nname != NULL);
 
     /* create a new variable */
-    varp = ncmpii_new_NC_var(name, type, ndims, dimids);
-    if (varp == NULL) DEBUG_RETURN_ERROR(NC_ENOMEM)
+    err = ncmpii_new_NC_var(&ncp->vars, nname, type, ndims, dimids, &varp);
+    free(nname);
+    if (err != NC_NOERR) DEBUG_RETURN_ERROR(err)
 
     /* set up array dimensional structures */
-    status = ncmpii_NC_var_shape64(ncp, varp, &ncp->dims);
-    if (status != NC_NOERR) {
+    err = ncmpii_NC_var_shape64(varp, &ncp->dims);
+    if (err != NC_NOERR) {
         ncmpii_free_NC_var(varp);
-        return status;
+        DEBUG_RETURN_ERROR(err)
     }
 
     /* Add a new handle to the end of an array of handles */
-    status = incr_NC_vararray(&ncp->vars, varp);
-    if (status != NC_NOERR) {
+    err = incr_NC_vararray(&ncp->vars, varp);
+    if (err != NC_NOERR) {
         ncmpii_free_NC_var(varp);
-        return status;
+        DEBUG_RETURN_ERROR(err)
     }
 
     if (varidp != NULL)
         *varidp = (int)ncp->vars.ndefined - 1; /* varid */
         /* ncp->vars.ndefined has been increased in incr_NC_vararray() */
 
+    assert(varp != NULL);
+
     /* default is NOFILL */
     varp->no_fill = 1;
 
@@ -569,27 +787,35 @@ ncmpi_def_var(int         ncid,
 
 
 /*----< ncmpi_inq_varid() >--------------------------------------------------*/
+/* This is an independent subroutine */
 int
 ncmpi_inq_varid(int         ncid,
                 const char *name,
-                int        *varid_ptr)
+                int        *varid)
 {
-    int status;
-    NC *ncp;
-    NC_var *varp;
-    int varid;
+    int err;
+    char *nname=NULL; /* normalized name */
+    NC *ncp=NULL;
 
-    status = ncmpii_NC_check_id(ncid, &ncp);
-    if (status != NC_NOERR) return status;
+    err = ncmpii_NC_check_id(ncid, &ncp);
+    if (err != NC_NOERR) DEBUG_RETURN_ERROR(err)
 
-    varid = ncmpii_NC_findvar(&ncp->vars, name, &varp);
-    if (varid == -1) DEBUG_RETURN_ERROR(NC_ENOTVAR)
+    if (name == NULL || *name == 0 || strlen(name) > NC_MAX_NAME)
+        DEBUG_RETURN_ERROR(NC_EBADNAME)
+
+    /* create a normalized character string */
+    nname = (char *)ncmpii_utf8proc_NFC((const unsigned char *)name);
+    if (nname == NULL) DEBUG_RETURN_ERROR(NC_ENOMEM)
+
+    err = ncmpii_NC_findvar(&ncp->vars, nname, varid);
+    free(nname);
+    if (err != NC_NOERR) DEBUG_RETURN_ERROR(err)
 
-    *varid_ptr = varid;
     return NC_NOERR;
 }
 
 /*----< ncmpi_inq_var() >----------------------------------------------------*/
+/* This is an independent subroutine */
 int
 ncmpi_inq_var(int      ncid,
               int      varid,
@@ -599,12 +825,12 @@ ncmpi_inq_var(int      ncid,
               int     *dimids,
               int     *nattsp)
 {
-    int status;
-    NC *ncp;
-    NC_var *varp;
+    int err;
+    NC *ncp=NULL;
+    NC_var *varp=NULL;
 
-    status = ncmpii_NC_check_id(ncid, &ncp);
-    if (status != NC_NOERR) return status;
+    err = ncmpii_NC_check_id(ncid, &ncp);
+    if (err != NC_NOERR) DEBUG_RETURN_ERROR(err)
 
     /* using NC_GLOBAL in varid is illegal for this API. See
      * http://www.unidata.ucar.edu/mailing_lists/archives/netcdfgroup/2015/msg00196.html
@@ -647,17 +873,18 @@ ncmpi_inq_var(int      ncid,
 
 
 /*----< ncmpi_inq_varname() >------------------------------------------------*/
+/* This is an independent subroutine */
 int
 ncmpi_inq_varname(int   ncid,
                   int   varid,
                   char *name)
 {
-    int status;
-    NC *ncp;
-    NC_var *varp;
+    int err;
+    NC *ncp=NULL;
+    NC_var *varp=NULL;
 
-    status = ncmpii_NC_check_id(ncid, &ncp);
-    if (status != NC_NOERR) return status;
+    err = ncmpii_NC_check_id(ncid, &ncp);
+    if (err != NC_NOERR) DEBUG_RETURN_ERROR(err)
 
     /* using NC_GLOBAL in varid is illegal for this API. See
      * http://www.unidata.ucar.edu/mailing_lists/archives/netcdfgroup/2015/msg00196.html
@@ -675,17 +902,18 @@ ncmpi_inq_varname(int   ncid,
 }
 
 /*----< ncmpi_inq_vartype() >------------------------------------------------*/
+/* This is an independent subroutine */
 int
 ncmpi_inq_vartype(int      ncid,
                   int      varid,
                   nc_type *typep)
 {
-    int status;
-    NC *ncp;
-    NC_var *varp;
+    int err;
+    NC *ncp=NULL;
+    NC_var *varp=NULL;
 
-    status = ncmpii_NC_check_id(ncid, &ncp);
-    if (status != NC_NOERR) return status;
+    err = ncmpii_NC_check_id(ncid, &ncp);
+    if (err != NC_NOERR) DEBUG_RETURN_ERROR(err)
 
     /* using NC_GLOBAL in varid is illegal for this API. See
      * http://www.unidata.ucar.edu/mailing_lists/archives/netcdfgroup/2015/msg00196.html
@@ -701,15 +929,16 @@ ncmpi_inq_vartype(int      ncid,
 }
 
 /*----< ncmpi_inq_varndims() >-----------------------------------------------*/
+/* This is an independent subroutine */
 int
 ncmpi_inq_varndims(int ncid, int varid, int *ndimsp)
 {
-    int status;
-    NC *ncp;
-    NC_var *varp;
+    int err;
+    NC *ncp=NULL;
+    NC_var *varp=NULL;
 
-    status = ncmpii_NC_check_id(ncid, &ncp);
-    if (status != NC_NOERR) return status;
+    err = ncmpii_NC_check_id(ncid, &ncp);
+    if (err != NC_NOERR) DEBUG_RETURN_ERROR(err)
 
     /* using NC_GLOBAL in varid is illegal for this API. See
      * http://www.unidata.ucar.edu/mailing_lists/archives/netcdfgroup/2015/msg00196.html
@@ -732,15 +961,16 @@ ncmpi_inq_varndims(int ncid, int varid, int *ndimsp)
 }
 
 /*----< ncmpi_inq_vardimid() >-----------------------------------------------*/
+/* This is an independent subroutine */
 int
 ncmpi_inq_vardimid(int ncid, int varid, int *dimids)
 {
-    int status;
-    NC *ncp;
-    NC_var *varp;
+    int err;
+    NC *ncp=NULL;
+    NC_var *varp=NULL;
 
-    status = ncmpii_NC_check_id(ncid, &ncp);
-    if (status != NC_NOERR) return status;
+    err = ncmpii_NC_check_id(ncid, &ncp);
+    if (err != NC_NOERR) DEBUG_RETURN_ERROR(err)
 
     /* using NC_GLOBAL in varid is illegal for this API. See
      * http://www.unidata.ucar.edu/mailing_lists/archives/netcdfgroup/2015/msg00196.html
@@ -764,20 +994,24 @@ ncmpi_inq_vardimid(int ncid, int varid, int *dimids)
 
 
 /*----< ncmpi_inq_varnatts() >------------------------------------------------*/
+/* This is an independent subroutine */
 int
 ncmpi_inq_varnatts(int  ncid,
                    int  varid,
                    int *nattsp)
 {
-    int status;
-    NC *ncp;
-    NC_var *varp;
+    int err;
+    NC *ncp=NULL;
+    NC_var *varp=NULL;
 
-    if (varid == NC_GLOBAL)
-        return ncmpi_inq_natts(ncid, nattsp);
+    err = ncmpii_NC_check_id(ncid, &ncp);
+    if (err != NC_NOERR) DEBUG_RETURN_ERROR(err)
 
-    status = ncmpii_NC_check_id(ncid, &ncp);
-    if (status != NC_NOERR) return status;
+    if (varid == NC_GLOBAL) {
+        if (nattsp != NULL)
+            *nattsp = (int) ncp->attrs.ndefined;
+        return NC_NOERR;
+    }
 
     varp = elem_NC_vararray(&ncp->vars, varid);
     if (varp == NULL) DEBUG_RETURN_ERROR(NC_ENOTVAR)
@@ -789,104 +1023,159 @@ ncmpi_inq_varnatts(int  ncid,
 }
 
 /*----< ncmpi_rename_var() >--------------------------------------------------*/
-/* This API is collective if called in data mode */
+/* This API is collective.
+ * If the new name is longer than the old name, the netCDF file must be in
+ * define mode. Otherwise, it can be called in either define or data mode.
+ */
 int
 ncmpi_rename_var(int         ncid,
                  int         varid,
                  const char *newname)
 {
-    int file_ver, status=NC_NOERR, other, err, mpireturn;
-    NC *ncp;
-    NC_var *varp;
+    int err;
+    char *nnewname=NULL; /* normalized name */
+    NC *ncp=NULL;
+    NC_var *varp=NULL;
+    NC_string *newStr=NULL;
 
-    status = ncmpii_NC_check_id(ncid, &ncp);
-    if (status != NC_NOERR) return status;
+    /* check whether ncid is valid */
+    err = ncmpii_NC_check_id(ncid, &ncp);
+    if (err != NC_NOERR) DEBUG_RETURN_ERROR(err)
 
-    if (NC_readonly(ncp)) DEBUG_RETURN_ERROR(NC_EPERM)
-
-    /* check if variable ID is valid*/
-    status = ncmpii_NC_lookupvar(ncp, varid, &varp);
-    if (status != NC_NOERR) return status;
-
-    file_ver = 1;
-    if (fIsSet(ncp->flags, NC_64BIT_OFFSET))
-        file_ver = 2;
-    else if (fIsSet(ncp->flags, NC_64BIT_DATA))
-        file_ver = 5;
-
-    status = ncmpii_NC_check_name(newname, file_ver);
-    if (status != NC_NOERR) return status;
-
-    /* check for name in use */
-    other = ncmpii_NC_findvar(&ncp->vars, newname, &varp);
-    if (other != -1)
-        DEBUG_RETURN_ERROR(NC_ENAMEINUSE)
-
-    /* if called in define mode, just update to the NC object */
-    if (NC_indef(ncp)) {
-        NC_string *newStr = ncmpii_new_NC_string(strlen(newname), newname);
-        if (newStr == NULL) DEBUG_RETURN_ERROR(NC_ENOMEM)
-
-        ncmpii_free_NC_string(varp->name);
-        varp->name = newStr;
-        return NC_NOERR;
+    /* check whether file's write permission */
+    if (NC_readonly(ncp)) {
+        DEBUG_ASSIGN_ERROR(err, NC_EPERM)
+        goto err_check;
     }
-    /* else, not in define mode.
-     * if called in data mode (collective or independent), this function must
-     * be called collectively, i.e. all processes must participate.
-     */
 
+    /* check whether variable ID is valid */
+    err = ncmpii_NC_lookupvar(ncp, varid, &varp);
+    if (err != NC_NOERR) {
+        DEBUG_TRACE_ERROR
+        goto err_check;
+    }
+
+    if (newname == NULL || *newname == 0 || strlen(newname) > NC_MAX_NAME) {
+        DEBUG_ASSIGN_ERROR(err, NC_EBADNAME)
+        goto err_check;
+    }
+
+    /* check whether new name is legal */
+    err = ncmpii_NC_check_name(newname, ncp->format);
+    if (err != NC_NOERR) {
+        DEBUG_TRACE_ERROR
+        goto err_check;
+    }
+
+    /* create a normalized character string */ 
+    nnewname = (char *)ncmpii_utf8proc_NFC((const unsigned char *)newname);
+    if (nnewname == NULL) {
+        DEBUG_ASSIGN_ERROR(err, NC_ENOMEM)
+        goto err_check;
+    }
+
+    /* check whether new name is already in use, for this API (rename) the
+     * name should NOT already exist */
+    err = ncmpii_NC_findvar(&ncp->vars, nnewname, NULL);
+    if (err != NC_ENOTVAR) {
+        DEBUG_ASSIGN_ERROR(err, NC_ENAMEINUSE)
+        goto err_check;
+    }
+
+    if (! NC_indef(ncp) && /* when file is in data mode */
+        varp->name->nchars < (MPI_Offset)strlen(nnewname)) {
+        /* must in define mode when newname is longer */
+        DEBUG_ASSIGN_ERROR(err, NC_ENOTINDEFINE)
+        goto err_check;
+    }
+
+    newStr = ncmpii_new_NC_string(strlen(nnewname), nnewname);
+    if (newStr == NULL) {
+        DEBUG_ASSIGN_ERROR(err, NC_ENOMEM)
+        goto err_check;
+    }
+
+#ifndef SEARCH_NAME_LINEARLY
+    /* update var name lookup table */
+    err = ncmpii_update_name_lookup_table(ncp->vars.nameT, varid,
+          ncp->vars.value[varid]->name->cp, nnewname);
+    if (err != NC_NOERR) {
+        DEBUG_TRACE_ERROR
+        goto err_check;
+    }
+#endif
+
+err_check:
+    if (nnewname != NULL) free(nnewname);
     if (ncp->safe_mode) {
-        int nchars = (int)strlen(newname);
-        TRACE_COMM(MPI_Bcast)(&nchars, 1, MPI_INT, 0, ncp->nciop->comm);
-        if (mpireturn != MPI_SUCCESS)
-            return ncmpii_handle_error(mpireturn, "MPI_Bcast"); 
+        int root_varid, status, mpireturn;
+        char root_name[NC_MAX_NAME];
 
-        if (nchars != (int) strlen(newname)) {
-            /* newname's length is inconsistent with root's */
-            printf("Warning: variable name(%s) used in %s() is inconsistent\n",
-                   newname, __func__);
-            if (status == NC_NOERR) DEBUG_ASSIGN_ERROR(status, NC_EMULTIDEFINE_VAR_NAME)
-        }
+        /* check if newname is consistent among all processes */
+        if (newname == NULL || *newname == 0)
+            root_name[0] = 0;
+        else
+            strncpy(root_name, newname, NC_MAX_NAME);
+        TRACE_COMM(MPI_Bcast)(root_name, NC_MAX_NAME, MPI_CHAR, 0, ncp->nciop->comm);
+        if (mpireturn != MPI_SUCCESS)
+            return ncmpii_handle_error(mpireturn, "MPI_Bcast");
+        if (err == NC_NOERR && strcmp(root_name, newname))
+            DEBUG_ASSIGN_ERROR(err, NC_EMULTIDEFINE_VAR_NAME)
+
+        /* check if varid is consistent across all processes */
+        root_varid = varid;
+        TRACE_COMM(MPI_Bcast)(&root_varid, 1, MPI_INT, 0, ncp->nciop->comm);
+        if (mpireturn != MPI_SUCCESS)
+            return ncmpii_handle_error(mpireturn, "MPI_Bcast");
+        if (err == NC_NOERR && root_varid != varid)
+            DEBUG_ASSIGN_ERROR(err, NC_EMULTIDEFINE_FNC_ARGS)
+
+        /* find min error code across processes */
+        TRACE_COMM(MPI_Allreduce)(&err, &status, 1, MPI_INT, MPI_MIN, ncp->nciop->comm);
+        if (mpireturn != MPI_SUCCESS)
+            return ncmpii_handle_error(mpireturn, "MPI_Allreduce");
+        if (err == NC_NOERR) err = status;
     }
 
-    /* ncmpii_set_NC_string() will check for strlen(newname) > nchars error */
-    err = ncmpii_set_NC_string(varp->name, newname);
-    if (status == NC_NOERR) status = err;
+    if (err != NC_NOERR) {
+        if (newStr != NULL) ncmpii_free_NC_string(newStr);
+        return err;
+    }
 
-    /* PnetCDF expects all processes use the same name, However, when names
-     * are not the same, only root's value is significant. Broadcast the
-     * new name at root to overwrite new names at other processes.
-     * (This API is collective if called in data mode)
-     */
-    TRACE_COMM(MPI_Bcast)(varp->name->cp, (int)varp->name->nchars, MPI_CHAR, 0,
-                          ncp->nciop->comm);
+    assert(varp != NULL);
 
-    /* Let root write the entire header to the file. Note that we cannot just
-     * update the variable name in its space occupied in the file header,
-     * because if the file space occupied by the name shrinks, all the metadata
-     * following it must be moved ahead.
-     */
-    err = ncmpii_write_header(ncp);
-    if (status == NC_NOERR) status = err;
+    /* replace the old name with new name */
+    ncmpii_free_NC_string(varp->name);
+    varp->name = newStr;
 
-    return status;
+    if (! NC_indef(ncp)) { /* when file is in data mode */
+        /* Let root write the entire header to the file. Note that we cannot
+         * just update the variable name in its space occupied in the file
+         * header, because if the file space occupied by the name shrinks, all
+         * the metadata following it must be moved ahead.
+         */
+        err = ncmpii_write_header(ncp);
+        if (err != NC_NOERR) DEBUG_RETURN_ERROR(err)
+    }
+
+    return err;
 }
 
 /* some utility functions for debugging purpose */
 
 /*----< ncmpi_inq_varoffset() >-----------------------------------------------*/
+/* This is an independent subroutine */
 int
 ncmpi_inq_varoffset(int         ncid,
                     int         varid,
                     MPI_Offset *offset)
 {
-    int     status;
-    NC     *ncp;
-    NC_var *varp;
+    int     err;
+    NC     *ncp=NULL;
+    NC_var *varp=NULL;
 
-    status = ncmpii_NC_check_id(ncid, &ncp);
-    if (status != NC_NOERR) return status;
+    err = ncmpii_NC_check_id(ncid, &ncp);
+    if (err != NC_NOERR) DEBUG_RETURN_ERROR(err)
 
     /* using NC_GLOBAL in varid is illegal for this API. See
      * http://www.unidata.ucar.edu/mailing_lists/archives/netcdfgroup/2015/msg00196.html
@@ -905,13 +1194,15 @@ ncmpi_inq_varoffset(int         ncid,
 #ifdef __DEBUG
 
 /*----< ncmpi_print_all_var_offsets() >---------------------------------------*/
+/* This is an independent subroutine */
 int
 ncmpi_print_all_var_offsets(int ncid) {
-    int i;
-    NC_var **vpp;
-    NC *ncp;
+    int i, err;
+    NC_var **vpp=NULL;
+    NC *ncp=NULL;
 
-    ncmpii_NC_check_id(ncid, &ncp);
+    err = ncmpii_NC_check_id(ncid, &ncp);
+    if (err != NC_NOERR) DEBUG_RETURN_ERROR(err)
 
     if (ncp->begin_var%1048576)
         printf("%s header size (ncp->begin_var)=%lld MB + %lld\n",
