@@ -2,7 +2,7 @@
  *  Copyright (C) 2003, Northwestern University and Argonne National Laboratory
  *  See COPYRIGHT notice in top-level directory.
  */
-/* $Id: ncmpivalid.c 2335 2016-03-03 14:55:12Z wkliao $ */
+/* $Id: ncmpivalid.c 2726 2016-12-18 06:48:19Z wkliao $ */
 
 #if HAVE_CONFIG_H
 # include <ncconfig.h>
@@ -44,9 +44,6 @@ static int val_get_NC_var(bufferinfo *gbp, NC_var **varpp);
 static int val_get_NC_vararray(bufferinfo *gbp, NC_vararray *ncap);
 static int val_get_NC(NC *ncp);
 
-static int val_fetch(bufferinfo *gbp, MPI_Offset fsize);
-static int val_check_buffer(bufferinfo *gbp, MPI_Offset nextread);
-
 
 #define ABORT {printf("Abort: at line=%d func=%s\n", __LINE__,__func__); fflush(stdout); MPI_Abort(MPI_COMM_WORLD, -1);}
 
@@ -56,7 +53,7 @@ static int val_check_buffer(bufferinfo *gbp, MPI_Offset nextread);
  * Fetch the next header chunk.
  */
 static int
-val_fetch(bufferinfo *gbp, MPI_Offset fsize) {
+val_fetch(bufferinfo *gbp) {
   ssize_t nn = 0;
   MPI_Offset slack;        /* any leftover data in the buffer */
   MPI_Aint pos_addr, base_addr;
@@ -71,8 +68,8 @@ val_fetch(bufferinfo *gbp, MPI_Offset fsize) {
     MPI_Address(gbp->base, &base_addr);
 #endif
   slack = gbp->size - (pos_addr - base_addr);
-  /* . if gbp->pos and gbp->base are the same, there is no leftover buffer data
-   *   to worry about.  
+  /* if gbp->pos and gbp->base are the same, there is no leftover buffer data
+   * to worry about.  
    * In the other extreme, where gbp->size == (gbp->pos - gbp->base), then all
    * data in the buffer has been consumed */
   if (slack == gbp->size) slack = 0;
@@ -114,7 +111,7 @@ val_check_buffer(bufferinfo *gbp,
     if (pos_addr + nextread <= base_addr + gbp->size)
         return NC_NOERR;
 
-    return val_fetch(gbp, MIN(gbp->size, nextread));
+    return val_fetch(gbp);
 } 
 
 static int
@@ -193,7 +190,7 @@ val_get_NC_string(bufferinfo *gbp, NC_string **ncstrpp) {
       cpos += strcount; 
       bufremain -= strcount;
     } else {
-      status = val_fetch(gbp, MIN(gbp->size, X_SIZEOF_CHAR * nchars));
+      status = val_fetch(gbp);
       if(status != NC_NOERR) {
 	printf("fetching the name string of ");
         ncmpii_free_NC_string(ncstrp);
@@ -374,7 +371,7 @@ val_get_NC_attrV(bufferinfo *gbp, NC_attr *attrp) {
       value = (void *)((char *)value + attcount);
       bufremain -= attcount;
     } else {
-      status = val_fetch(gbp, MIN(gbp->size, esz * nvalues));
+      status = val_fetch(gbp);
       if(status != NC_NOERR) {
 	printf("fetching the values of ");
         return status;
@@ -502,9 +499,8 @@ val_get_NC_attrarray(bufferinfo *gbp, NC_attrarray *ncap){
 static int
 val_get_NC_var(bufferinfo *gbp, NC_var **varpp) {
   NC_string *strp;
-  int status;
+  int dim, status;
   MPI_Offset ndims, *tmp_dim;
-  size_t dim;
   NC_var *varp;
 
   status = val_get_NC_string(gbp, &strp);
@@ -527,7 +523,7 @@ val_get_NC_var(bufferinfo *gbp, NC_var **varpp) {
   for (dim = 0; dim < ndims; dim++ ) {
     status = val_check_buffer(gbp, (gbp->version == 5 ? 8 : 4));
     if(status != NC_NOERR) {
-      printf("the dimid[%d] is expected for \"%s\" - ", (int)dim, strp->cp);
+      printf("the dimid[%d] is expected for \"%s\" - ", dim, strp->cp);
       ncmpii_free_NC_var(varp);
       return status;
     }
@@ -659,7 +655,7 @@ static int
 val_get_NC(NC *ncp) {
     int status;
     bufferinfo getbuf;
-    schar magic[sizeof(ncmagic)];
+    char magic[sizeof(ncmagic)];
     MPI_Offset nrecs = 0;
     MPI_Aint pos_addr, base_addr;
 
@@ -668,8 +664,6 @@ val_get_NC(NC *ncp) {
     /* Initialize the get buffer that stores the header read from the file */
     getbuf.nciop = ncp->nciop;
     getbuf.offset = 0;     /* read from start of the file */
-    getbuf.put_size = 0;   /* amount of writes so far in bytes */
-    getbuf.get_size = 0;   /* amount of reads  so far in bytes */
 
     /* CDF-5's minimum header size is 4 bytes more than CDF-1 and CDF-2's */
     getbuf.size = _RNDUP( MAX(MIN_NC_XSZ+4, ncp->chunk), X_ALIGN );
@@ -680,7 +674,7 @@ val_get_NC(NC *ncp) {
     getbuf.index = 0;
 
     /* Fetch the next header chunk. The chunk is 'gbp->size' bytes big */
-    status = val_fetch(&getbuf, sizeof(magic));
+    status = val_fetch(&getbuf);
     if (status != NC_NOERR) {
         printf("magic number (C D F \\001) is expected!\n");
         return status;
@@ -688,8 +682,8 @@ val_get_NC(NC *ncp) {
   
     /* First get the file format information, magic */
     memset(magic, 0, sizeof(magic));
-    status = ncmpix_getn_schar_schar((const void **)(&getbuf.pos),
-                                     sizeof(magic), magic);
+    status = ncmpix_getn_text((const void **)(&getbuf.pos), sizeof(magic),
+                              magic);
     getbuf.index += sizeof(magic);
 
     if (memcmp(magic, ncmagic, sizeof(ncmagic)-1) != 0) {
@@ -843,7 +837,13 @@ int main(int argc, char **argv) {
     }
 
     /* check data size */
-    fstat(ncp->nciop->fd, &ncfilestat);
+    if (-1 == fstat(ncp->nciop->fd, &ncfilestat)) {
+        printf("Error at line %d fstat (%s)\n",__LINE__,strerror(errno));
+        close(ncp->nciop->fd);
+        ncmpiio_free(ncp->nciop);
+        ncmpii_free_NC(ncp);
+        return 0;
+    }
     if ( ncp->begin_rec + ncp->recsize * ncp->numrecs < ncfilestat.st_size ) {
         printf("Error: \n\tData size is larger than defined!\n");
         close(ncp->nciop->fd);
